/* $Id: FCModeler.java,v 1.18 2002/07/31 08:31:59 adamtj Exp $
 *
 *
 */

package fcmodeler;

import javax.swing.JScrollPane;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JMenuBar;
import javax.swing.JToolBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JButton;
import javax.swing.KeyStroke;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import diva.graph.EdgeRenderer;
import diva.graph.NodeRenderer;
import diva.graph.GraphView;
import diva.graph.model.GraphModel;
import diva.graph.model.Node;
import diva.graph.model.Graph;
import diva.graph.GraphPane;
import diva.canvas.Figure;
import diva.canvas.selection.BasicSelectionRenderer;
import diva.canvas.connector.PerimeterTarget;
import diva.canvas.event.MouseFilter;

import fcmodeler.view.ScrollableJGraph;
import fcmodeler.view.NodeFigureRenderer;
import fcmodeler.view.BezierEdgeFigureRenderer;
import fcmodeler.actions.OpenGraphAction;
import fcmodeler.actions.SaveGraphAction;
import fcmodeler.actions.JGraphToJPEGAction;
//import fcmodeler.actions.JGraphToSVGAction;
import fcmodeler.actions.GraphToPostscriptAction;
import fcmodeler.actions.SimpleDotLayoutAction;
import fcmodeler.actions.GEMLayoutAction;
import fcmodeler.actions.AGEMLayoutAction;
import fcmodeler.actions.FastGEMLayoutAction;
import fcmodeler.actions.RankClusterDotLayoutAction;
import fcmodeler.actions.CreateSubgraphAction;
import fcmodeler.actions.FindCyclesAction;
import fcmodeler.actions.SCCAction;
import fcmodeler.actions.FindNodeAction;
import fcmodeler.actions.OpenPropertyViewerAction;
import fcmodeler.actions.OpenGraphDatabaseAction;
//import fcmodeler.actions.OpenAttributeFileAction;
import fcmodeler.actions.OpenCycleFileAction;
import fcmodeler.actions.ZoomInAction;
import fcmodeler.actions.ZoomOutAction;
//import fcmodeler.actions.TestViewportAction;
//import fcmodeler.actions.OpenAttributeEditorAction;
import fcmodeler.actions.OpenMappingEditorAction;
import fcmodeler.actions.OpenAnimationFileAction;
//import fcmodeler.actions.SaveAttributeFileAction;
//import fcmodeler.actions.OpenLayoutFileAction;
//import fcmodeler.actions.LayoutGraphFileAction;
//import fcmodeler.actions.TestRankClusterDotWriter;
import fcmodeler.actions.SOMAction;
import fcmodeler.actions.ExitAction;
import fcmodeler.layout.InitialLayout;
//import fcmodeler.view.update.ViewUpdater;
import fcmodeler.view.BasicConnectorHighlighter;
import fcmodeler.interaction.CustomGraphController;
import fcmodeler.util.ObservableSet;
import fcmodeler.mapping.NodeMappingObserver;
import fcmodeler.mapping.EdgeMappingObserver;

import fcmodeler.interaction.GeneralConnector.GeneralConnectorRenderer;
import fcmodeler.interaction.BezierConnector.BezierShapeFactory;

//debug
/*import diva.canvas.selection.*;
import diva.canvas.manipulator.*;
import diva.canvas.*;
import diva.canvas.connector.*;*/

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.18 $
 * @since JDK1.3
 */

public class FCModeler extends Object {
	private ScrollableJGraph _scrollableJGraph = null;
	private JScrollPane _scrollPane = null;
	private List _actions = null;
	private String _pathToDot = null;
	private String _pathToGraphs = null;
	//private NodeVisualState _nodeVisualState = null;
	//private EdgeVisualState _edgeVisualState = null;
	private ObservableSet _nodeMappings = null;
	private ObservableSet _edgeMappings = null;
	private NodeMappingObserver _nodeObserver = null;
	private EdgeMappingObserver _edgeObserver = null;
	private NodeRenderer _nodeRenderer = null;
	private EdgeRenderer _edgeRenderer = null;
	private Component _parentComponent = null;
	private FCModelerContainer _container = null;
	private OpenGraphAction _openGraphAction = null;
	private OpenGraphDatabaseAction _openGraphDatabaseAction = null;
	//private OpenAttributeFileAction _openAttributeFileAction = null;
	private OpenCycleFileAction _openCycleFileAction = null;
	private SaveGraphAction _saveGraphAction = null;
	private JGraphToJPEGAction _jgraphToJPEGAction = null;
	//private JGraphToSVGAction _jgraphToSVGAction = null;
	private GraphToPostscriptAction _graphToPostscriptAction = null;
	private SimpleDotLayoutAction _simpleDotLayoutAction = null;
	private GEMLayoutAction _gemLayoutAction = null;
	private AGEMLayoutAction _agemLayoutAction = null;
	private FastGEMLayoutAction _fastgemLayoutAction = null;
	private RankClusterDotLayoutAction _rankClusterDotLayoutAction = null;
	private CreateSubgraphAction _createSubgraphAction = null;
	private FindCyclesAction _findCyclesAction = null;
	private SCCAction _sccAction = null;
	private FindNodeAction _findNodeAction = null;
	private OpenPropertyViewerAction _openPropertyViewerAction = null;
	private ZoomInAction _zoomInAction = null;
	private ZoomOutAction _zoomOutAction = null;
	//private OpenAttributeEditorAction _openAttributeEditorAction = null;
	private OpenMappingEditorAction _openMappingEditorAction = null;
	private OpenAnimationFileAction _openAnimationFileAction = null;
	//private SaveAttributeFileAction _saveAttributeFileAction = null;
	//private OpenLayoutFileAction _openLayoutFileAction = null;
	//private LayoutGraphFileAction _layoutGraphFileAction = null;
	//private TestRankClusterDotWriter _testRankClusterDotWriter = null;
	private SOMAction _somAction = null;
	
	/** Creates new FCModeler */
	public FCModeler() {
		createActions();
		
		//_edgeRenderer = new BezierEdgeFigureRenderer();
		_edgeRenderer = new GeneralConnectorRenderer(new BezierShapeFactory());
		
		//_edgeVisualState = new EdgeVisualState();
		_edgeMappings = new ObservableSet(new HashSet());
		
		//_scrollPane = new JScrollPane();
		_scrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		_scrollPane.setPreferredSize(new Dimension(500, 500));
		
		_nodeRenderer = new NodeFigureRenderer();
		
		//_nodeVisualState = new NodeVisualState();
		_nodeMappings = new ObservableSet(new HashSet());
		
		_scrollableJGraph = new ScrollableJGraph(this);
		_scrollableJGraph.setGraphPane(new GraphPane(new GraphModel(), new CustomGraphController()));
		_scrollableJGraph.setScrollPane(_scrollPane);
		_scrollPane.setViewportView(_scrollableJGraph);
	}
	
	public FCModeler(FCModelerContainer container, Component parentComponent) {
		this();
		setContainer(container);
		setParentComponent(parentComponent);
	}
	
	public FCModeler(FCModeler fcmodeler) {
		createActions();
		//_scrollPane = new JScrollPane();
		_scrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		_scrollPane.setPreferredSize(new Dimension(500, 500));
		_scrollableJGraph = new ScrollableJGraph(this);
		_scrollableJGraph.setGraphPane(new GraphPane(new GraphModel(), new CustomGraphController()));
		_scrollableJGraph.setScrollPane(_scrollPane);
		_scrollPane.setViewportView(_scrollableJGraph);
		_edgeMappings = new ObservableSet(new HashSet());
		_nodeMappings = new ObservableSet(new HashSet());
		setAllFrom(fcmodeler);
	}
	
	public FCModeler(FCModeler fcmodeler, FCModelerContainer container, Component parentComponent) {
		this(fcmodeler);
		setContainer(container);
		setParentComponent(parentComponent);
	}
	
	public void addAction(Action action) {
		_actions.add(action);
	}
	
	private void createActions() {
		_openGraphAction = new OpenGraphAction("open graph file", 
			new ImageIcon("icons/OpenDoc.png"));
		_openGraphAction.putValue(Action.SHORT_DESCRIPTION, "open graph file");
		
		_openGraphDatabaseAction = new OpenGraphDatabaseAction("open database", 
			new ImageIcon("icons/OpenArrow.png"));
		_openGraphDatabaseAction.putValue(Action.SHORT_DESCRIPTION, "open graph from database");
		
		//_openAttributeFileAction = new OpenAttributeFileAction("open attributes file");
		
		_openCycleFileAction = new OpenCycleFileAction("open cycles file");
		
		_openAnimationFileAction = new OpenAnimationFileAction("open animation file");
		
		_saveGraphAction = new SaveGraphAction("save graph", 
			new ImageIcon("icons/Save.png"));
		_saveGraphAction.putValue(Action.SHORT_DESCRIPTION, "save graph");
		
		//_saveAttributeFileAction = new SaveAttributeFileAction("save attributes");
		
		_jgraphToJPEGAction = new JGraphToJPEGAction("save as jpg");
		
		//_jgraphToSVGAction = new JGraphToSVGAction("save as svg");
		
		_graphToPostscriptAction = new GraphToPostscriptAction("save as ps");
		
		_simpleDotLayoutAction = new SimpleDotLayoutAction("dot layout", 
			new ImageIcon("icons/FlowGraph.png"));
		_simpleDotLayoutAction.putValue(Action.SHORT_DESCRIPTION, "dot layout");
		
		_gemLayoutAction = new GEMLayoutAction("GEM layout",
			new ImageIcon("icons/FlowGraph.png"));
		_gemLayoutAction.putValue(Action.SHORT_DESCRIPTION, "GEM layout");
		
		_agemLayoutAction = new AGEMLayoutAction("AGEM layout",
			new ImageIcon("icons/FlowGraph.png"));
		_agemLayoutAction.putValue(Action.SHORT_DESCRIPTION, "AGEM layout");
		
		_fastgemLayoutAction = new FastGEMLayoutAction("FastGEM layout",
			new ImageIcon("icons/FlowGraph.png"));
		_fastgemLayoutAction.putValue(Action.SHORT_DESCRIPTION, "FastGEM layout");
		
		_rankClusterDotLayoutAction = new RankClusterDotLayoutAction("dot rank-cluster layout");
		
		_createSubgraphAction = new CreateSubgraphAction("create subgraph", 
			new ImageIcon("icons/DocumentDiagram.gif"));
		_createSubgraphAction.putValue(Action.SHORT_DESCRIPTION, "create subgraph");
		
		_findCyclesAction = new FindCyclesAction("find cycles", 
			new ImageIcon("icons/RotCCUp.png"));
		_findCyclesAction.putValue(Action.SHORT_DESCRIPTION, "find cycles");
		
		_sccAction = new SCCAction("find strongly connected components");
		
		_findNodeAction = new FindNodeAction("find node");
		
		/*_openAttributeEditorAction = new OpenAttributeEditorAction("attribute editor", 
			new ImageIcon("icons/Palette.png"));
		_openAttributeEditorAction.putValue(Action.SHORT_DESCRIPTION, "attribute editor");*/
		_openMappingEditorAction = new OpenMappingEditorAction("mapping editor", 
			new ImageIcon("icons/Palette.png"));
		_openMappingEditorAction.putValue(Action.SHORT_DESCRIPTION, "mapping editor");
		
		_openPropertyViewerAction = new OpenPropertyViewerAction("property viewer");
		
		_zoomInAction = new ZoomInAction("zoom in", 
			new ImageIcon("icons/MagnifyPlus.png"));
		_zoomInAction.putValue(Action.SHORT_DESCRIPTION, "zoom in");
		
		_zoomOutAction = new ZoomOutAction("zoom out", 
			new ImageIcon("icons/MagnifyMinus.png"));
		_zoomOutAction.putValue(Action.SHORT_DESCRIPTION, "zoom out");
		
		//_openLayoutFileAction = new OpenLayoutFileAction("open layout file");
		
		//_layoutGraphFileAction = new LayoutGraphFileAction("layout graph with file");
		
		//_testRankClusterDotWriter = new TestRankClusterDotWriter("write dot file");
		
		_somAction = new SOMAction("run som on cycles");
		
		_actions = new ArrayList();
		
		_openGraphAction.setEnabled(true);
		_openGraphDatabaseAction.setEnabled(true);
		//_openAttributeFileAction.setEnabled(false);
		_openCycleFileAction.setEnabled(false);
		_saveGraphAction.setEnabled(false);
		_jgraphToJPEGAction.setEnabled(false);
		//_jgraphToSVGAction.setEnabled(false);
		_graphToPostscriptAction.setEnabled(false);
		_simpleDotLayoutAction.setEnabled(false);
		_gemLayoutAction.setEnabled(false);
		_agemLayoutAction.setEnabled(false);
		_fastgemLayoutAction.setEnabled(false);
		_rankClusterDotLayoutAction.setEnabled(false);
		_createSubgraphAction.setEnabled(false);
		_findCyclesAction.setEnabled(false);
		_sccAction.setEnabled(false);
		_findNodeAction.setEnabled(false);
		_openPropertyViewerAction.setEnabled(false);
		_zoomInAction.setEnabled(false);
		_zoomOutAction.setEnabled(false);
		//_openAttributeEditorAction.setEnabled(false);
		_openMappingEditorAction.setEnabled(false);
		_openAnimationFileAction.setEnabled(false);
		//_saveAttributeFileAction.setEnabled(false);
		//_openLayoutFileAction.setEnabled(false);
		//_layoutGraphFileAction.setEnabled(false);
		//_testRankClusterDotWriter.setEnabled(false);
		_somAction.setEnabled(false);
		
		_actions.add(_openGraphAction);
		_actions.add(_openGraphDatabaseAction);
		//_actions.add(_openAttributeFileAction);
		_actions.add(_openAnimationFileAction);
		_actions.add(_saveGraphAction);
		//_actions.add(_saveAttributeFileAction);
		_actions.add(_jgraphToJPEGAction);
		//_actions.add(_jgraphToSVGAction);
		_actions.add(_graphToPostscriptAction);
		_actions.add(_simpleDotLayoutAction);
		_actions.add(_gemLayoutAction);
		_actions.add(_agemLayoutAction);
		_actions.add(_fastgemLayoutAction);
		_actions.add(_createSubgraphAction);
		_actions.add(_findCyclesAction);
		_actions.add(_openCycleFileAction);
		_actions.add(_sccAction);
		_actions.add(_findNodeAction);
		//_actions.add(_openAttributeEditorAction);
		_actions.add(_openMappingEditorAction);
		_actions.add(_openPropertyViewerAction);
		_actions.add(_zoomInAction);
		_actions.add(_zoomOutAction);
		//_actions.add(_openLayoutFileAction);
		//_actions.add(_layoutGraphFileAction);
		_actions.add(_somAction);
	}
	
	public Iterator getActions() {
		Iterator actions = _actions.iterator();
		
		return actions;
	}
	
	public FCModelerContainer getContainer() {
		return _container;
	}
	
	public EdgeMappingObserver getEdgeObserver() {
		return _edgeObserver;
	}
	
	public EdgeRenderer getEdgeRenderer() {
		return _edgeRenderer;
	}
	
	/*public EdgeVisualState getEdgeVisualState() {
		return _edgeVisualState;
	}*/
	public ObservableSet getEdgeMappings() {
		return _edgeMappings;
	}
	
	public Graph getGraph() {
		return _scrollableJGraph.getGraphModel().getGraph();
	}
	
	public GraphView getGraphView() {
		return _scrollableJGraph.getGraphPane().getGraphView();
	}
	
	public JScrollPane getJScrollPane() {
		return _scrollPane;
	}
	
	public JMenuBar getMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenu menu = null;
		JMenuItem menuItem = null;
			
		//create file menu
		menu = new JMenu("file");
		menuItem = new JMenuItem(_openGraphAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_openGraphDatabaseAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		//menuItem = new JMenuItem(_openAttributeFileAction);
		//menuItem.setIcon(null);
		//menu.add(menuItem);
		menuItem = new JMenuItem(_openAnimationFileAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menu.addSeparator();
		menuItem = new JMenuItem(_saveGraphAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		//menuItem = new JMenuItem(_saveAttributeFileAction);
		//menuItem.setIcon(null);
		//menu.add(menuItem);
		menuItem = new JMenuItem(_jgraphToJPEGAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		//menuItem = new JMenuItem(_jgraphToSVGAction);
		//menuItem.setIcon(null);
		//menu.add(menuItem);
		menuItem = new JMenuItem(_graphToPostscriptAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		//menuItem = new JMenuItem(_testRankClusterDotWriter);
		//menuItem.setIcon(null);
		//menu.add(menuItem);
		menu.addSeparator();
		menuItem = new JMenuItem(new ExitAction("exit", null));
		menuItem.setMnemonic(KeyEvent.VK_X);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,ActionEvent.ALT_MASK));
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuBar.add(menu);
		
		//create the view menu
		menu = new JMenu("view");
		/*menuItem = new JMenuItem(_layoutGraphFileAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_openLayoutFileAction);
		menuItem.setIcon(null);
		menu.add(menuItem);*/
		menuItem = new JMenuItem(_zoomInAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_zoomOutAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		/*menuItem = new JMenuItem(_openAttributeEditorAction);
		menuItem.setIcon(null);
		menu.add(menuItem);*/
		menuItem = new JMenuItem(_openMappingEditorAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_findNodeAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuBar.add(menu);
		
		//create the layout menu
		menu = new JMenu("layout");
		menuItem = new JMenuItem(_simpleDotLayoutAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_rankClusterDotLayoutAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_gemLayoutAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_agemLayoutAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_fastgemLayoutAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuBar.add(menu);
		
		//create the graph menu
		menu = new JMenu("graph");
		menuItem = new JMenuItem(_createSubgraphAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_sccAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_findCyclesAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_openCycleFileAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuItem = new JMenuItem(_somAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuBar.add(menu);
		
		//create the properties menu
		menu = new JMenu("properties");
		menuItem = new JMenuItem(_openPropertyViewerAction);
		menuItem.setIcon(null);
		menu.add(menuItem);
		menuBar.add(menu);
		
		return menuBar;
	}
	
	public NodeMappingObserver getNodeObserver() {
		return _nodeObserver;
	}
	
	public NodeRenderer getNodeRenderer() {
		return _nodeRenderer;
	}
	
	/*public NodeVisualState getNodeVisualState() {
		return _nodeVisualState;
	}*/
	public ObservableSet getNodeMappings() {
		return _nodeMappings;
	}
	
	public Component getParentComponent() {
		return _parentComponent;
	}
	
	public String getPathToDot() {
		return _pathToDot;
	}
	
	public String getPathToGraphs() {
		return _pathToGraphs;
	}
	
	public ScrollableJGraph getScrollableJGraph() {
		return _scrollableJGraph;
	}
	
	public JToolBar getToolBar() {
		JToolBar toolBar = null;
		JButton button = null;
		
		//create the toolbar
		toolBar = new JToolBar();
		button = toolBar.add(_openGraphAction);
		button.setText("");
		button.setToolTipText((String) _openGraphAction.getValue(Action.SHORT_DESCRIPTION));
		button = toolBar.add(_openGraphDatabaseAction);
		button.setText("");
		button.setToolTipText((String) _openGraphDatabaseAction.getValue(Action.SHORT_DESCRIPTION));
		button = toolBar.add(_saveGraphAction);
		button.setText("");
		button.setToolTipText((String) _saveGraphAction.getValue(Action.SHORT_DESCRIPTION));
		toolBar.addSeparator();
		button = toolBar.add(_zoomInAction);
		button.setText("");
		button.setToolTipText((String) _zoomInAction.getValue(Action.SHORT_DESCRIPTION));
		button = toolBar.add(_zoomOutAction);
		button.setText("");
		button.setToolTipText((String) _zoomOutAction.getValue(Action.SHORT_DESCRIPTION));
		button = toolBar.add(_simpleDotLayoutAction);
		button.setText("");
		button.setToolTipText((String) _simpleDotLayoutAction.getValue(Action.SHORT_DESCRIPTION));
		/*button = toolBar.add(_openAttributeEditorAction);
		button.setText("");
		button.setToolTipText((String) _openAttributeEditorAction.getValue(Action.SHORT_DESCRIPTION));*/
		button = toolBar.add(_openMappingEditorAction);
		button.setText("");
		button.setToolTipText((String) _openMappingEditorAction.getValue(Action.SHORT_DESCRIPTION));
		toolBar.addSeparator();
		button = toolBar.add(_createSubgraphAction);
		button.setText("");
		button.setToolTipText((String) _createSubgraphAction.getValue(Action.SHORT_DESCRIPTION));
		button = toolBar.add(_findCyclesAction);
		button.setText("");
		button.setToolTipText((String) _findCyclesAction.getValue(Action.SHORT_DESCRIPTION));
		
		return toolBar;
	}
	
	public void removeAction(Action action) {
		int index = _actions.indexOf(action);
		removeAction(index);
	}
	
	public void removeAction(int index) {
		_actions.remove(index);
	}
	
	public void setAllFrom(FCModeler fcmodeler) {
		//_edgeVisualState = fcmodeler.getEdgeVisualState();
		//_nodeVisualState = fcmodeler.getNodeVisualState();
		_edgeMappings.clear();
		_edgeMappings.addAll(fcmodeler.getEdgeMappings());
		_nodeMappings.clear();
		_nodeMappings.addAll(fcmodeler.getNodeMappings());
		_pathToDot = fcmodeler.getPathToDot();
		_pathToGraphs = fcmodeler.getPathToGraphs();
		_nodeRenderer = fcmodeler.getNodeRenderer();
		_edgeRenderer = fcmodeler.getEdgeRenderer();
	}
	
	public void setContainer(FCModelerContainer container) {
		_container = container;
	}
	
	public void setEdgeRenderer(EdgeRenderer edgeRenderer) {
		_edgeRenderer = edgeRenderer;
	}
	
	/*public void setEdgeVisualState(EdgeVisualState edgeVisualState) {
		_edgeVisualState = edgeVisualState;
	}*/
	
	public void setGraphModel(GraphModel graphModel) {
		//clear all node & edge selections
		_scrollableJGraph.getGraphPane().getGraphView().getGraphController().
		getEdgeInteractor().getSelectionModel().clearSelection();
		_scrollableJGraph.getGraphPane().getGraphView().getGraphController().
		getNodeInteractor().getSelectionModel().clearSelection();
		
		//set the initial layout (the one that comes with diva sucks)
		_scrollableJGraph.getGraphPane().getGraphView().setIncrLayout(new InitialLayout(
		_scrollableJGraph.getGraphPane().getGraphView()));
		
		//set the node & edge renderers
		_scrollableJGraph.getGraphPane().getGraphView().setNodeRenderer(_nodeRenderer);
		_scrollableJGraph.getGraphPane().getGraphView().setEdgeRenderer(_edgeRenderer);
		
		//set the preferred size of the ScrollableJGraph
		_scrollableJGraph.setPreferredSize(_scrollPane.getPreferredSize());
		_scrollableJGraph.setSize(_scrollPane.getPreferredSize());
		
		//give the GraphPane the new GraphModel
		_scrollableJGraph.getGraphPane().setGraphModel(graphModel);
		
		if (_edgeObserver != null) { _edgeMappings.removeSetChangeListener(_edgeObserver); }
		_edgeObserver = new EdgeMappingObserver(getGraph(), getGraphView());
		_edgeMappings.addSetChangeListener(_edgeObserver);
		
		if (_nodeObserver != null) { _nodeMappings.removeSetChangeListener(_nodeObserver); }
		_nodeObserver = new NodeMappingObserver(getGraph(), getGraphView());
		_nodeMappings.addSetChangeListener(_nodeObserver);
		
		//have ViewUpdater check for any mappings in Node & EdgeVisualState
//FIX		//ViewUpdater.setModified(_nodeVisualState, _edgeVisualState);
		//ViewUpdater.updateView(this);
//FIX		//ViewUpdater.setUnmodified(_nodeVisualState, _edgeVisualState);
		
		//scrollbars?
		_scrollPane.revalidate();
		_scrollPane.repaint();
		
		//test FCMLayerListener
		_scrollableJGraph.getGraphPane().getBackgroundEventLayer().addLayerListener(
		new fcmodeler.event.PopupLayerListener(_scrollableJGraph));
		
		//enable all the Actions
		_openGraphAction.setEnabled(true);
		_openGraphDatabaseAction.setEnabled(true);
		//_openAttributeFileAction.setEnabled(true);
		_openCycleFileAction.setEnabled(true);
		_saveGraphAction.setEnabled(true);
		_jgraphToJPEGAction.setEnabled(true);
		//_jgraphToSVGAction.setEnabled(true);
		_graphToPostscriptAction.setEnabled(true);
		_simpleDotLayoutAction.setEnabled(true);
		_gemLayoutAction.setEnabled(true);
		_agemLayoutAction.setEnabled(true);
		_fastgemLayoutAction.setEnabled(true);
		_rankClusterDotLayoutAction.setEnabled(true);
		_createSubgraphAction.setEnabled(true);
		_findCyclesAction.setEnabled(true);
		_sccAction.setEnabled(true);
		_findNodeAction.setEnabled(true);
		_openPropertyViewerAction.setEnabled(true);
		_zoomInAction.setEnabled(true);
		_zoomOutAction.setEnabled(true);
		//_openAttributeEditorAction.setEnabled(true);
		_openMappingEditorAction.setEnabled(true);
		_openAnimationFileAction.setEnabled(true);
		//_saveAttributeFileAction.setEnabled(true);
		//_openLayoutFileAction.setEnabled(true);
		//_layoutGraphFileAction.setEnabled(true);
		//_testRankClusterDotWriter.setEnabled(true);
		_somAction.setEnabled(true);
		
		//DEBUG
		/*BasicSelectionRenderer bsr = (BasicSelectionRenderer) _scrollableJGraph.getGraphPane().getGraphController().
		getEdgeInteractor().getSelectionRenderer();
		ConnectorManipulator cm = (ConnectorManipulator) bsr.getDecorator();
		cm.setHandleInteractor(new fcmodeler.view.DebugConnectorInteractor(cm));*/
	}
	
	public void setJScrollPane(JScrollPane scrollPane) {
		_scrollPane = scrollPane;
	}
	
	public void setNodeRenderer(NodeRenderer nodeRenderer) {
		_nodeRenderer = nodeRenderer;
	}
	
	/*public void setNodeVisualState(NodeVisualState nodeVisualState) {
		_nodeVisualState = nodeVisualState;
	}*/
	
	public void setParentComponent(Component component) {
		_parentComponent = component;
	}
	
	public void setPathToDot(String pathToDot) {
		_pathToDot = pathToDot;
	}
	
	public void  setPathToGraphs(String pathToGraphs) {
		_pathToGraphs = pathToGraphs;
	}
	
	public void setScrollableJGraph(ScrollableJGraph scrollableJGraph) {
		_scrollableJGraph = scrollableJGraph;
	}
}

/*
 * $Log: FCModeler.java,v $
 * Revision 1.18  2002/07/31 08:31:59  adamtj
 * added variations on the GEM layout.
 *
 * Revision 1.17  2002/07/30 21:23:08  adamtj
 * removed dot layout from the View menu and moved rank-cluster layout from the View to the Layout menu
 *
 * Revision 1.16  2002/07/26 23:12:25  adamtj
 * This activates the GEM layout.  It also adds an exit command to the file menu.
 *
 * Revision 1.15  2002/06/27 06:08:35  adamtj
 * Activate BezierConnector edge type.  This turns on the bezier curve control point grab handles.
 *
 * Revision 1.14  2002/05/21 20:19:48  zcox
 * more som work
 *
 * Revision 1.13  2002/05/16 21:09:54  zcox
 * implemented cycle filtering
 *
 * Revision 1.12  2002/04/04 18:02:21  zcox
 * fixed subgraph problem
 *
 * Revision 1.11  2002/02/15 20:25:32  zcox
 * big time update
 *
 * Revision 1.10  2002/01/15 22:05:52  zcox
 * lots of changes
 *
 *
 *
 */
