/* $Id: FCModelerFrame.java,v 1.2 2002/02/15 20:25:32 zcox Exp $
 *
 *
 * Created on September 21, 2000, 10:45 PM
 */

package fcmodeler;

import javax.swing.JButton;
import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import java.awt.Cursor;
import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.util.Iterator;
import java.util.List;
import java.io.BufferedReader;
import java.io.FileReader;

import diva.graph.model.GraphModel;

/** The main application class.  Basically sets up the user interface, and provides
 * handler methods for GUI events.  These handler methods typically just delegate the
 * work to other specific classes.
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class FCModelerFrame extends JFrame implements FCModelerApp, FCModelerContainer {
	//the following variables were defined by Forte for Java's GUI Editor
	//DO NOT MODIFY THEM IN ANY WAY EXCEPT USING THE GUI EDITOR!!!!!
	// Variables declaration - do not modify//GEN-BEGIN:variables
	// End of variables declaration//GEN-END:variables
	//the rest of the member variables were hand-coded
	//feel free to modify them
	private FCModeler _fcmodeler = null;
	
	public FCModelerFrame(FCModeler fcmodeler) {
		_fcmodeler = fcmodeler;
		_fcmodeler.setParentComponent(this);
		_fcmodeler.setContainer(this);
		
		//setup the user interface & window
		initComponents();
		getContentPane().add(_fcmodeler.getJScrollPane(), BorderLayout.CENTER);
		setJMenuBar(_fcmodeler.getMenuBar());
		getContentPane().add(_fcmodeler.getToolBar(), BorderLayout.NORTH);
		setLocation(100, 100);
		setSize(800, 800);
		setTitle("empty");
		setIconImage(Toolkit.getDefaultToolkit().getImage("icons/FlowGraphWhite.gif"));
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		pack();
	}
	
	/** Creates new FCModeler window.
	 * @param configFile a String that represents the path to the FCModelerConfig.txt file.
	 */
	public FCModelerFrame(String configFile) {
		_fcmodeler = new FCModeler(this, this);
		
		//read the configuration file
		readConfigurationFile(configFile);
		
		//setup the user interface & window
		initComponents();
		getContentPane().add(_fcmodeler.getJScrollPane(), BorderLayout.CENTER);
		setJMenuBar(_fcmodeler.getMenuBar());
		getContentPane().add(_fcmodeler.getToolBar(), BorderLayout.NORTH);
		setLocation(100, 100);
		setSize(800, 800);
		setTitle("empty");
		setIconImage(Toolkit.getDefaultToolkit().getImage("icons/FlowGraphWhite.gif"));
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		pack();
	}
	
	public FCModeler getFCModeler() {
		return _fcmodeler;
	}
	
	private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
		//exit the application only if this was the main window
		if (getDefaultCloseOperation() == HIDE_ON_CLOSE) {
			System.exit(0);
		}
	}//GEN-LAST:event_exitForm
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the FormEditor.
	 */
	private void initComponents() {//GEN-BEGIN:initComponents
		
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		});
		
	}//GEN-END:initComponents

	/** Starts the FCModeler application.
	 * @param args the path to FCModelerConfig.txt if the default will not be used.
	 * If this parameter is not specified, the default configuration file
	 * will be used (FCModelerConfig.txt in the current working directory).
	 */
	public static void main (String args[]) {
		String configFile = "FCModelerConfig.txt";
		if (args.length > 0) {
			configFile = args[0];
		}
		FCModelerFrame frame = new FCModelerFrame(configFile);
		frame.setDefaultCloseOperation(HIDE_ON_CLOSE);
		frame.show();
	}
	
	private void readConfigurationFile(String configFile) {
		BufferedReader config = null;
		String line = null;
		
		//read in the configuration file
		try {
			config = new BufferedReader(new FileReader(configFile));
			while ((line = config.readLine()) != null) {
				if (line.startsWith("pathToDot")) {
					_fcmodeler.setPathToDot(line.substring(line.indexOf("=")+1));
				}
				else if (line.startsWith("pathToGraphs")) {
					_fcmodeler.setPathToGraphs(line.substring(line.indexOf("=")+1));
				}
			}
		} catch (java.io.FileNotFoundException e) {
			System.out.println("couldn't find " + configFile);
			System.exit(1);
		} catch (java.io.IOException e) {
			System.out.println("IOException: " + e.getMessage());
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public void setFCModeler(FCModeler fcmodeler) {
		_fcmodeler = fcmodeler;
	}
	
	public void setTitle(String title) {
		if (getDefaultCloseOperation() == HIDE_ON_CLOSE) {
			super.setTitle("FCModeler - " + title);
		}
		else {
			super.setTitle(title);
		}
	}
	
	public void showNewFCModeler(FCModeler fcmodeler, GraphModel graphModel) {
		showNewFCModeler(fcmodeler, graphModel, null);
	}
	
	public void showNewFCModeler(FCModeler fcmodeler, GraphModel graphModel, String title) {
		FCModeler newFCModeler = new FCModeler(fcmodeler);
		FCModelerFrame frame = new FCModelerFrame(newFCModeler);
		
		frame.setTitle(title);
		frame.show();
		newFCModeler.setGraphModel(graphModel);
	}
	
	public void useWaitCursor(boolean uwc) {
		if (uwc) {
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
		}
		else {
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
	}
}

/*
 * $Log: FCModelerFrame.java,v $
 * Revision 1.2  2002/02/15 20:25:32  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 22:06:30  zcox
 * initial commit of some application-level classes
 *
 * Revision 1.9  2001/05/31 22:56:18  zcox
 * added subgraph package
 *
 * Revision 1.8  2001/05/24 18:44:14  zcox
 * rearranged some methods into proper order
 *
 * Revision 1.7  2001/05/23 18:46:46  zcox
 * added fcmodeler.event.actions package, to support using Action objects
 * to handle specific GUI events
 *
 * Revision 1.6  2001/05/22 16:38:57  zcox
 * code clean-up
 *
 * Revision 1.5  2001/04/06 22:25:04  zcox
 * made import statements more class-specific
 *
 * Revision 1.4  2001/04/05 22:13:37  zcox
 * added support for showing the currently open file's name in the title bar
 *
 * Revision 1.3  2001/04/05 21:27:58  zcox
 * added InitialLayout, which speeds up the initial loading of a graph
 * by a HUGE factor
 *
 * Revision 1.2  2001/04/04 02:19:05  zcox
 * huge architectural changes
 *
 * Revision 1.1  2001/02/22 00:16:43  zcox
 * initial commit of FCModeler.java
 *
 *
 *
 */
