/* $Id: CreateSubgraphAction.java,v 1.1 2002/01/15 19:46:52 zcox Exp $
 *
 *
 */

package fcmodeler.actions;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.awt.Frame;

import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.GraphModel;
import diva.graph.model.BasicGraph;
import diva.graph.model.BasicNode;
import diva.graph.model.BasicEdge;
import diva.canvas.selection.SelectionModel;

import fcmodeler.FCModeler;
import fcmodeler.FCModelerApp;
import fcmodeler.FCModelerUtilities;
import fcmodeler.view.NodeFigure;
import fcmodeler.view.EdgeFigure;
import fcmodeler.subgraph.SubgraphImpl;
import fcmodeler.subgraph.Subgraph;
import fcmodeler.subgraph.SubgraphEdge;
import fcmodeler.subgraph.SubgraphNode;
import fcmodeler.subgraph.SubgraphFactoryDialog;


/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class CreateSubgraphAction extends AbstractGraphAction {
	
	/** Creates new CreateSubgraphAction */
	public CreateSubgraphAction() {
		super();
	}
	
	public CreateSubgraphAction(String name) {
		super(name);
	}
	
	public CreateSubgraphAction(String name, Icon icon) {
		super(name, icon);
	}
	
	public void actionPerformed(ActionEvent actionEvent) {
		Object selection = null;
		SelectionModel selectionModel = null;
		GraphModel subModel = null;
		Node tail = null, head = null;
		Edge edge = null;
		SubgraphNode subTail = null, subHead = null;
		SubgraphEdge subEdge = null;
		Iterator inEdges = null, outEdges = null;
		FCModelerApp app = null;
		Iterator selections = null;
		
		//have AbstractGraphAction get the fcmodeler object
		super.actionPerformed(actionEvent);
		
		//get the SelectionModel, which contains all the selected graph objects
		selectionModel = _fcmodeler.getScrollableJGraph().getGraphPane().getGraphView().
			getGraphController().getNodeInteractor().getSelectionModel();
		if (selectionModel.getSelectionCount() == 1) {
			selection = selectionModel.getFirstSelection();
			if (selection instanceof NodeFigure) {
				//get the selected node from the selected figure
				head = (Node) ((NodeFigure) selection).getUserObject();
				
				//create the new graph model to hold the subgraph
				subModel = new GraphModel(new SubgraphImpl());
				((Subgraph) subModel.getGraph()).setOriginalGraph((BasicGraph) head.getParent());
				
				//populate the subgraph
				subHead = (SubgraphNode) subModel.createNode(head);
				subModel.addNode(subHead);
				inEdges = head.inEdges();
				while (inEdges.hasNext()) {
					edge = (Edge) inEdges.next();
					tail = edge.getTail();
					subTail = ((Subgraph) subModel.getGraph()).getNode((BasicNode) tail);
					if (subTail == null) {
						subTail = (SubgraphNode) subModel.createNode(tail);
						subModel.addNode(subTail);
					}
					subEdge = (SubgraphEdge) subModel.createEdge(edge);
					subModel.connectEdge(subEdge, subTail, subHead);
				}
				subTail = subHead;
				tail = head;
				outEdges = tail.outEdges();
				while (outEdges.hasNext()) {
					edge = (Edge) outEdges.next();
					head = edge.getHead();
					subHead = ((Subgraph) subModel.getGraph()).getNode((BasicNode) head);
					if (subHead == null) {
						subHead = (SubgraphNode) subModel.createNode(head);
						subModel.addNode(subHead);
					}
					subEdge = (SubgraphEdge) subModel.createEdge(edge);
					subModel.connectEdge(subEdge, subTail, subHead);
				}
				
				if (actionEvent.getSource() instanceof Component) {
					app = FCModelerUtilities.getFCModelerApp((Component) actionEvent.getSource());
					if (app != null) {
						app.showNewFCModeler(_fcmodeler, subModel, "subgraph for " + tail.getSemanticObject());
					}
				}
				
			}
		}
		else {
			java.util.Collection nodes = new java.util.HashSet();
			java.util.Collection edges = new java.util.HashSet();
			
			selections = selectionModel.getSelection();
			while (selections.hasNext()) {
				selection = selections.next();
				if (selection instanceof NodeFigure) {
					nodes.add(((NodeFigure) selection).getUserObject());
				}
				if (selection instanceof EdgeFigure) {
					edges.add(((EdgeFigure) selection).getUserObject());
				}
			}
			//subModel = _fcmodeler.subgraph.SubgraphFactory.getInstance().createFromNodes(nodes);
			//subModel = _fcmodeler.subgraph.SubgraphFactory.getInstance().createFromNodes(nodes, 2);
			//subModel = _fcmodeler.subgraph.SubgraphFactory.getInstance().createFromEdges(edges);
			//subModel = _fcmodeler.subgraph.SubgraphFactory.getInstance().createFromNodesAndEdges(nodes, edges);
			SubgraphFactoryDialog dialog = new SubgraphFactoryDialog(new Frame(), true);
			subModel = dialog.showDialog(nodes, edges);
			if (subModel != null) {
				if (actionEvent.getSource() instanceof Component) {
					app = FCModelerUtilities.getFCModelerApp((Component) actionEvent.getSource());
					if (app != null) {
						app.showNewFCModeler(_fcmodeler, subModel, "subgraph");
					}
				}
			}
		}
	}
}

/* 
 * $Log: CreateSubgraphAction.java,v $
 * Revision 1.1  2002/01/15 19:46:52  zcox
 * initial commit of actions package
 *
 * Revision 1.3  2001/08/27 16:18:46  zcox
 * summer 2001 changes
 *
 *
 *
 */
