/* $Id: FindCyclesAction.java,v 1.3 2002/05/16 21:07:16 zcox Exp $
 *
 *
 */

package fcmodeler.actions;

import java.awt.Component;
import java.awt.event.ActionEvent;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;

import diva.util.PropertyContainer;
import diva.graph.model.Graph;
import diva.graph.model.IndexedGraph;
import diva.graph.model.GraphModel;
import diva.graph.model.Node;
import diva.graph.model.Edge;

import fcmodeler.subgraph.Path;
import fcmodeler.subgraph.Cycle;
import fcmodeler.subgraph.Subgraph;
import fcmodeler.subgraph.SubgraphNode;
import fcmodeler.subgraph.SubgraphEdge;
import fcmodeler.subgraph.SubgraphFactory;
import fcmodeler.subgraph.CycleFrame;
import fcmodeler.WalkSelectionFrame;
import fcmodeler.actions.SCCAction;
import fcmodeler.FCModelerApp;
import fcmodeler.FCModelerUtilities;
import fcmodeler.io.CycleFileWriter;
import fcmodeler.io.xml.XMLFileFilter;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public class FindCyclesAction extends AbstractGraphAction implements Runnable {
	private Set _cycles = null;
	private ActionEvent _actionEvent = null;
	//private PrintWriter _out = null;
	//private File _outFile = null;
	private CycleFileWriter _writer = null;
	private boolean _saveToFile = false;
	private boolean _saveInMemory = false;
	private int _maxLength = -1;
	private static final String _leftBracket = "[";
	private static final String _rightBracket = "]";
	private static final String _space = " ";
	private static final String _cycleString = "-cycle: ";
	
	/** Creates new FindCyclesAction */
	public FindCyclesAction() {
		super();
	}
	
	public FindCyclesAction(String name) {
		super(name);
	}
	
	public FindCyclesAction(String name, Icon icon) {
		super(name, icon);
	}
	
	//called when user clicks "find cycles" button
	public void actionPerformed(ActionEvent actionEvent) {
		int returnVal = 0;
		CSAConfigDialog dialog = null;
		JFileChooser fileChooser = null;
		
		//have AbstractGraphAction get the fcmodeler object
		super.actionPerformed(actionEvent);
		
		_actionEvent = actionEvent;
		
		//start the cycle-search algorithm in a new thread (run() method is called)
		dialog = new CSAConfigDialog(this);
		if (dialog.showDialog()) {
			if (_saveToFile) {
				//open a file chooser dialog with custom file filters
				fileChooser = new JFileChooser(_fcmodeler.getPathToGraphs());
				fileChooser.addChoosableFileFilter(new XMLFileFilter());
				returnVal = fileChooser.showOpenDialog(_fcmodeler.getParentComponent());

				//if the user opened a file
				if(returnVal == JFileChooser.APPROVE_OPTION) {
					//_outFile = fileChooser.getSelectedFile();
					_writer = new CycleFileWriter(fileChooser.getSelectedFile());
				}
				else {
					_saveToFile = false;
				}
			}
			
			new Thread(this).start();
		}
	}
	
	private void addCycle(Path path, SubgraphEdge edge, SubgraphNode head) {
		Cycle cycle = null;
		Iterator pathIterator = null;
		Object obj = null;
		SubgraphNode subNode = null;
		SubgraphEdge subEdge = null;
		int cycleCount = 0;
		StringBuffer sb = new StringBuffer();
		
		//create the new cycle & store it
		cycle = new Cycle();
		//the nodes in path are SubgraphNode obj's - need to put their
		//original nodes in the cycle obj
		pathIterator = path.iterator();
		while (pathIterator.hasNext()) {
			obj = pathIterator.next();
			if (obj instanceof SubgraphNode) {
				subNode = (SubgraphNode) obj;
				cycle.add(((SubgraphNode) subNode.getOriginalNode()).getOriginalNode());
			}
			else {
				subEdge = (SubgraphEdge) obj;
				cycle.add(((SubgraphEdge) subEdge.getOriginalEdge()).getOriginalEdge());
			}
		}
		cycle.add(((SubgraphEdge) ((SubgraphEdge) edge).getOriginalEdge()).getOriginalEdge());
		cycle.add(((SubgraphNode) ((SubgraphNode) head).getOriginalNode()).getOriginalNode());
		
		if (_saveInMemory) {
			_cycles.add(cycle);
		}
		
		if (_saveToFile) {
			_writer.writeCycle(cycle);
		}
	}
	
	private boolean circuit(Node tail, Node rootNode, Path path) {
		Iterator outEdges = null;
		Node head = null;
		Edge edge = null;
		boolean cycleFound = false;
		
		//add tail to path
		path.add(tail);
		
		//make tail blocked
		setBlocked(tail, true);
		
		//iterate over tail's out-edges
		outEdges = tail.outEdges();
		while (outEdges.hasNext()) {
			edge = (Edge) outEdges.next();
			head = edge.getHead();
			
			//if head == rootNode ==> we just found a cycle
			if (head.equals(rootNode)) {
				//add cycle to cycle set
				addCycle(path, (SubgraphEdge) edge, (SubgraphNode) head);
				
				//set the cycle found flag
				cycleFound = true;
			}
			//otherwise see if head is available
			else if (!getBlocked(head) && (_maxLength == -1 || path.size() < 2*_maxLength-2)) {
				//extend path to head
				path.add(edge);
				if (circuit(head, rootNode, path)) {
					cycleFound = true;
				}
			}
		}
		
		//if a cycle was found from tail
		if (cycleFound) {
			//make tail & all of it's unavailable predecessors available
			unblock(tail);
		}
		else {
			//add tail to successors' lists of unavailable predecessors
			outEdges = tail.outEdges();
			while (outEdges.hasNext()) {
				edge = (Edge) outEdges.next();
				head = edge.getHead();
				
				if (!getBlockedNodes(head).contains(tail)) { // <-------- is contains() necessary???
					getBlockedNodes(head).add(tail);
				}
			}
		}
		
		//back path up to previous node, if there is one
		path.remove(path.size()-1);
		if (!path.isEmpty()) {
			path.remove(path.size()-1);
		}
		
		return cycleFound;
	}
	
	public Set findCycles(IndexedGraph graph) {
		List nodeList = new ArrayList();
		Set sccs = null;
		Iterator nodes = null, iterator = null;
		Node node = null;
		SCCAction sccAction = getSCCAction();
		Subgraph scc = null, subgraph = null, currentSCC = null;
		SubgraphNode subNode = null;
		Path path = new Path();
		GraphModel subModel = null;
		boolean sccFound = false;
		FCModelerApp app = FCModelerUtilities.getFCModelerApp((Component) _actionEvent.getSource());
		SubgraphNode rootNode = null;
		
		if (_saveInMemory) {
			_cycles = new HashSet();
		}
		if (_saveToFile) {
			/*try {
				_out = new PrintWriter(new BufferedWriter(new FileWriter(_outFile)));
			} catch (IOException e) {
				e.printStackTrace();
			}*/
			_writer.writeBeginning();
		}
		
		//create the node set for subgraph creation
		nodes = graph.nodes();
		while (nodes.hasNext()) {
			nodeList.add(nodes.next());
		}
		
		//create subgraph induced by node set & delete the node set obj
		subModel = SubgraphFactory.getInstance().createFromNodes(nodeList);
		subgraph = (Subgraph) subModel.getGraph();
		nodeList = null;
		
		//for each node in graph
		for (int s=subgraph.getNodeCount()-1; s>=0; s--) {
			rootNode = (SubgraphNode) subgraph.getNode(s);
			
			//app.showNewFCModeler(_fcmodeler, subModel, "subgraph");
			
			//find the strongly connected components of the graph
			sccAction.findStronglyConnectedComponents(subgraph);
			sccs = sccAction.getSubgraphs();
			
			//System.out.println("root node = " + rootNode.getOriginalNode().getSemanticObject());
			//System.out.println(sccs);
			//System.out.println();
			
			//find SCC that contains root node
			scc = null;
			sccFound = false;
			iterator = sccs.iterator();
			while (iterator.hasNext() && !sccFound) {
				currentSCC = (Subgraph) iterator.next();
				nodes = currentSCC.nodes();
				while (nodes.hasNext()) {
					subNode = (SubgraphNode) nodes.next();
					if (subNode.getOriginalNode().equals(rootNode)) {
						scc = currentSCC;
						sccFound = true;
						break;
					}
				}
			}
			
			//if (scc == null) {
			//	System.out.println("root node not found in SCC's");
			//}
			//else {
			//	System.out.println("root node " + rootNode.getOriginalNode().getSemanticObject() + " is in SCC " + scc);
			//}
			//System.out.println();
			
			//make sure scc is non-trivial
			if (scc != null && scc.getNodeCount() > 1) {
				//set up the node variables
				nodes = scc.nodes();
				while (nodes.hasNext()) {
					node = (Node) nodes.next();
					setBlocked(node, false);
					setBlockedNodes(node, new HashSet());
				}
				
				//find cycles rooted at node
				//System.out.println("subNode = " + ((SubgraphNode) subNode.getOriginalNode()).getOriginalNode().getSemanticObject());
				//System.out.println();
				circuit(subNode, subNode, path);
			}
			
			//don't consider root node anymore
			subModel.removeNode(rootNode);
			
			if (!path.isEmpty()) {
				System.out.println("path was not empty!!!!!!!!!!!!!!!!");
			}
		}
		
		if (_saveToFile) {
			//_out.close();
			_writer.writeEnding();
		}
		
		return _cycles;
	}
	
	private boolean getBlocked(PropertyContainer pc) {
		boolean blocked = false;
		Boolean b = null;
		
		b = (Boolean) pc.getProperty("blocked");
		if (b != null) {
			blocked = b.booleanValue();
		}
		
		return blocked;
	}
	
	public Set getCycles() {
		return _cycles;
	}
	
	private SCCAction getSCCAction() {
		Iterator actions = _fcmodeler.getActions();
		SCCAction sccaction = null;
		Object action = null;
		
		while (actions.hasNext()) {
			action = actions.next();
			if (action instanceof SCCAction) {
				sccaction = (SCCAction) action;
				break;
			}
		}
		
		return sccaction;
	}
	
	private Set getBlockedNodes(PropertyContainer pc) {
		Set blockedNodes = null;
		
		blockedNodes = (Set) pc.getProperty("blockedNodes");
		if (blockedNodes == null) {
			blockedNodes = Collections.EMPTY_SET;
		}
		
		return blockedNodes;
	}
	
	public int getMaxLength() {
		return _maxLength;
	}
	
	public boolean isSaveInMemory() {
		return _saveInMemory;
	}
	
	public boolean isSaveToFile() {
		return _saveToFile;
	}
	
	//called when the new thread starts running
	public void run() {
		Set cycles = null;
		//WalkSelectionFrame selFrame = null;
		CycleFrame frame = null;
		
		System.out.println("cycle search algorithm started...");
		long before = System.currentTimeMillis();
		
		//start the cycle search algorithm
		cycles = findCycles((IndexedGraph) _fcmodeler.getScrollableJGraph().getGraphModel().getGraph());
		
		long after = System.currentTimeMillis();
		System.out.println("cycle search algorithm finished in " + (after-before)/1000.0d + " seconds");
		
		//if at least one cycle was found
		if (_saveInMemory && !cycles.isEmpty()) {
			//open the window that displays the list of cycles
			//selFrame = new WalkSelectionFrame(cycles.toArray(), _fcmodeler.getScrollableJGraph().getGraphPane().getGraphView(),
			//	_fcmodeler.getJScrollPane().getViewport());
			frame = new CycleFrame(cycles.toArray(), _fcmodeler.getGraphView(), _fcmodeler.getJScrollPane().getViewport());
			//selFrame.setTitle("cycles");
			frame.setTitle("cycles");
			//selFrame.setLocation(_fcmodeler.getParentComponent().getX()+50, _fcmodeler.getParentComponent().getY()+50);
			frame.setLocation(_fcmodeler.getParentComponent().getX()+50, _fcmodeler.getParentComponent().getY()+50);
			
			//can't show the window in this thread, needs to be done in the event-dispatch thread
			//invokeLater() takes care of this
			//SwingUtilities.invokeLater(selFrame);
			SwingUtilities.invokeLater(frame);
		}
	}
	
	private void setBlocked(PropertyContainer pc, boolean blocked) {
		if (blocked) {
			pc.setProperty("blocked", Boolean.TRUE);
		}
		else {
			pc.setProperty("blocked", Boolean.FALSE);
		}
	}
	
	private void setBlockedNodes(PropertyContainer pc, Set blockedNodes) {
		pc.setProperty("blockedNodes", blockedNodes);
	}
	
	public void setMaxLength(int maxLength) {
		_maxLength = maxLength;
	}
	
	public void setSaveInMemory(boolean saveInMemory) {
		_saveInMemory = saveInMemory;
	}
	
	public void setSaveToFile(boolean saveToFile) {
		_saveToFile = saveToFile;
	}
	
	private void unblock(Node head) {
		Node blockedNode = null;
		Edge edge = null;
		Iterator blockedNodes = null;
		
		//set head to unblocked
		setBlocked(head, false);
		
		//unblock all blocked predecessors
		blockedNodes = getBlockedNodes(head).iterator();
		while (blockedNodes.hasNext()) {
			blockedNode = (Node) blockedNodes.next();
			blockedNodes.remove();
			if (getBlocked(blockedNode)) {
				unblock(blockedNode);
			}
		}
	}
}

/*
 * $Log: FindCyclesAction.java,v $
 * Revision 1.3  2002/05/16 21:07:16  zcox
 * implemented cycle filtering
 *
 * Revision 1.2  2002/02/15 19:40:13  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:46:53  zcox
 * initial commit of actions package
 *
 *
 *
 */
