/* $Id: HighlightWalk.java,v 1.1 2002/01/15 19:46:53 zcox Exp $
 *
 *
 */

package fcmodeler.actions;

import javax.swing.JViewport;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.util.Iterator;

import diva.graph.GraphView;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.canvas.FigureDecorator;
import diva.canvas.selection.BasicSelectionRenderer;
import diva.canvas.selection.SelectionInteractor;

import fcmodeler.subgraph.Walk;
import fcmodeler.view.BasicConnectorHighlighter;
import fcmodeler.FCModelerUtilities;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class HighlightWalk extends Object implements Runnable {
	protected Walk _walk = null;
	protected GraphView _graphView = null;
	protected JViewport _viewport = null;
	
	/** Creates new HighlightWalk */
	public HighlightWalk() {
		
	}
	
	public HighlightWalk(Walk walk, GraphView graphView, JViewport viewport) {
		_walk = walk;
		_graphView = graphView;
		_viewport = viewport;
	}
	
	public void clearHighlights() {
		if (_graphView != null) {
			_graphView.getGraphController().getNodeInteractor().getSelectionModel().clearSelection();
			_graphView.getGraphController().getEdgeInteractor().getSelectionModel().clearSelection();
		}
	}
	
	public void clearHighlights(GraphView graphView) {
		_graphView = graphView;
		clearHighlights();
	}
	
	public GraphView getGraphView() {
		return _graphView;
	}
	
	public JViewport getJViewport() {
		return _viewport;
	}
	
	public Walk getWalk() {
		return _walk;
	}
	
	public void highlightWalk() {
		//Iterator nodes = null, edges = null;
		Node node = null;
		Edge edge = null;
		FigureDecorator origDecorator = null;
		BasicConnectorHighlighter highlighter = new BasicConnectorHighlighter();;
		BasicSelectionRenderer selRenderer = null;
		Rectangle2D nodeBounds = null;
		double x = 0.0d, y = 0.0d;
		int nodeCount = 0;
		Dimension viewSize = null;
		Point viewPosition = null;
		Point2D.Double nodeCenter = new Point2D.Double();
		Iterator iterator = null;
		Object element = null;
		
		if (_walk != null && _graphView != null && _viewport != null) {
			iterator = _walk.iterator();
			while (iterator.hasNext()) {
				element = iterator.next();
				
				if (element instanceof Node) {
					node = (Node) element;
					nodeBounds = _graphView.getNodeFigure(node).getBounds();
					nodeCenter.x = nodeBounds.getCenterX();
					nodeCenter.y = nodeBounds.getCenterY();
					_graphView.getGraphicsPane().getTransformContext().getScreenTransform().
						transform(nodeCenter, nodeCenter);
					x += nodeCenter.x;
					y += nodeCenter.y;
					nodeCount++;
					_graphView.getGraphController().getNodeInteractor().getSelectionModel().
						addSelection(_graphView.getNodeFigure(node));
				}
				else if (element instanceof Edge) {
					edge = (Edge) element;
					selRenderer = (BasicSelectionRenderer) ((SelectionInteractor) _graphView.
						getEdgeFigure(edge).getInteractor()).getSelectionRenderer();
					origDecorator = selRenderer.getDecorator();
					selRenderer.setDecorator(highlighter);
					_graphView.getGraphController().getEdgeInteractor().getSelectionModel().
						addSelection(_graphView.getEdgeFigure(edge));
					selRenderer.setDecorator(origDecorator);
				}
			}
			
			
			/*nodes = _walk.nodes();
			while (nodes.hasNext()) {
				node = (Node) nodes.next();
				nodeBounds = _graphView.getNodeFigure(node).getBounds();
				//DEBUG
				//x += nodeBounds.getCenterX();
				//y += nodeBounds.getCenterY();
				nodeCenter.x = nodeBounds.getCenterX();
				nodeCenter.y = nodeBounds.getCenterY();
				_graphView.getGraphicsPane().getTransformContext().getScreenTransform().
					transform(nodeCenter, nodeCenter);
				x += nodeCenter.x;
				y += nodeCenter.y;
				//END DEBUG
				nodeCount++;
				_graphView.getGraphController().getNodeInteractor().getSelectionModel().
					addSelection(_graphView.getNodeFigure(node));
			}
			
			highlighter = new BasicConnectorHighlighter();
			edges = _walk.edges();
			while (edges.hasNext()) {
				edge = (Edge) edges.next();
				selRenderer = (BasicSelectionRenderer) ((SelectionInteractor) _graphView.
					getEdgeFigure(edge).getInteractor()).getSelectionRenderer();
				origDecorator = selRenderer.getDecorator();
				selRenderer.setDecorator(highlighter);
				_graphView.getGraphController().getEdgeInteractor().getSelectionModel().
					addSelection(_graphView.getEdgeFigure(edge));
				selRenderer.setDecorator(origDecorator);
			}*/
			
			viewSize = _viewport.getExtentSize();
			viewPosition = new Point((int) (x/nodeCount - 0.5*viewSize.width), (int) (y/nodeCount - 0.5*viewSize.height));
			FCModelerUtilities.positionViewport(_viewport, viewPosition);
		}
	}
	
	public void highlightWalk(Walk walk, GraphView graphView, JViewport viewport) {
		_walk = walk;
		_graphView = graphView;
		_viewport = viewport;
		
		highlightWalk();
	}
	
	public void run() {
		highlightWalk();
	}
	
	public void setGraphView(GraphView graphView) {
		_graphView = graphView;
	}
	
	public void setJViewport(JViewport viewport) {
		_viewport = viewport;
	}
	
	public void setWalk(Walk walk) {
		_walk = walk;
	}
}

/* 
 * $Log: HighlightWalk.java,v $
 * Revision 1.1  2002/01/15 19:46:53  zcox
 * initial commit of actions package
 *
 *
 *
 */
