/* $Id: OpenCycleFileAction.java,v 1.3 2002/05/21 02:29:30 zcox Exp $
 *
 *
 */

package fcmodeler.actions;

import javax.swing.Icon;
import javax.swing.JFileChooser;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;

import fcmodeler.io.CycleFileReader;
import fcmodeler.io.xml.XMLFileFilter;
import fcmodeler.FCModelerUtilities;
//import fcmodeler.WalkSelectionFrame;
import fcmodeler.subgraph.CycleFrame;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public class OpenCycleFileAction extends fcmodeler.actions.AbstractGraphAction {
	
	/** Creates new OpenCycleFileAction */
	public OpenCycleFileAction() {
		super();
	}
	
	public OpenCycleFileAction(String name) {
		super(name);
	}
	
	public OpenCycleFileAction(String name, Icon icon) {
		super(name, icon);
	}
	
	public void actionPerformed(ActionEvent actionEvent) {
		JFileChooser openFile = null;
		int returnVal = 0;
		File file = null;
		SAXParserFactory spf = null;
		XMLReader xmlReader = null;
		SAXParser saxParser = null;
		CycleFileReader reader = null;
		//WalkSelectionFrame selFrame = null;
		CycleFrame selFrame = null;
		
		//have AbstractGraphAction get the fcmodeler object
		super.actionPerformed(actionEvent);
		
		//use the little hour glass figure for the cursor
		_fcmodeler.getParentComponent().setCursor(FCModelerUtilities.getWaitCursor());
		
		//open a file chooser dialog with custom file filters
		openFile = new JFileChooser(_fcmodeler.getPathToGraphs());
		openFile.addChoosableFileFilter(new XMLFileFilter());
		returnVal = openFile.showOpenDialog(_fcmodeler.getParentComponent());
		
		//if the user opened a file
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			file = openFile.getSelectedFile();
			
			// Create a JAXP SAXParserFactory and configure it
			spf = SAXParserFactory.newInstance();
			spf.setValidating(true);
			try {
				// Create a JAXP SAXParser
				saxParser = spf.newSAXParser();
				
				// Get the encapsulated SAX XMLReader
				xmlReader = saxParser.getXMLReader();
			} catch (Exception ex) {
				System.err.println(ex);
				System.exit(1);
			}
			
			// Set the ContentHandler of the XMLReader
			reader = new CycleFileReader(_fcmodeler.getScrollableJGraph().getGraphModel().getGraph());
			xmlReader.setContentHandler(reader);
			
			// Set an ErrorHandler before parsing
			xmlReader.setErrorHandler(reader);
			
			try {
				// Tell the XMLReader to parse the XML document
				xmlReader.parse(file.toURL().toString());
			} catch (SAXException se) {
				System.err.println(se.getMessage());
				System.exit(1);
			} catch (IOException ioe) {
				System.err.println(ioe);
				System.exit(1);
			}
			
			if (reader.getCycles().size() > 0) {
				//selFrame = new WalkSelectionFrame(reader.getCycles().toArray(), _fcmodeler.getScrollableJGraph().getGraphPane().getGraphView(),
				//_fcmodeler.getJScrollPane().getViewport());
				selFrame = new CycleFrame(reader.getCycles().toArray(), _fcmodeler.getGraphView(), _fcmodeler.getJScrollPane().getViewport());
				selFrame.setTitle("cycles from " + file.getName());
				selFrame.setLocation(_fcmodeler.getParentComponent().getX()+50, _fcmodeler.getParentComponent().getY()+50);
				selFrame.run();
			}
		}
		
		//change back to normal cursor
		_fcmodeler.getParentComponent().setCursor(FCModelerUtilities.getDefaultCursor());
	}
}

/* 
 * $Log: OpenCycleFileAction.java,v $
 * Revision 1.3  2002/05/21 02:29:30  zcox
 * use CycleFrame instead of WalkSelectionFrame
 *
 * Revision 1.2  2002/02/15 19:40:14  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:46:53  zcox
 * initial commit of actions package
 *
 * 
 */
