/* $Id: OpenGraphAction.java,v 1.2 2002/02/15 19:40:14 zcox Exp $
 *
 *
 * Created on May 23, 2001, 11:48 AM
 */

package fcmodeler.actions;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.JInternalFrame;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileReader;
import java.io.FileNotFoundException;

import fcmodeler.io.GraphIOFactory;
import fcmodeler.io.xml.XMLFileFilter;
import fcmodeler.io.dot.SimpleDotFileFilter;
import fcmodeler.io.dot.RankClusterDotFileFilter;
import fcmodeler.io.GraphReader;
import fcmodeler.io.GraphFileOptionsPanel;
import fcmodeler.FCModelerUtilities;

/** is...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class OpenGraphAction extends AbstractGraphAction {
	
	/** Creates new OpenGraphAction */
	public OpenGraphAction() {
		super();
	}
	
	public OpenGraphAction(String name) {
		super(name);
	}
	
	public OpenGraphAction(String name, Icon icon) {
		super(name, icon);
	}
	
	public void actionPerformed(ActionEvent actionEvent) {
		super.actionPerformed(actionEvent);
		
		_fcmodeler.getParentComponent().setCursor(FCModelerUtilities.getWaitCursor());
		
		JFileChooser openGraph = new JFileChooser(_fcmodeler.getPathToGraphs());
		openGraph.addChoosableFileFilter(new RankClusterDotFileFilter());
		openGraph.addChoosableFileFilter(new SimpleDotFileFilter());
		openGraph.addChoosableFileFilter(new XMLFileFilter());
		GraphFileOptionsPanel options = new GraphFileOptionsPanel();
		options.setTitle("read from file");
		openGraph.setAccessory(options);
		int returnVal = openGraph.showOpenDialog(_fcmodeler.getParentComponent());
		
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			//get the current file filter, and make sure it's one of fcmodeler's filters
			FileFilter fileFilter = openGraph.getFileFilter();
			if (fileFilter instanceof GraphIOFactory) {
				//get the appropriate graph reader, read the graph, and update fcmodeler's view
				GraphReader graphReader = ((GraphIOFactory) fileFilter).getGraphReader();
				File file = openGraph.getSelectedFile();
				try {
					graphReader.readCoordinates(options.isCoordinatesSelected());
					graphReader.readMappings(options.isMappingsSelected());
					graphReader.readTopology(options.isTopologySelected());
					graphReader.readGraph(new FileReader(file));
					//_fcmodeler.setGraphModel(graphReader.getGraphModel());
					graphReader.configure(_fcmodeler);
				
					//update the fcmodeler title bar
					if (_fcmodeler.getParentComponent() instanceof Frame) {
						((Frame) _fcmodeler.getParentComponent()).setTitle(file.getName());
					}
					else if (_fcmodeler.getParentComponent() instanceof JInternalFrame) {
						((JInternalFrame) _fcmodeler.getParentComponent()).setTitle(file.getName());
					}
				} catch (FileNotFoundException e) {
					System.err.println("file not found: " + file);
				}
			}
			else {
				throw new UnsupportedOperationException("not a valid file type");
			}
		}
		
		//change back to normal cursor
		_fcmodeler.getParentComponent().setCursor(FCModelerUtilities.getDefaultCursor());
	}
}

/*
 * $Log: OpenGraphAction.java,v $
 * Revision 1.2  2002/02/15 19:40:14  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:46:53  zcox
 * initial commit of actions package
 *
 * Revision 1.1  2001/05/23 18:49:26  zcox
 * added fcmodeler.event.actions package, to support using Action objects
 * to handle specific GUI events
 *
 *
 *
 *
 */
