/* $Id: SCCAction.java,v 1.1 2002/01/15 19:46:54 zcox Exp $
 *
 *
 */

package fcmodeler.actions;

import java.awt.event.ActionEvent;
import javax.swing.Icon;
import javax.swing.SwingUtilities;
import java.util.Set;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.ListIterator;
import java.util.ArrayList;
import java.util.Iterator;

import diva.util.PropertyContainer;
import diva.graph.model.Graph;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.GraphModel;

import fcmodeler.subgraph.Path;
import fcmodeler.subgraph.Cycle;
import fcmodeler.subgraph.SubgraphFactory;
import fcmodeler.WalkSelectionFrame;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class SCCAction extends AbstractGraphAction implements Runnable {
	private static Integer ZERO = new Integer(0);
	private static Integer ONE = new Integer(1);
	private static Integer TWO = new Integer(2);
	private int _time = 0;
	private int _i = 0;
	private Set _sccs = null;
	private List _ordering = null;
	
	/** Creates new SCCAction */
	public SCCAction() {
		super();
	}
	
	public SCCAction(String name) {
		super(name);
	}
	
	public SCCAction(String name, Icon icon) {
		super(name, icon);
	}
	
	//called when user clicks "find cycles" button
	public void actionPerformed(ActionEvent actionEvent) {
		//have AbstractGraphAction get the fcmodeler object
		super.actionPerformed(actionEvent);
		
		//start the cycle-search algorithm in a new thread (run() method is called)
		new Thread(this).start();
	}
	
	private Set dfs(Graph graph, List ordering) {
		Set sccs = new HashSet();
		Path scc = null;
		Node node = null;
		Iterator nodes = null;
		
		//init all nodes
		nodes = graph.nodes();
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			settvisit(node, 0);
			settexpl(node, 0);
		}
		
		//init global vars
		_time = 0;
		
		nodes = ordering.iterator();
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			if (gettvisit(node) == 0) {
				scc = new Path();
				dfsproc(node, scc);
				if (scc.size() > 1) {
					sccs.add(scc);
				}
			}
		}
		
		return sccs;
	}
	
	private void dfsproc(Node head, Path scc) {
		Node tail = null;
		Iterator inEdges = null;
		Edge edge = null;
		
		_time++;
		settvisit(head, _time);
		
		scc.add(head);
		
		inEdges = head.inEdges();
		while (inEdges.hasNext()) {
			edge = (Edge) inEdges.next();
			tail = edge.getTail();
			if (gettvisit(tail) == 0) {
				dfsproc(tail, scc);
			}
		}
		
		_time++;
		settexpl(head, _time);
	}
	
	private List dfsa(Graph graph) {
		List ordering = null;
		Node node = null;
		Iterator nodes = null;
		int nodeCount = 0;
		
		//init all nodes
		nodes = graph.nodes();
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			settvisit(node, 0);
			settexpl(node, 0);
			nodeCount++;
		}
		
		ordering = new ArrayList(Collections.nCopies(nodeCount, new Object()));
		
		//init global vars
		_time = 0;
		_i = nodeCount;
		
		nodes = graph.nodes();
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			if (gettvisit(node) == 0) {
				dfsaproc(node, ordering);
			}
		}
		
		return ordering;
	}
	
	private void dfsaproc(Node tail, List ordering) {
		Node head = null;
		Iterator outEdges = null;
		Edge edge = null;
		
		_time++;
		settvisit(tail, _time);
		
		outEdges = tail.outEdges();
		while (outEdges.hasNext()) {
			edge = (Edge) outEdges.next();
			head = edge.getHead();
			if (gettvisit(head) == 0) {
				dfsaproc(head, ordering);
			}
		}
		
		_time++;
		settexpl(tail, _time);
		_i--;
		ordering.set(_i, tail);
	}
	
	public Set findStronglyConnectedComponents(Graph graph) {
		//Set sccs = null;
		//List ordering = null;
		
		//compute 'acyclic' ordering of graph
		_ordering = dfsa(graph);
		
		//find strongly connected components
		_sccs = dfs(graph, _ordering);
		
		return _sccs;
	}
	
	public List getNodeOrdering() {
		return _ordering;
	}
	
	public Set getSCCs() {
		return _sccs;
	}
	
	public Set getSubgraphs() {
		Set subgraphs = new HashSet();
		Iterator iterator = _sccs.iterator();
		Path scc = null;
		GraphModel subModel = null;
		
		while (iterator.hasNext()) {
			scc = (Path) iterator.next();
			if (scc.size() > 1) {
				subModel = SubgraphFactory.getInstance().createFromNodes(scc);
				subgraphs.add(subModel.getGraph());
			}
		}
		
		return subgraphs;
	}
	
	private int gettexpl(PropertyContainer pc) {
		int texpl = 0;
		
		texpl = ((Integer) pc.getProperty("texpl")).intValue();
		
		return texpl;
	}
	
	private int gettvisit(PropertyContainer pc) {
		int tvisit = 0;
		
		tvisit = ((Integer) pc.getProperty("tvisit")).intValue();
		
		return tvisit;
	}
	
	//called when the new thread starts running
	public void run() {
		WalkSelectionFrame selFrame = null;
		//Set sccs = null;
		
		//System.out.println("cycle search algorithm started...");
		//long before = System.currentTimeMillis();
		
		//find the set of strongly connected components
		_sccs = findStronglyConnectedComponents(_fcmodeler.getScrollableJGraph().getGraphModel().getGraph());
		
		//long after = System.currentTimeMillis();
		//System.out.println("cycle search algorithm finished in " + (after-before)/1000.0d + " seconds");
		
		//if at least one cycle was found
		if (!_sccs.isEmpty()) {
			//open the window that displays the list of cycles
			selFrame = new WalkSelectionFrame(_sccs.toArray(), _fcmodeler.getScrollableJGraph().getGraphPane().getGraphView(),
				_fcmodeler.getJScrollPane().getViewport());
			selFrame.setTitle("strongly connected components");
			selFrame.setLocation(_fcmodeler.getParentComponent().getX()+50, _fcmodeler.getParentComponent().getY()+50);
		 
			//can't show the window in this thread, needs to be done in the event-dispatch thread
			//invokeLater() takes care of this
			SwingUtilities.invokeLater(selFrame);
		}
	}
	
	private void settexpl(PropertyContainer pc, int texpl) {
		pc.setProperty("texpl", new Integer(texpl));
	}
	
	private void settvisit(PropertyContainer pc, int tvisit) {
		pc.setProperty("tvisit", new Integer(tvisit));
	}
}

/*
 * $Log: SCCAction.java,v $
 * Revision 1.1  2002/01/15 19:46:54  zcox
 * initial commit of actions package
 *
 *
 *
 */
