/*
 * SOMAction.java
 *
 * Created on May 14, 2002, 11:22 AM
 */

package fcmodeler.actions;

import javax.swing.Icon;
import javax.swing.JFrame;
import java.awt.event.ActionEvent;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.Iterator;
import java.util.Collection;

import jsomap.data.DoubleData;
import jsomap.data.LatticeDoubleData;
import jsomap.data.IntegerData;
import jsomap.data.RectangularAdjacencyMatrix;
import jsomap.map.BasicMap;
import jsomap.map.Model;
import jsomap.map.Node;
import jsomap.map.DoubleNode;
import jsomap.algorithm.Algorithm;
import jsomap.algorithm.BatchAlgorithm;
import jsomap.algorithm.parameters.Metric;
import jsomap.algorithm.parameters.Kernel;
import jsomap.algorithm.parameters.GaussianKernel;
import jsomap.algorithm.parameters.EuclideanDoubleMetric;
import jsomap.algorithm.parameters.Decay1;
import jsomap.algorithm.parameters.BatchUpdater;
import jsomap.algorithm.parameters.BatchParameters;

import fcmodeler.jsomap.WGData;
import fcmodeler.jsomap.CycleWGData;
import fcmodeler.jsomap.RandomModels;
import fcmodeler.jsomap.EuclideanWGMetric;
import fcmodeler.jsomap.BatchWGUpdater;
import fcmodeler.jsomap.Projector;
import fcmodeler.jsomap.MapPanel;
import fcmodeler.jsomap.HistogramPanel;

/**
 *
 * @author  default
 * @version
 */
public class SOMAction extends AbstractGraphAction {
	
	/** Creates new SOMAction */
	public SOMAction() {
		super();
	}
	
	public SOMAction(String name) {
		super(name);
	}
	
	public SOMAction(String name, Icon icon) {
		super(name, icon);
	}
	
	public void actionPerformed(ActionEvent event) {
		super.actionPerformed(event);
		
		new Thread(new Runnable() {
			public void run() {
				long start = System.currentTimeMillis();
				System.out.println("som algorithm started...");
				
				//get the cycles
				Set cycles = null;
				for (Iterator iterator = _fcmodeler.getActions(); iterator.hasNext();) {
					Object obj = iterator.next();
					if (obj instanceof FindCyclesAction) {
						cycles = ((FindCyclesAction) obj).getCycles();
						break;
					}
				}
				if (cycles == null) { throw new IllegalStateException("must find cycles before running som"); }

				//create the data
				WGData data = new CycleWGData(cycles);

				//create the map size
				int rows = 4;
				int cols = 4;

				//create the nodes
				DoubleData nodes = new LatticeDoubleData(rows, cols);

				//create the initial models
				Collection models = new RandomModels(data, rows*cols);
				//out.println();
				//out.println("initial models:");
				//for (Iterator iterator = models.iterator(); iterator.hasNext();) {
				//	out.println(((WGModel) iterator.next()).getWG());
				//}

				//create the map architecture (the connections between the nodes)
				IntegerData adjMatrix = new RectangularAdjacencyMatrix(rows, cols);

				//create the map
				BasicMap map = new BasicMap();
				Iterator iterator = models.iterator();
				for (int i=0, count=nodes.count(); i<count; i++) {
					Model model = (Model) iterator.next();
					Node node = new DoubleNode(nodes.getDouble(i));
					map.add(node, model);
				}
				for (int i=0, count=adjMatrix.count(); i<count; i++) {
					int[] row = adjMatrix.getInteger(i).get();
					Set neighbors = new HashSet();
					for (int j=0, length=row.length; j<length; j++) {
						if (row[j] > 0) {
							neighbors.add(map.getNode(j));
						}
					}
					map.setConnectedNodes(i, neighbors);
				}

				//create the parameters
				int iterations = 20;
				double initialValue = 0.9d, finalValue = 0.001d;
				Metric metric = new EuclideanWGMetric();
				Kernel kernel = new GaussianKernel(new EuclideanDoubleMetric(), new Decay1(initialValue, finalValue, iterations));
				BatchUpdater updater = new BatchWGUpdater();
				BatchParameters parameters = new BatchParameters(iterations, metric, kernel, updater);

				//create the algorithm
				//long start = System.currentTimeMillis();
				//System.out.println("starting algorithm at " + new java.util.Date(start));
				Algorithm algorithm = new BatchAlgorithm(data, map, parameters);
				algorithm.start();

				//output the results
				List setList = (new Projector()).project(data, map, metric);
				/*for (int i=0, count=map.count(); i<count; i++) {
					out.println();
					out.println("node " + i + ":");
					out.println(map.getModel(i).getPattern());
					Set nodeSet = (Set) setList.get(i);
					for (Iterator iterator2 = nodeSet.iterator(); iterator2.hasNext();) {
						out.println(iterator2.next());
					}
				}*/

				//output sorted results
				/*for (int i=0, count=map.count(); i<count; i++) {
					WeightedSet ws = ((WeightedGraph) map.getModel(i).getPattern()).getNodes();
					SortedMap smap = new TreeMap(new ReverseComparator());
					for (Iterator iterator2 = ws.iterator(); iterator2.hasNext();) {
						Object obj = iterator2.next();
						Double weight = new Double(ws.get(obj));
						Set set = (Set) smap.get(weight);
						if (set == null) { set = new HashSet(); smap.put(weight, set); }
						set.add(obj);
					}
					out.println();
					out.println("node " + i + ":");
					for (Iterator iterator2 = smap.entrySet().iterator(); iterator2.hasNext();) {
						out.println(iterator2.next());
					}
				}*/

				//show the map panel
				JFrame frame = new JFrame();
				frame.getContentPane().add(new MapPanel(map, rows, cols));
				frame.setTitle("map view");
				frame.setSize(500, 500);
				frame.setLocation(120, 120);
				frame.show();

				//show the node histogram
				frame = new JFrame();
				frame.getContentPane().add(new HistogramPanel(map, setList, true));
				frame.setTitle("node histogram");
				frame.setSize(500, 500);
				frame.setLocation(140, 140);
				frame.show();

				//show the edge histogram
				frame = new JFrame();
				frame.getContentPane().add(new HistogramPanel(map, setList, false));
				frame.setTitle("edge histogram");
				frame.setSize(500, 500);
				frame.setLocation(160, 160);
				frame.show();
				
				System.out.println("som algorithm finished in " + ((System.currentTimeMillis() - start)/1000.0d) + " seconds");
			}
		}).start();
	}
	
}
