/* $Id: AnimationControlPanel.java,v 1.2 2002/02/15 19:40:36 zcox Exp $
 *
 *
 */

package fcmodeler.animation;

import javax.swing.JPanel;
import javax.swing.JLabel;
import java.util.Hashtable; // Hashtable has to be used for JSlider (can't take a Map)

/** A user interface for controlling an animator.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class AnimationControlPanel extends JPanel implements AnimationListener  {
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel _controlPanel;
	private javax.swing.JButton _previous;
	private javax.swing.JButton _play;
	private javax.swing.JButton _pause;
	private javax.swing.JButton _stop;
	private javax.swing.JButton _next;
	private javax.swing.JScrollBar _frameScroll;
	private javax.swing.JLabel _frameLabel;
	private javax.swing.JPanel _delayPanel;
	private javax.swing.JLabel _delayLabel;
	private javax.swing.JSlider _delaySlider;
	// End of variables declaration//GEN-END:variables
	private Animator _animator = null;
	private int _maxDelay = 5000;
	
	/** Creates a new <CODE>AnimationControlPanel</CODE> that controls the specified animator.
	 *
	 * @param animator the animator that this user interface controls.
	 */	
	public AnimationControlPanel(Animator animator) {
		if (animator == null) { throw new NullPointerException("animator was null"); }
		
		initComponents();
		_animator = animator;
		_animator.addAnimationListener(this);
		_frameLabel.setText("frame 0 / " + (_animator.getFrameCount()-1));
		_frameScroll.setMaximum(_animator.getFrameCount());
		//_frameScroll.setVisibleAmount(_animator.getFrameCount());
		_frameScroll.setValue(0);
		_delayLabel.setText((_delaySlider.getValue()/1000.0d) + "s delay");
	}

	/** Invoked when the animation is paused.
	 *
	 * @param evt the object representing the pause.
	 */
	public void animationPaused(AnimationEvent evt) {
		
	}
	
	/** Invoked when the animation is started.
	 *
	 * @param evt the object representing the start.
	 */
	public void animationStarted(AnimationEvent evt) {
		
	}
	
	/** Invoked when the animation is stopped.
	 *
	 * @param evt the object representing the stop.
	 */
	public void animationStopped(AnimationEvent evt) {
		
	}
	
	/** Invoked when the delay slider is adjusted.
	 *
	 * @param evt the event representing the slider change.
	 */	
	private void _delaySliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event__delaySliderStateChanged
		int value = _delaySlider.getValue();
		
		if (value == _maxDelay) {
			_animator.pause();
			_delayLabel.setText("paused");
		}
		else {
			if (_animator.isPaused()) {
				_animator.start();
			}
			_animator.setDelay(value);
			_delayLabel.setText((value/1000.0d) + "s delay");
		}
	}//GEN-LAST:event__delaySliderStateChanged
	
	/** Invoked when a new frame is shown by the animation.
	 *
	 * @param evt the object representing the new frame event.
	 */
	public void frameShown(AnimationEvent evt) {
		//System.out.println("frame " + evt.getFrame() + " shown");
		_frameLabel.setText("frame " + evt.getFrame() + " / " + (_animator.getFrameCount()-1));
		_frameScroll.setValue(evt.getFrame());
	}
	
	/** Returns the animator controlled by this user interface.
	 *
	 * @return the animator controlled by this user interface.
	 */	
	public Animator getAnimator() {
		return _animator;
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the FormEditor.
	 */
	private void initComponents() {//GEN-BEGIN:initComponents
		_controlPanel = new javax.swing.JPanel();
		_previous = new javax.swing.JButton();
		_play = new javax.swing.JButton();
		_pause = new javax.swing.JButton();
		_stop = new javax.swing.JButton();
		_next = new javax.swing.JButton();
		_frameScroll = new javax.swing.JScrollBar();
		_frameLabel = new javax.swing.JLabel();
		_delayPanel = new javax.swing.JPanel();
		_delayLabel = new javax.swing.JLabel();
		_delaySlider = new javax.swing.JSlider();
		
		setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		
		_controlPanel.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints2;
		
		_controlPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "control"));
		_previous.setIcon(new javax.swing.ImageIcon("icons/VCRBegin.png"));
		_previous.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_previousActionPerformed(evt);
			}
		});
		
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 0;
		gridBagConstraints2.gridy = 2;
		gridBagConstraints2.insets = new java.awt.Insets(0, 5, 5, 0);
		_controlPanel.add(_previous, gridBagConstraints2);
		
		_play.setIcon(new javax.swing.ImageIcon("icons/VCRPlay.png"));
		_play.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_playActionPerformed(evt);
			}
		});
		
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 1;
		gridBagConstraints2.gridy = 2;
		gridBagConstraints2.insets = new java.awt.Insets(0, 0, 5, 0);
		_controlPanel.add(_play, gridBagConstraints2);
		
		_pause.setIcon(new javax.swing.ImageIcon("icons/VCRPause.png"));
		_pause.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_pauseActionPerformed(evt);
			}
		});
		
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 2;
		gridBagConstraints2.gridy = 2;
		gridBagConstraints2.insets = new java.awt.Insets(0, 0, 5, 0);
		_controlPanel.add(_pause, gridBagConstraints2);
		
		_stop.setIcon(new javax.swing.ImageIcon("icons/VCRStop.png"));
		_stop.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_stopActionPerformed(evt);
			}
		});
		
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 3;
		gridBagConstraints2.gridy = 2;
		gridBagConstraints2.insets = new java.awt.Insets(0, 0, 5, 0);
		_controlPanel.add(_stop, gridBagConstraints2);
		
		_next.setIcon(new javax.swing.ImageIcon("icons/VCREnd.png"));
		_next.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_nextActionPerformed(evt);
			}
		});
		
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 4;
		gridBagConstraints2.gridy = 2;
		gridBagConstraints2.insets = new java.awt.Insets(0, 0, 5, 5);
		_controlPanel.add(_next, gridBagConstraints2);
		
		_frameScroll.setVisibleAmount(1);
		_frameScroll.setOrientation(javax.swing.JScrollBar.HORIZONTAL);
		_frameScroll.setMaximum(10);
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 0;
		gridBagConstraints2.gridy = 1;
		gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints2.insets = new java.awt.Insets(5, 5, 5, 5);
		_controlPanel.add(_frameScroll, gridBagConstraints2);
		
		_frameLabel.setText("frame");
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 0;
		gridBagConstraints2.gridy = 0;
		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints2.insets = new java.awt.Insets(0, 5, 0, 0);
		_controlPanel.add(_frameLabel, gridBagConstraints2);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		add(_controlPanel, gridBagConstraints1);
		
		_delayPanel.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints3;
		
		_delayPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "delay"));
		_delayLabel.setText("delay");
		gridBagConstraints3 = new java.awt.GridBagConstraints();
		gridBagConstraints3.gridx = 0;
		gridBagConstraints3.gridy = 0;
		gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints3.insets = new java.awt.Insets(0, 20, 0, 0);
		_delayPanel.add(_delayLabel, gridBagConstraints3);
		
		_delaySlider.setPaintLabels(true);
		_delaySlider.setPaintTicks(true);
		_delaySlider.setMajorTickSpacing(_maxDelay/5);
		_delaySlider.setMaximum(_maxDelay);
		_delaySlider.setValue(_maxDelay/5);
		Hashtable labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0") );
		labelTable.put(new Integer(1000), new JLabel("1") );
		labelTable.put(new Integer(2000), new JLabel("2") );
		labelTable.put(new Integer(3000), new JLabel("3") );
		labelTable.put(new Integer(4000), new JLabel("4") );
		labelTable.put(new Integer(5000), new JLabel("pause") );
		_delaySlider.setLabelTable(labelTable);
		
		_delaySlider.addChangeListener(new javax.swing.event.ChangeListener() {
			public void stateChanged(javax.swing.event.ChangeEvent evt) {
				_delaySliderStateChanged(evt);
			}
		});
		
		gridBagConstraints3 = new java.awt.GridBagConstraints();
		gridBagConstraints3.gridx = 0;
		gridBagConstraints3.gridy = 1;
		gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints3.insets = new java.awt.Insets(0, 5, 0, 5);
		_delayPanel.add(_delaySlider, gridBagConstraints3);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		add(_delayPanel, gridBagConstraints1);
		
	}//GEN-END:initComponents
	
	/** Invoked when the next button is pressed.
	 *
	 * @param evt not even used.
	 */	
	private void _nextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__nextActionPerformed
		_animator.next();
	}//GEN-LAST:event__nextActionPerformed

	/** Invoked when the pause button is pressed.
	 *
	 * @param evt not even used.
	 */	
	private void _pauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__pauseActionPerformed
		_animator.pause();
	}//GEN-LAST:event__pauseActionPerformed

	/** Invoked when the play button is pressed.
	 *
	 * @param evt not even used.
	 */	
	private void _playActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__playActionPerformed
		_animator.start();
	}//GEN-LAST:event__playActionPerformed

	/** Invoked when the previous button is pressed.
	 *
	 * @param evt not even used.
	 */	
	private void _previousActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__previousActionPerformed
		_animator.previous();
	}//GEN-LAST:event__previousActionPerformed
	
	/** Invoked when the stop button is pressed.
	 *
	 * @param evt not even used.
	 */	
	private void _stopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__stopActionPerformed
		_animator.stop();
	}//GEN-LAST:event__stopActionPerformed
}

/* 
 * $Log: AnimationControlPanel.java,v $
 * Revision 1.2  2002/02/15 19:40:36  zcox
 * big time update
 *
 * Revision 1.1  2002/01/14 19:55:56  zcox
 * test
 *
 *
 *
 */
