/* $Id: AnimationEvent.java,v 1.2 2002/02/15 19:40:36 zcox Exp $
 *
 *
 */

package fcmodeler.animation;

import java.util.EventObject;

/** An event representing a change in an animation.  The animator controlling the animation can be
 * accessed through the <CODE>getAnimator</CODE> method, and the current frame can be accesed through
 * the <CODE>getFrame</CODE> method.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class AnimationEvent extends EventObject {
	private int _frame = -1;
	private Animator _animator = null;
	
	/** Creates a new <CODE>AnimationEvent</CODE> using the specified animator and frame number.
	 *
	 * @param animator the animator that caused this event to occur.
	 * @param frame the current frame number.
	 */	
	public AnimationEvent(Animator animator, int frame) {
		super(animator);
		if (animator == null) { throw new NullPointerException("animator was null"); }
		if (frame < -1) { throw new IllegalArgumentException("frame must be greater than -1"); }
		
		_animator = animator;
		_frame = frame;
	}
	
	/** Returns the animator that caused this event to occur.
	 *
	 * @return the animator that caused this event to occur.
	 */	
	public Animator getAnimator() {
		return _animator;
	}
	
	/** Returns the frame number of this event.
	 *
	 * @return the frame number of this event.
	 */	
	public int getFrame() {
		return _frame;
	}
}

/* 
 * $Log: AnimationEvent.java,v $
 * Revision 1.2  2002/02/15 19:40:36  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:41:02  zcox
 * initial commit of animation package
 *
 *
 *
 */
