/* $Id: AnimationListener.java,v 1.2 2002/02/15 19:40:36 zcox Exp $
 *
 *
 */

package fcmodeler.animation;

import java.util.EventListener;

/** The listener interface for receiving animation events.  The class that is interested in the state of
 * an animation implements this interface.  The object created with that class is then registered with
 * an animator using the animator's <CODE>addAnimationListener</CODE> method.  When the animation is
 * started, stopped, paused, or a new frame is shown, the relevant method in the listener object is
 * called, and the <CODE>AnimationEvent</CODE> is passed to it.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public interface AnimationListener extends EventListener {
	/** Invoked when the animation is paused.
	 *
	 * @param evt the object representing the pause.
	 */	
	public void animationPaused(AnimationEvent evt);
	
	/** Invoked when the animation is started.
	 *
	 * @param evt the object representing the start.
	 */	
	public void animationStarted(AnimationEvent evt);
	
	/** Invoked when the animation is stopped.
	 *
	 * @param evt the object representing the stop.
	 */	
	public void animationStopped(AnimationEvent evt);
	
	/** Invoked when a new frame is shown by the animation.
	 *
	 * @param evt the object representing the new frame event.
	 */	
	public void frameShown(AnimationEvent evt);
}

/* 
 * $Log: AnimationListener.java,v $
 * Revision 1.2  2002/02/15 19:40:36  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:41:02  zcox
 * initial commit of animation package
 *
 *
 *
 */
