/* $Id: ColorAttribute.java,v 1.2 2002/02/15 19:40:56 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

/** The color visual attribute.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class ColorAttribute extends Object implements VisualAttribute {
	private static final ColorAttribute _instance = new ColorAttribute();
	private static final String _string = "color";
	
	/** This constructor is private so that clients cannot instantiate this class.
	 *
	 */
	private ColorAttribute() {
		
	}
	
	/** Returns a color attribute editor.  The returned editor is guaranteed to be of type
	 * <CODE>ColorEditor</CODE>.
	 *
	 * @return a <CODE>ColorEditor</CODE>.
	 */
	public VisualAttributeEditor getEditor() {
		return new ColorEditor();
	}
	
	/** Returns an instance of <CODE>ColorAttribute</CODE>.
	 *
	 * @return an instance of <CODE>ColorAttribute</CODE>.
	 */	
	public static ColorAttribute getInstance() {
		return _instance;
	}
	
	/** Returns a color value, using the specified string description.  The returned value is guaranteed
	 * to be of type <CODE>ColorValue</CODE>.
	 *
	 * @param value the string description of the color value.
	 * @return a color value.
	 */
	public VisualAttributeValue getValue(String value) {
		return new ColorValue(value);
	}
	
	/** Returns the string "color".
	 *
	 * @return the string "color".
	 */	
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: ColorAttribute.java,v $
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 *
 *
 */
