/* $Id: ColorEditor.java,v 1.1 2002/02/15 19:54:35 zcox Exp $
 *
 *
 * Created on March 6, 2001, 6:24 PM
 */

package fcmodeler.attributes;

import javax.swing.JColorChooser;
import java.awt.Component;

/** An editor for the color visual attribute.  <CODE>ColorEditor</CODE> uses a
 * <CODE>javax.swing.JColorChooser</CODE> as its gui component and adds a <CODE>ColorFactoryPanel</CODE>
 * to the <CODE>JColorChoooser</CODE>.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class ColorEditor extends Object implements VisualAttributeEditor {
	private final JColorChooser _chooser;
	
	/** Creates a new <CODE>ColorEditor</CODE>.
	 *
	 */
	public ColorEditor() {
		_chooser = new JColorChooser();
		_chooser.addChooserPanel(new ColorFactoryPanel());
	}
	
	/** Returns the gui component used to edit the color visual attribute.  The returned component is
	 * guaranteed to be a <CODE>javax.swing.JColorChooser</CODE>.
	 *
	 * @return the gui component.
	 */	
	public Component getComponent() {
		return _chooser;
	}
	
	/** Returns the edited color value.  The returned value is guaranteed to be of type <CODE>ColorValue</CODE>.
	 *
	 * @return the edited color value.
	 */	
	public VisualAttributeValue getValue() {
		return new ColorValue(_chooser.getColor());
	}
}

/* 
 * $Log: ColorEditor.java,v $
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:28  zcox
 * initial commit of attributes package
 *
 * Revision 1.1  2001/04/04 02:07:32  zcox
 * initial commit of fcmodeler.view.propviz.attributes package
 *
 *
 *
 */
