/*
 * ConnectorEndEditor.java
 *
 * Created on April 12, 2002, 1:27 PM
 */

package fcmodeler.attributes;

import diva.canvas.connector.ConnectorEnd;

import fcmodeler.view.figures.ArrowheadExt;
import fcmodeler.view.figures.FilledRectangleHead;
import fcmodeler.view.figures.BlobCircleHeadExt;
import fcmodeler.view.figures.BlobDiamondHeadExt;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.EtchedBorder;

/**
 *
 * @author  xein
 */
public class ConnectorEndEditor extends JPanel implements VisualAttributeEditor {
  private JPanel panel;
  private JList list;
  private JScrollPane scrollPane;
  private ConnectorEnd _connectorEnd;

  private final ArrowheadExt _arrowhead;
  private final FilledRectangleHead _filledRectangleHead;
  private final BlobCircleHeadExt _blobCircleHead;
  private final BlobDiamondHeadExt _blobDiamondHead;

  /** Creates a new instance of <CODE>ConnectorEndEditor</CODE> */
  public ConnectorEndEditor() {
    _arrowhead = new ArrowheadExt();
    _filledRectangleHead = new FilledRectangleHead();
    _blobCircleHead = new BlobCircleHeadExt();
    _blobDiamondHead = new BlobDiamondHeadExt();

    initComponents();
    _connectorEnd = _arrowhead;
  }

  /** Returns the gui component used to edit a visual attribute value.
   *
   * @return the gui component.
   */
  public Component getComponent() {
    return this;
  }

  /** Returns the edited visual attribute value.  The returned value is guaranteed to be of type <CODE>ConnectorEndValue</CODE>.
   *
   * @return the edited visual attribute value.
   */
  public VisualAttributeValue getValue() {
    return new ConnectorEndValue(_connectorEnd);
  }

  private void initComponents() {
    setLayout(new java.awt.BorderLayout());

    // setup the panel
    panel = new JPanel();
		panel.setLayout(new GridLayout(1, 0));
		panel.setBorder(new EtchedBorder());

    // put all the connector ends into the Object array
    Object[] obj = new Object[4];
    obj[0] = _arrowhead.toString();
    obj[1] = _filledRectangleHead.toString();
    obj[2] = _blobCircleHead.toString();
    obj[3] = _blobDiamondHead.toString();

    // setup the list and scroll pane
    list = new JList(obj);
    list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    MouseListener mouseListener = new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if (list.getSelectedValue().equals("arrowhead")) {
          _connectorEnd = _arrowhead;
        }
        else if (list.getSelectedValue().equals("filled rectangle head")) {
          _connectorEnd = _filledRectangleHead;
        }
        else if (list.getSelectedValue().equals("circle head")) {
          _connectorEnd = _blobCircleHead;
        }
        else if (list.getSelectedValue().equals("diamond head")) {
          _connectorEnd = _blobDiamondHead;
        }
      }
    };
    list.addMouseListener(mouseListener);

    scrollPane = new JScrollPane(list);
    scrollPane.setAutoscrolls(true);

    panel.add(scrollPane);
    add(panel, BorderLayout.CENTER);
  }
}