/*
 * ConnectorShapeValue.java
 *
 * Created on April 11, 2002, 9:08 PM
 */

package fcmodeler.attributes;

import diva.canvas.connector.ConnectorEnd;

import fcmodeler.view.figures.ArrowheadExt;
import fcmodeler.view.figures.FilledRectangleHead;
import fcmodeler.view.figures.BlobCircleHeadExt;
import fcmodeler.view.figures.BlobDiamondHeadExt;
import fcmodeler.view.figures.ConnectorEndExt;
import fcmodeler.view.EdgeFigure;
import fcmodeler.attributes.EdgeVisualAttributeValue;

/**
 *
 * @author  xein
 */
public class ConnectorEndValue extends Object implements VisualAttributeValue, EdgeVisualAttributeValue {
  private final ConnectorEndExt _connectorEnd;


  /** Creates a new <CODE>ConnectorEndValue</CODE> using a <CODE>ArrowheadExt</CODE> as the default <CODE>ConnectorEnd</CODE>*/
  public ConnectorEndValue() {
    _connectorEnd = new ArrowheadExt();
  }


  /** Creates a new <CODE>ConnectorEndValue</CODE> using the specified connector end.  The supplied
	 * <CODE>ConnectorEnd</CODE> is cloned in order to create a defensive copy.
	 *
	 * @param connectorEnd the connector end to use for this connector end value.
	 */
  public ConnectorEndValue(ConnectorEnd connectorEnd) {
    try {
      _connectorEnd = (ConnectorEndExt)((ConnectorEndExt)connectorEnd).clone();
    }
    catch (CloneNotSupportedException e) {
			throw new UnsupportedOperationException(connectorEnd + " does not support the clone method - BUT IT SHOULD!!!");
		}
  }


  /** Creates a new <CODE>ConnectorEndValue</CODE> using the specified string description of the connector end.
	 *
	 * @param connectorEnd the string description of the connector end for this connector end value.
	 */
  public ConnectorEndValue(String connectorEnd) {
    if (connectorEnd.equals("arrowhead")) {
      _connectorEnd = new ArrowheadExt();
    }
    else if (connectorEnd.equals("filled rectangle head")) {
      _connectorEnd = new FilledRectangleHead();
    }
    else if (connectorEnd.equals("circle head")) {
      _connectorEnd = new BlobCircleHeadExt();
    }
    else if (connectorEnd.equals("diamond head")) {
      _connectorEnd = new BlobDiamondHeadExt();
    }
    else {
			throw new IllegalArgumentException(connectorEnd + " is not a valid connector end");
		}
  }


  /** Compares the specified object with this visual attribute value for equality.
   *
   * @param o the object to be compared for equality with this visual attribute value.
   * @return <CODE>true</CODE> if the specified object is equal to this visual attribute value.
   */
  public boolean equals(Object o) {
    if (o == this)
			return true;

		if (!(o instanceof ConnectorEndValue))
			return false;

		ConnectorEndValue other = (ConnectorEndValue) o;
		return _connectorEnd == other.getConnectorEnd();
  }


  public VisualAttribute getAttribute() {
    return ConnectorEndAttribute.getInstance();
  }


  /** Returns the hash code for this visual attribute value.
   *
   * @return the hash code for this visual attribute value.
   */
  public int hashCode() {
    int result = 17;
		result = 37 * result + _connectorEnd.hashCode();
		return result;
  }


  /** Returns a string representation of this visual attribute value.
   *
   * @return a string representation of this visual attribute value.
   */
  public String toString() {
    return _connectorEnd.toString();
  }


  /** Returns the connector end of this connector end value.
	 * @return the connector end of this connector end value.
	 */
  public ConnectorEnd getConnectorEnd() {
    return _connectorEnd;
  }


  /** Sets the visual attribute of the specified edge figure to the value contained in this visual
	 * attribute value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setEdgeValue(EdgeFigure figure) {
    ConnectorEnd clone = null;
    try {
      clone = (ConnectorEnd)_connectorEnd.clone();
    }
    catch (CloneNotSupportedException e) {
      throw new UnsupportedOperationException(clone + " does not support the clone method - BUT IT SHOULD!!!");
    }
    figure.setHeadEnd(clone);
  }

	/** Sets the permanent visual attribute of the specified edge figure to the value contained in
	 * this visual attribute value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setPermanentEdgeValue(EdgeFigure figure) {
    ConnectorEnd clone = null;
    try {
      clone = (ConnectorEnd)_connectorEnd.clone();
    }
    catch (CloneNotSupportedException e) {
      throw new UnsupportedOperationException(clone + " does not support the clone method - BUT IT SHOULD!!!");
    }
    figure.setHeadEnd(clone);
  }
}
