/* $Id: DashAttribute.java,v 1.2 2002/02/15 19:40:56 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

/** The dash visual attribute.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class DashAttribute extends Object implements VisualAttribute {
	private static final DashAttribute _instance = new DashAttribute();
	private static final String _string = "dash";
	
	/** This constructor is private so that clients cannot instantiate this class.
	 *
	 */
	private DashAttribute() {
		
	}
	
	/** Returns a dash attribute editor.  The returned editor is guaranteed to be of type
	 * <CODE>DashEditor</CODE>.
	 *
	 * @return a <CODE>DashEditor</CODE>.
	 */
	public VisualAttributeEditor getEditor() {
		return new DashEditor();
	}
	
	/** Returns an instance of <CODE>DashAttribute</CODE>.
	 *
	 * @return an instance of <CODE>DashAttribute</CODE>.
	 */	
	public static DashAttribute getInstance() {
		return _instance;
	}
	
	/** Returns a dash value, using the specified string description.  The returned value is guaranteed
	 * to be of type <CODE>DashValue</CODE>.
	 *
	 * @param value the string description of the dash value.
	 * @return a dash value.
	 */
	public VisualAttributeValue getValue(String value) {
		return new DashValue(value);
	}
	
	/** Returns the string "dash".
	 *
	 * @return the string "dash".
	 */
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: DashAttribute.java,v $
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 *
 *
 */
