/* $Id: DefaultEdgeValues.java,v 1.2 2002/04/21 20:47:54 xein Exp $
 *
 *
 */

package fcmodeler.attributes;

import java.awt.Color;

/** A factory for default edge visual attribute values.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class DefaultEdgeValues extends Object {

	/** This constructor is private so clients never instantiate this class.
	 *
	 */
	private DefaultEdgeValues() {

	}

	/** Returns the black color value.
	 *
	 * @return the black color value.
	 */
	public static ColorValue getColor() {
		return new ColorValue(Color.black);
	}

	/** Returns the dash value <CODE>[0.0, 0.0]</CODE>.
	 *
	 * @return the dash value <CODE>[0.0, 0.0]</CODE>.
	 */
	public static DashValue getDash() {
		//all zeros represents a solid line
		return new DashValue(new float[] {0.0f, 0.0f});
	}

  public static ConnectorEndValue getConnectorEnd() {
    return new ConnectorEndValue("arrowhead");
  }

	/** Returns the default value of the specified visual attribute.
	 *
	 * @param attribute the visual attribute.
	 * @return the default value of the visual attribute.
	 */
	public static EdgeVisualAttributeValue getDefault(VisualAttribute attribute) {
		if (attribute instanceof ColorAttribute) {
			return getColor();
		}
		else if (attribute instanceof DashAttribute) {
			return getDash();
		}
		else if (attribute instanceof LineWidthAttribute) {
			return getLineWidth();
		}
    else if (attribute instanceof ConnectorEndAttribute) {
      return getConnectorEnd();
    }
		else {
			throw new IllegalArgumentException("attribute is not an edge visual attribute: " + attribute);
		}
	}

	/** Returns the default value for the visual attribute of the specified visual attribute value.
	 *
	 * @param value the visual attribute value.
	 * @return the default visual attribute value.
	 */
	public static EdgeVisualAttributeValue getDefault(VisualAttributeValue value) {
		return getDefault(value.getAttribute());
	}

	/** Returns the line width value <CODE>1.0</CODE>.
	 *
	 * @return the line width value <CODE>1.0</CODE>.
	 */
	public static LineWidthValue getLineWidth() {
		return new LineWidthValue(1.0f);
	}

}

/*
 * $Log: DefaultEdgeValues.java,v $
 * Revision 1.2  2002/04/21 20:47:54  xein
 * - added ConnectorEnd's default value and attribute
 *
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 *
 *
 */
