/* $Id: DefaultNodeValues.java,v 1.1 2002/02/15 19:54:35 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

import java.awt.Color;

import fcmodeler.view.figures.RectangleNodeShape;

/** A factory for default node visual attribute values.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class DefaultNodeValues {
	
	/** This constructor is private so clients never instantiate this class.
	 *
	 */
	private DefaultNodeValues() {
		
	}
	
	/** Returns the color value black.
	 *
	 * @return the color value black.
	 */	
	public static ColorValue getColor() {
		return new ColorValue(Color.black);
	}
	
	/** Returns the fill value white.
	 *
	 * @return the fill value white.
	 */	
	public static FillValue getFill() {
		return new FillValue(Color.white);
	}
	
	/** Returns the default value of the specified visual attribute.
	 *
	 * @param attribute the visual attribute.
	 * @return the default value of the visual attribute.
	 */	
	public static NodeVisualAttributeValue getDefault(VisualAttribute attribute) {
		if (attribute instanceof ColorAttribute) {
			return getColor();
		}
		else if (attribute instanceof FillAttribute) {
			return getFill();
		}
		else if (attribute instanceof NodeShapeAttribute) {
			return getNodeShape();
		}
		else {
			throw new IllegalArgumentException("attribute is not a node visual attribute: " + attribute);
		}
	}
	
	/** Returns the default value for the visual attribute of the specified visual attribute value.
	 *
	 * @param value the visual attribute value.
	 * @return the default visual attribute value.
	 */	
	public static NodeVisualAttributeValue getDefault(VisualAttributeValue value) {
		return getDefault(value.getAttribute());
	}
	
	/** Returns the rectangle node shape value.
	 *
	 * @return the rectangle node shape value.
	 */	
	public static NodeShapeValue getNodeShape() {
		return new NodeShapeValue(new RectangleNodeShape());
	}
	
}

/* 
 * $Log: DefaultNodeValues.java,v $
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 *
 *
 */
