/* $Id: EditorDialog.java,v 1.1 2002/02/15 19:54:35 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

import javax.swing.JDialog;
import java.awt.Dimension;

/** A convenience class for showing a visual attribute editor in a dialog.
 *
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class EditorDialog extends JDialog {
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel _options;
	private javax.swing.JButton _ok;
	private javax.swing.JButton _cancel;
	// End of variables declaration//GEN-END:variables
	private final VisualAttributeEditor _editor;
	private VisualAttributeValue _value = null;
	
	/** Creates a new <CODE>EditorDialog</CODE> using the specifed editor.
	 *
	 * @param editor the editor to use in this editor dialog.
	 */	
	public EditorDialog(VisualAttributeEditor editor) {
		super(new javax.swing.JFrame(), true);
		initComponents();
		getRootPane().setDefaultButton(_ok);
		
		_editor = editor;
		getContentPane().add(_editor.getComponent());
	}

	/** Closes the dialog.
	 *
	 * @param evt the window event.
	 */
	private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
		setVisible(false);
		dispose();
	}//GEN-LAST:event_closeDialog
	
	/** Returns the visual attribute editor displayed in this editor dialog.
	 *
	 * @return the editor displayed in this editor dialog.
	 */	
	public VisualAttributeEditor getEditor() {
		return _editor;
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	private void initComponents() {//GEN-BEGIN:initComponents
		_options = new javax.swing.JPanel();
		_ok = new javax.swing.JButton();
		_cancel = new javax.swing.JButton();
		
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});
		
		_options.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		
		_options.setBorder(new javax.swing.border.EtchedBorder());
		_ok.setText("ok");
		_ok.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_okActionPerformed(evt);
			}
		});
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(5, 5, 5, 5);
		_options.add(_ok, gridBagConstraints1);
		
		_cancel.setText("cancel");
		_cancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				_cancelActionPerformed(evt);
			}
		});
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(5, 0, 5, 5);
		_options.add(_cancel, gridBagConstraints1);
		
		getContentPane().add(_options, java.awt.BorderLayout.SOUTH);
		
		pack();
	}//GEN-END:initComponents

	/** Called when the user clicks the cancel button.
	 *
	 * @param evt the action event.
	 */	
	private void _cancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__cancelActionPerformed
		_value = null;
		closeDialog(null);
	}//GEN-LAST:event__cancelActionPerformed

	/** Called when the user clicks the ok button.
	 *
	 * @param evt the action event.
	 */	
	private void _okActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__okActionPerformed
		_value = _editor.getValue();
		closeDialog(null);
	}//GEN-LAST:event__okActionPerformed
	
	/** Shows the editor dialog and returns the visual attribute value created by the user.
	 *
	 * @return the visual attriubte value created by the user.
	 */	
	public VisualAttributeValue showEditor() {
		Dimension component = _editor.getComponent().getPreferredSize();
		Dimension buttons = _options.getPreferredSize();
		setSize(Math.max(component.width, buttons.width), component.height + buttons.height + 30);
		show();
		
		return _value;
	}
	
}

/* 
 * $Log: EditorDialog.java,v $
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 *
 *
 */
