/* $Id: FillAttribute.java,v 1.2 2002/02/15 19:40:56 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

/** The fill visual attribute.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class FillAttribute extends Object implements VisualAttribute {
	private static final FillAttribute _instance = new FillAttribute();
	private static final String _string = "fill";
	
	/** This constructor is private so that clients cannot instantiate this class.
	 *
	 */
	private FillAttribute() {
		
	}
	
	/** Returns a fill attribute editor.  The returned editor is guaranteed to be of type
	 * <CODE>FillEditor</CODE>.
	 *
	 * @return a <CODE>FillEditor</CODE>.
	 */
	public VisualAttributeEditor getEditor() {
		return new FillEditor();
	}
	
	/** Returns an instance of <CODE>FillAttribute</CODE>.
	 * @return an instance of <CODE>FillAttribute</CODE>.
	 */	
	public static FillAttribute getInstance() {
		return _instance;
	}
	
	/** Returns a fill value, using the specified string description.  The returned value is guaranteed
	 * to be of type <CODE>FillValue</CODE>.
	 *
	 * @param value the string description of the fill value.
	 * @return a fill value.
	 */
	public VisualAttributeValue getValue(String value) {
		return new FillValue(value);
	}
	
	/** Returns the string "fill".
	 *
	 * @return the string "fill".
	 */
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: FillAttribute.java,v $
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 *
 *
 */
