/* $Id: FillEditor.java,v 1.1 2002/02/15 19:54:35 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

import javax.swing.JColorChooser;
import java.awt.Component;

/** An editor for the fill visual attribute.  <CODE>FillEditor</CODE> uses a
 * <CODE>javax.swing.JColorChooser</CODE> as its gui component and adds a <CODE>ColorFactoryPanel</CODE>
 * to the <CODE>JColorChooser</CODE>.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class FillEditor extends Object implements VisualAttributeEditor {
	private final JColorChooser _chooser;
	
	/** Creates a new <CODE>FillEditor</CODE>.
	 *
	 */
	public FillEditor() {
		_chooser = new JColorChooser();
		_chooser.addChooserPanel(new ColorFactoryPanel());
	}
	
	/** Returns the gui component used to edit the fill visual attribute.  The returned component is
	 * guaranteed to be a <CODE>javax.swing.JColorChooser</CODE>.
	 *
	 * @return the gui component.
	 */	
	public Component getComponent() {
		return _chooser;
	}
	
	/** Returns the edited fill value.  The returned value is guaranteed to be of type <CODE>FillValue</CODE>.
	 *
	 * @return the edited fill value.
	 */	
	public VisualAttributeValue getValue() {
		return new FillValue(_chooser.getColor());
	}
}

/* 
 * $Log: FillEditor.java,v $
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:29  zcox
 * initial commit of attributes package
 *
 *
 *
 */
