/* $Id: LineWidthAttribute.java,v 1.1 2002/02/15 19:54:36 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

/** The line width visual attribute.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class LineWidthAttribute extends Object implements VisualAttribute {
	private static final LineWidthAttribute _instance = new LineWidthAttribute();
	private static final String _string = "line width";
	
	/** This constructor is private so that clients cannot instantiate this class.
	 *
	 */
	private LineWidthAttribute() {
		
	}
	
	/** Returns a line width attribute editor.  The returned editor is guaranteed to be of type
	 * <CODE>LineWidthEditor</CODE>.
	 *
	 * @return a <CODE>LineWidthEditor</CODE>.
	 */
	public VisualAttributeEditor getEditor() {
		return new LineWidthEditor();
	}
	
	/** Returns an instance of <CODE>LineWidthAttribute</CODE>.
	 *
	 * @return an instance of <CODE>LineWidthAttribute</CODE>.
	 */	
	public static LineWidthAttribute getInstance() {
		return _instance;
	}
	
	/** Returns a line width value, using the specified string description.  The returned value is
	 * guaranteed to be of type <CODE>LineWidthValue</CODE>.
	 *
	 * @param value the string description of the line width value.
	 * @return a line width value.
	 */
	public VisualAttributeValue getValue(String value) {
		return new LineWidthValue(value);
	}
	
	/** Returns the string "line width".
	 *
	 * @return the string "line width".
	 */
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: LineWidthAttribute.java,v $
 * Revision 1.1  2002/02/15 19:54:36  zcox
 * big time update
 *
 *
 *
 */
