/* $Id: LineWidthValue.java,v 1.1 2002/02/15 19:54:36 zcox Exp $
 *
 *
 * Created on April 9, 2001, 2:29 PM
 */

package fcmodeler.attributes;

import fcmodeler.view.EdgeFigure;

/** A specific value of the line width attribute.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class LineWidthValue extends Object implements VisualAttributeValue, EdgeVisualAttributeValue {
	private final float _width;
	
	/** Creates a new <CODE>LineWidthValue</CODE> with a width of 1.0.
	 *
	 */
	public LineWidthValue() {
		_width = 1.0f;
	}
	
	/** Creates a new <CODE>LineWidthValue</CODE> using the specified line width.
	 *
	 * @param lineWidth the line width of this line width value.
	 */	
	public LineWidthValue(float lineWidth) {
		_width = lineWidth;
	}
	
	/** Creates a new <CODE>LineWidthValue</CODE> using the specified string description.  The string
	 * should just be a floating-point number, like "2.345".
	 *
	 * @param lineWidth string description of the line width of this line width value.
	 */	
	public LineWidthValue(String lineWidth) {
		_width = Float.parseFloat(lineWidth);
	}
	
	/** Compares the specified object with this line width value for equality.
	 *
	 * @param o the object to be compared for equality with this line width value.
	 * @return <CODE>true</CODE> if the specified object is equal to this line width value.
	 */
	public boolean equals(Object o) {
		if (o == this) 
			return true;
		
		if (!(o instanceof LineWidthValue))
			return false;
		
		LineWidthValue other = (LineWidthValue) o;
		return _width == other.getLineWidth();
	}
	
	public VisualAttribute getAttribute() {
		return LineWidthAttribute.getInstance();
	}
	
	/** Returns the width of this line width value.
	 *
	 * @return the width of this line width value.
	 */	
	public float getLineWidth() {
		return _width;
	}
	
	/** Returns the hash code for this line width value.
	 *
	 * @return the hash code for this line width value.
	 */
	public int hashCode() {
		int result = 17;
		
		result = 37*result + Float.floatToIntBits(_width);
		
		return result;
	}
	
	/** Sets the line width of the specified edge figure to the width contained in this line width value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setEdgeValue(EdgeFigure figure) {
		figure.getFigure().setLineWidth(_width);
	}
	
	/** Sets the permanent line width of the specified edge figure to the width contained in this line width value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setPermanentEdgeValue(EdgeFigure figure) {
		figure.getPermanentFigure().setLineWidth(_width);
	}
	
	/** Returns a string representation of this line width value.
	 *
	 * @return a string representation of this line width value.
	 */
	public String toString() {
		return "" + _width;
	}
	
	/** Returns a new <CODE>LineWidthValue</CODE> instance, created using the specified string description.
	 *
	 * @param value the string description to be used in creating the new <CODE>LineWidthValue</CODE> instance.
	 * @return the new <CODE>LineWidthValue</CODE> instance.
	 */
	public static LineWidthValue valueOf(String value) {
		return new LineWidthValue(value);
	}
	
}

/* 
 * $Log: LineWidthValue.java,v $
 * Revision 1.1  2002/02/15 19:54:36  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:29  zcox
 * initial commit of attributes package
 *
 * Revision 1.2  2001/05/22 16:54:59  zcox
 * code clean-up
 *
 *
 *
 *
 */
