/* $Id: NodeShapeAttribute.java,v 1.2 2002/02/15 19:40:56 zcox Exp $
 *
 *
 */

package fcmodeler.attributes;

/** The node shape visual attribute.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class NodeShapeAttribute extends Object implements VisualAttribute {
	private static final NodeShapeAttribute _instance = new NodeShapeAttribute();
	private static final String _string = "node shape";
	
	/** This constructor is private so that clients cannot instantiate this class.
	 *
	 */
	private NodeShapeAttribute() {
		
	}
	
	/** Returns a node shape attribute editor.  The returned editor is guaranteed to be of type
	 * <CODE>NodeShapeEditor</CODE>.
	 *
	 * @return a <CODE>NodeShapeEditor</CODE>.
	 */
	public VisualAttributeEditor getEditor() {
		return new NodeShapeEditor();
	}
	
	/** Returns an instance of <CODE>NodeShapeAttribute</CODE>.
	 *
	 * @return an instance of <CODE>NodeShapeAttribute</CODE>.
	 */	
	public static NodeShapeAttribute getInstance() {
		return _instance;
	}
	
	/** Returns a node shape value, using the specified string description.  The returned value is
	 * guaranteed to be of type <CODE>NodeShapeValue</CODE>.
	 *
	 * @param value the string description of the node shape value.
	 * @return a node shape value.
	 */
	public VisualAttributeValue getValue(String value) {
		return new NodeShapeValue(value);
	}
	
	/** Returns the string "node shape".
	 *
	 * @return the string "node shape".
	 */
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: NodeShapeAttribute.java,v $
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 *
 *
 */
