/* $Id: NodeShapeEditor.java,v 1.3 2002/04/11 22:36:22 xein Exp $
 *
 *
 * Created on March 29, 2001, 9:02 PM
 */

package fcmodeler.attributes;

import java.awt.Component;
import javax.swing.Icon;
import javax.swing.JPanel;

import fcmodeler.view.figures.NodeShape;
import fcmodeler.view.figures.RectangleNodeShape;
import fcmodeler.view.figures.RoundRectangleNodeShape;
import fcmodeler.view.figures.EllipseNodeShape;
import fcmodeler.view.figures.TestNodeShape;

import fcmodeler.view.figures.DiamondNodeShape; //xein

/** An editor for the node shape visual attribute.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public final class NodeShapeEditor extends JPanel implements VisualAttributeEditor {
	// Variables declaration - do not modify
	private javax.swing.JPanel jPanel1;
	private javax.swing.JButton jbtnRectangle;
	private javax.swing.JButton jbtnRoundRectangle;
	private javax.swing.JButton jbtnEllipse;
	private javax.swing.JButton jbtnTest;
        private javax.swing.JButton jbtnDiamond; //xein
	// End of variables declaration

        private NodeShape _nodeShape = null;
	private final NodeShape _rectangle;
	private final NodeShape _roundRectangle;
	private final NodeShape _ellipse;
	private final NodeShape _test;
        private final NodeShape _diamond; //xein


	/** Creates a new <CODE>NodeShapeEditor</CODE>.
	 *
	 */
	public NodeShapeEditor() {
		initComponents ();

		_rectangle = new RectangleNodeShape();
		_roundRectangle = new RoundRectangleNodeShape();
		_ellipse = new EllipseNodeShape();
		_test = new TestNodeShape();
		_diamond = new DiamondNodeShape(); //xein

		jbtnRectangle.setIcon(getNodeShapeIcon(_rectangle));
		jbtnRoundRectangle.setIcon(getNodeShapeIcon(_roundRectangle));
		jbtnEllipse.setIcon(getNodeShapeIcon(_ellipse));
		jbtnTest.setIcon(getNodeShapeIcon(_test));
                jbtnDiamond.setIcon(getNodeShapeIcon(_diamond)); //xein
		_nodeShape = _rectangle;
	}

	/** Returns the gui component used to edit the node shape visual attribute.
	 *
	 * @return the gui component.
	 */
	public Component getComponent() {
		return this;
	}

	/** Returns a node shape icon for the specified node shape.
	 *
	 * @param shape the node shape.
	 * @return a node shape icon for the specified node shape.
	 */
	private Icon getNodeShapeIcon(NodeShape shape) {
		return new NodeShapeIcon(shape);
	}

	/** Returns the edited node shape value.  The returned value is guaranteed to be of type
	 * <CODE>NodeShapeValue</CODE>.
	 *
	 * @return the edited node shape value.
	 */
	public VisualAttributeValue getValue() {
		return new NodeShapeValue(_nodeShape);
	}

	/** This method is called from within the constructor to
	* initialize the form.
	* WARNING: Do NOT modify this code. The content of this method is
	* always regenerated by the FormEditor.
	*/
	private void initComponents() {
		jPanel1 = new javax.swing.JPanel();
		jbtnRectangle = new javax.swing.JButton();
		jbtnRoundRectangle = new javax.swing.JButton();
		jbtnEllipse = new javax.swing.JButton();
		jbtnTest = new javax.swing.JButton();
                jbtnDiamond = new javax.swing.JButton(); //xein

		setLayout(new java.awt.BorderLayout());

		jPanel1.setLayout(new java.awt.GridLayout(1, 0));

		jPanel1.setBorder(new javax.swing.border.EtchedBorder());
		jbtnRectangle.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbtnRectangleActionPerformed(evt);
			}
		});

		jPanel1.add(jbtnRectangle);

		jbtnRoundRectangle.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbtnRoundRectangleActionPerformed(evt);
			}
		});

		jPanel1.add(jbtnRoundRectangle);

		jbtnEllipse.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbtnEllipseActionPerformed(evt);
			}
		});

		jPanel1.add(jbtnEllipse);

		jbtnTest.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbtnTestActionPerformed(evt);
			}
		});

		jPanel1.add(jbtnTest);

                // xein
                jbtnDiamond.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jbtnDiamondActionPerformed(evt);
			}
		});

		jPanel1.add(jbtnDiamond);
                // xein

		add(jPanel1, java.awt.BorderLayout.CENTER);

	}


	/** Called when the user clicks the ellipse node shape button.
	 *
	 * @param evt the action event.
	 */
	private void jbtnEllipseActionPerformed(java.awt.event.ActionEvent evt) {
		_nodeShape = _ellipse;
	}

	/** Called when the user clicks the rectangle node shape button.
	 *
	 * @param evt the action event.
	 */
	private void jbtnRectangleActionPerformed(java.awt.event.ActionEvent evt) {
		_nodeShape = _rectangle;
	}

	/** Called when the user clicks the round rectangle node shape button.
	 *
	 * @param evt the action event.
	 */
	private void jbtnRoundRectangleActionPerformed(java.awt.event.ActionEvent evt) {
		_nodeShape = _roundRectangle;
	}

	/** Called when the user clicks the test node shape button.
	 *
	 * @param evt the action event.
	 */
	private void jbtnTestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnTestActionPerformed
		_nodeShape = _test;
	}//GEN-LAST:event_jbtnTestActionPerformed

        //xein
        /** Called when the user clicks the diamond node shape button.
	 *
	 * @param evt the action event.
	 */
	private void jbtnDiamondActionPerformed(java.awt.event.ActionEvent evt) {
		_nodeShape = _diamond;
	}
}

/** An internal class representing an icon to put on a node shape button.
 *
 */
class NodeShapeIcon extends Object implements javax.swing.Icon {
	int width = 0, height = 0;
	private java.awt.image.BufferedImage image = null;

	/** Creates a new <CODE>NodeShapeIcon</CODE> using the specified node shape.
	 *
	 * @param nodeShape the node shape to use in this icon.
	 */
	public NodeShapeIcon(NodeShape nodeShape) {
		diva.canvas.toolbox.LabelFigure label = new diva.canvas.toolbox.LabelFigure("node label");
		label.setFont(new java.awt.Font("Serif", java.awt.Font.ITALIC, 16));
		label.translate(20, 30);
		nodeShape.enclose(label.getBounds());
		width = nodeShape.getShape().getBounds().width + 20;
		height = nodeShape.getShape().getBounds().height + 20;

		image = new java.awt.image.BufferedImage(width, height, java.awt.image.BufferedImage.TYPE_INT_RGB);
		java.awt.Graphics2D g2 = image.createGraphics();
		g2.setBackground(java.awt.Color.white);
		g2.clearRect(0, 0, width, height);
		g2.setPaint(java.awt.Color.white);
		g2.fill(nodeShape.getShape());
		g2.setPaint(java.awt.Color.black);
		g2.draw(nodeShape.getShape());
		label.paint(g2);
	}

	/** Returns the icon's height.
	 *
	 * @return the icon's height.
	 */
	public int getIconHeight() {
		return height;
	}

	/** Returns the icon's width.
	 *
	 * @return the icon's width.
	 */
	public int getIconWidth() {
		return width;
	}

	/** Paints the icon at the specified location.
	 *
	 * @param c the underlying component.
	 * @param g the graphics context to paint to.
	 * @param x the x-location to paint to.
	 * @param y the y-location to paint to.
	 */
	public void paintIcon(java.awt.Component c, java.awt.Graphics g, int x, int y) {
		java.awt.Graphics2D g2 = (java.awt.Graphics2D) g;

		g2.drawImage(image, x, y, c);
	}
}

/*
 * $Log: NodeShapeEditor.java,v $
 * Revision 1.3  2002/04/11 22:36:22  xein
 * no message
 *
 * Revision 1.2  2002/03/22 16:54:30  xein
 * - Changed NodeShapeEditor and NodeShapeValue to include   DiamondNodeShape
 * - BezierEdgeFigureRenderer was used to test FilledRectangleHead connector   end
 * - No change to RectangleNodeShape
 *
 * Revision 1.1  2002/02/15 19:54:36  zcox
 * big time update
 *
 * Revision 1.2  2002/01/18 19:53:30  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.1  2002/01/15 19:51:29  zcox
 * initial commit of attributes package
 *
 * Revision 1.3  2001/05/22 16:46:49  zcox
 * code clean-up
 *
 * Revision 1.2  2001/04/05 03:00:49  zcox
 * made NodeShape a real flyweight
 *
 * Revision 1.1  2001/04/04 02:07:32  zcox
 * initial commit of fcmodeler.view.propviz.attributes package
 *
 *
 *
 */
