/* $Id: VisualAttribute.java,v 1.2 2002/02/15 19:40:56 zcox Exp $
 *
 *
 * Created on March 5, 2001, 10:08 PM
 */

package fcmodeler.attributes;

/** A visual attribute.  This interface specifies two factory methods: one for a visual attribute
 * editor and another for a visual attribute value.  A client can thus retrieve both an editor and
 * a value for the specific visual attribute represented by a concrete implementation, without
 * needing to know the specific type of the implementation.  Implementations of
 * <CODE>VisualAttribute</CODE> are encouraged to be singletons, as no intrinsic state should be
 * necessary.  If an implementation is a singleton, it should provide a static factory method called
 * <CODE>getInstance</CODE>.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public interface VisualAttribute {
	/** Returns a visual attribute editor, appropriate for editing values of this type of visual attribute.
	 *
	 * @return an editor appropriate for editing values of this type of visual attribute.
	 */	
	public VisualAttributeEditor getEditor();
	
	/** Returns a value of this type of visual attribute, using the specified string description.
	 *
	 * @param value the string description of the visual attribute value.
	 * @return a value of this type of visual attribute.
	 */	
	public VisualAttributeValue getValue(String value);
	
	/** Returns a string representation of this visual attribute.
	 *
	 * @return a string representation of this visual attribute.
	 */	
	public String toString();
}

/* 
 * $Log: VisualAttribute.java,v $
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:30  zcox
 * initial commit of attributes package
 *
 * Revision 1.1  2001/04/04 02:07:32  zcox
 * initial commit of fcmodeler.view.propviz.attributes package
 *
 *
 *
 */
