/* $Id: VisualAttributeFactory.java,v 1.3 2002/04/21 20:44:26 xein Exp $
 *
 *
 */

package fcmodeler.attributes;

import java.util.Collection;
import java.util.Map;
import java.util.HashMap;

/** A factory for visual attributes.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public final class VisualAttributeFactory extends Object {
	private static final VisualAttributeFactory _instance = new VisualAttributeFactory();
	private final Map _attributes;
	private final Map _nodeAttributes;
	private final Map _edgeAttributes;

	/** This constructor is private so that clients cannot instantiate this class.
	 *
	 */
	private VisualAttributeFactory() {
		ColorAttribute ca = ColorAttribute.getInstance();
		DashAttribute da = DashAttribute.getInstance();
		FillAttribute fa = FillAttribute.getInstance();
		LineWidthAttribute lwa = LineWidthAttribute.getInstance();
		NodeShapeAttribute nsa = NodeShapeAttribute.getInstance();
    ConnectorEndAttribute cea = ConnectorEndAttribute.getInstance(); //xein

		_attributes = new HashMap();
		_attributes.put(ca.toString(), ca);
		_attributes.put(da.toString(), da);
		_attributes.put(fa.toString(), fa);
		_attributes.put(lwa.toString(), lwa);
		_attributes.put(nsa.toString(), nsa);
    _attributes.put(cea.toString(), cea); //xein

		_nodeAttributes = new HashMap();
		_nodeAttributes.put(ca.toString(), ca);
		_nodeAttributes.put(fa.toString(), fa);
		_nodeAttributes.put(nsa.toString(), nsa);

		_edgeAttributes = new HashMap();
		_edgeAttributes.put(ca.toString(), ca);
		_edgeAttributes.put(da.toString(), da);
		_edgeAttributes.put(lwa.toString(), lwa);
    _edgeAttributes.put(cea.toString(), cea); //xein
	}

	/** Returns the visual attribute represented by the specified string.
	 *
	 * @param attribute the string representation of the visual attribute.
	 * @return the visual attribute represented by the specified string.
	 */
	public VisualAttribute getAttribute(String attribute) {
		return (VisualAttribute) _attributes.get(attribute);
	}

	/** Returns the collection of edge visual attributes.
	 *
	 * @return the collection of edge visual attributes.
	 */
	public Collection getEdgeAttributes() {
		return _edgeAttributes.values();
	}

	/** Returns an instance of <CODE>VisualAttributeFactory</CODE>.
	 *
	 * @return an instance of <CODE>VisualAttributeFactory</CODE>.
	 */
	public static VisualAttributeFactory getInstance() {
		return _instance;
	}

	/** Returns the collection of node visual attributes.
	 *
	 * @return the collection of edge visual attributes.
	 */
	public Collection getNodeAttributes() {
		return _nodeAttributes.values();
	}

	/** Returns the collection of all visual attriubutes.
	 *
	 * @return the collection of all visual attributes.
	 */
	public Collection getVisualAttributes() {
		return _attributes.values();
	}

}

/*
 * $Log: VisualAttributeFactory.java,v $
 * Revision 1.3  2002/04/21 20:44:26  xein
 * - added ConnectorEndAttribute
 *
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:30  zcox
 * initial commit of attributes package
 *
 *
 *
 */
