/* $Id: VisualAttributeValue.java,v 1.2 2002/02/15 19:40:56 zcox Exp $
 *
 *
 * Created on March 6, 2001, 3:13 PM
 */

package fcmodeler.attributes;

import diva.canvas.Figure;

/** A specific value of a visual attribute.  Implementations of this interface should provide three
 * constructors: a default constructor, one with class-specific parameters, and one that accepts a
 * String representation.  In addition, implemenations should provide accessors for any relevant
 * internal fields and a static factory method that accepts a String representation and returns an
 * appropriate instance.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public interface VisualAttributeValue {
	/** Compares the specified object with this visual attribute value for equality.
	 *
	 * @param o the object to be compared for equality with this visual attribute value.
	 * @return <CODE>true</CODE> if the specified object is equal to this visual attribute value.
	 */	
	public boolean equals(Object o);
	
	public VisualAttribute getAttribute();
	
	/** Returns the hash code for this visual attribute value.
	 *
	 * @return the hash code for this visual attribute value.
	 */	
	public int hashCode();
	
	/** Returns a string representation of this visual attribute value.
	 *
	 * @return a string representation of this visual attribute value.
	 */	
	public String toString();
}

/* 
 * $Log: VisualAttributeValue.java,v $
 * Revision 1.2  2002/02/15 19:40:56  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:30  zcox
 * initial commit of attributes package
 *
 * Revision 1.1  2001/04/04 02:07:32  zcox
 * initial commit of fcmodeler.view.propviz.attributes package
 *
 *
 *
 */
