/* $Id: BezierConnectorHighlighter.java,v 1.2 2002/06/28 08:06:03 adamtj Exp $
 *
 */

package fcmodeler.interaction.BezierConnector;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.geom.GeneralPath;
import java.util.List;

import diva.canvas.Figure;
import diva.canvas.FigureDecorator;

import fcmodeler.interaction.GeneralConnector.GeneralConnector;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorHighlighter;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorInteractor;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorSite;

/**
 * BezierConnectorHighlighter is a GeneralConnectorHighlighter that draws extra
 * lines connecting the control points of a bezier curve to the end point of a curve
 * segment.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class BezierConnectorHighlighter extends GeneralConnectorHighlighter {
	/** the Stroke to use for drawing the connecting lines */
	protected Stroke _stroke = new BasicStroke(0.5f);
	/** the Paint to use for drawing the connecting lines */
	protected Paint _paint = Color.gray;
	
	public BezierConnectorHighlighter() {
		super();
		setCustomHandleInteractor((GeneralConnectorInteractor)(new BezierConnectorInteractor()));		
	}
	
	/** Draw the lines connecting the bezier curve control point grab handles to
	 *  the grab handles at the ends of the segments, then call super.paint() do the
	 *  normal highlighting.
	 */
	public void paint(Graphics2D g) {
		GeneralConnector connector = (GeneralConnector) getChild();
		List vertices = connector.getVertices();
		
		int numVertices = vertices.size();
		for (int i=0; i<numVertices; i++) {
			GeneralConnectorSite curSite = (GeneralConnectorSite) vertices.get(i);
			int id = curSite.getID();
			if (id != BezierShapeFactory.CTRL_POINT) {
				if (i > 0) {
					GeneralConnectorSite prevSite = (GeneralConnectorSite) vertices.get(i-1);
					drawLine(g, prevSite, curSite);
				}
				if (i < numVertices-1) {
					GeneralConnectorSite nextSite = (GeneralConnectorSite) vertices.get(i+1);
					drawLine(g, curSite, nextSite);
				}
			}
		}
		
		//Paint the rest
		super.paint(g);
	}
	
	/** Draw a line connecting two sites. */
	protected void drawLine(Graphics2D g, GeneralConnectorSite s1, GeneralConnectorSite s2) {
		GeneralPath path = new GeneralPath();
		path.moveTo((float)s1.getX(), (float)s1.getY());
		path.lineTo((float)s2.getX(), (float)s2.getY());
		
		g.setStroke(_stroke);
		g.setPaint(_paint);
		g.draw(path);
	}
}

/*
 * $Log: BezierConnectorHighlighter.java,v $
 * Revision 1.2  2002/06/28 08:06:03  adamtj
 * Implemented grab handle locking.  Grab handles may be locked to related handles so that dragging it will cause the others to move as well.
 *
 * Revision 1.1  2002/06/27 06:03:57  adamtj
 * Initial commit of BezierConnector files
 *
 *
 */
