/* $Id: BezierGrabHandle.java,v 1.2 2002/06/28 08:06:03 adamtj Exp $
 *
 */

package fcmodeler.interaction.BezierConnector;

import java.awt.Color;
import java.awt.Paint;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;

import diva.canvas.Site;
import diva.canvas.manipulator.GrabHandle;
import diva.canvas.toolbox.BasicFigure;

import fcmodeler.interaction.GeneralConnector.GeneralConnectorGrabHandle;

/**
 * BezierGrabHandle is a GeneralConnectorGrabHandle that changes it appearance
 * depending on which type of bezier curve site it is attached to.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class BezierGrabHandle extends GeneralConnectorGrabHandle {
	
	/** this indicates if this handle should be considered locked to related
	 *  handles.
	 */
	protected boolean _lockState = false;
	
	/** Save the color of the normal grab handles.
	 *  It should be blue, but somebody might want to change it.
	 */
	protected Paint _defaultHandlePaint = null;
	
	/** Set the shape and color of the grab handles depending on the type of the
	 *  site to which they are attached.
	 */
	protected void initShape() {
		if (BezierShapeFactory.SEG_END == _site.getID()) {
			_shape.setShape( new Ellipse2D.Double(_x-_size/2,_y-_size/2,_size,_size) );
			_shape.setStrokePaint(Color.black);
			setLockState(true);
		} else {
			super.initShape();
		}
		_defaultHandlePaint = _shape.getFillPaint();
	}
	
	/** Return the state of the lock.  The lock isn't actually used by this
	 *  this class.  It is for the DragInteractor's use.
	 */
	public boolean getLockState() {
		return _lockState;
	}
	
	/** Toggle the state of the lock */
	public void flipLockState() {
		setLockState(!_lockState);
	}
	
	/** Set the lock to the requested value, and change the colors to
	 *  show that the lock is on or off
	 */
	public void setLockState(boolean lock) {
		_lockState = lock;
		if (BezierShapeFactory.SEG_END == _site.getID()) {
			if (_lockState) {
				_shape.setFillPaint(Color.gray);
			} else {
				_shape.setFillPaint(Color.orange);
			}
		} else if (BezierShapeFactory.CTRL_POINT == _site.getID()) {
			if (_lockState) {
				_shape.setFillPaint(Color.orange);
			} else {
				_shape.setFillPaint(_defaultHandlePaint);
			}
		}
		
		repaint();
	}
}

/*
 * $Log: BezierGrabHandle.java,v $
 * Revision 1.2  2002/06/28 08:06:03  adamtj
 * Implemented grab handle locking.  Grab handles may be locked to related handles so that dragging it will cause the others to move as well.
 *
 * Revision 1.1  2002/06/27 06:03:57  adamtj
 * Initial commit of BezierConnector files
 *
 *
 */
