/* $Id: CustomGraphController.java,v 1.2 2002/06/27 06:08:35 adamtj Exp $
 *
 *
 */

package fcmodeler.interaction;

import java.awt.event.InputEvent;

import diva.graph.AbstractGraphController;
import diva.canvas.selection.SelectionModel;
import diva.canvas.selection.SelectionDragger;
import diva.canvas.GraphicsPane;
import diva.graph.NodeInteractor;
import diva.graph.EdgeInteractor;
import diva.canvas.connector.PerimeterTarget;
import diva.canvas.connector.ConnectorManipulator;
import diva.canvas.Figure;
import diva.graph.model.Node;
import diva.canvas.event.MouseFilter;

import fcmodeler.interaction.BezierConnector.BezierConnectorHighlighter;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorManipulator;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class CustomGraphController extends AbstractGraphController {
	private SelectionDragger _selectionDragger = null;
	private MouseFilter _controlFilter = null;
	private NodeAdder _nodeAdder = null;
	private NodeRemover _nodeRemover = null;
	private EdgeAdder _edgeAdder = null;
	private EdgeRemover _edgeRemover = null;
	
	/** Creates new CustomGraphController */
	public CustomGraphController() {
		// The interactors attached to nodes and edges
		SelectionModel sm = getSelectionModel();
		NodeInteractor ni = new NodeInteractor(this, sm);
		EdgeInteractor ei = new EdgeInteractor(this, sm);
		setNodeInteractor(ni);
		setEdgeInteractor(ei);
		
		PerimeterTarget ct = new PerimeterTarget() {
			public boolean accept(Figure f) {
				return (f.getUserObject() instanceof Node);
				// FIXME Used to also have ||
				// (f instanceof FigureWrapper &&
				//             ((FigureWrapper)f).getChild().instanceof Node);
			}
		};
		setConnectorTarget(ct);
		
		//create and set up the manipulator for connectors
		//BasicConnectorHighlighter is a ConnectorManipulator that highlights the Connector with red paint
		//BasicConnectorHighlighter highlighter = new BasicConnectorHighlighter();
		GeneralConnectorManipulator highlighter = new BezierConnectorHighlighter();
		highlighter.setSnapHalo(4.0);
		highlighter.setConnectorTarget(ct);
		ei.setPrototypeDecorator(highlighter);
		highlighter.addConnectorListener(new EdgeModifier(this));
		
		// The mouse filter needs to accept regular click or control click
		MouseFilter handleFilter = new MouseFilter(1, 0, 0);
		highlighter.setHandleFilter(handleFilter);
		
		//this mouse filter accepts control button + left click
		_controlFilter = new MouseFilter(InputEvent.BUTTON1_MASK, InputEvent.CTRL_MASK);
	}
	
	public void initializeInteraction() {
		GraphicsPane pane = getGraphView().getGraphicsPane();
		
		// Create and set up the selection dragger
		//the constructor adds the SelectionDragger to layer listeners
		_selectionDragger = new SelectionDragger(pane);
		_selectionDragger.addSelectionInteractor(getEdgeInteractor());
		_selectionDragger.addSelectionInteractor(getNodeInteractor());
		
		// Create a listener that creates new nodes
		//it only responds to ctrl + left button clicks
		//it's added to the background event layer's set of interactors
		_nodeAdder = new NodeAdder(this);
		_nodeAdder.setMouseFilter(_controlFilter);
		pane.getBackgroundEventLayer().addInteractor(_nodeAdder);
		
		_nodeRemover = new NodeRemover(this);
		_nodeRemover.setMouseFilter(new MouseFilter(InputEvent.BUTTON3_MASK, InputEvent.CTRL_MASK));
		getNodeInteractor().addInteractor(_nodeRemover);
		
		// Create the interactor that drags new edges.
		_edgeAdder = new EdgeAdder(this);
		_edgeAdder.setMouseFilter(_controlFilter);
		getNodeInteractor().addInteractor(_edgeAdder);
		
		//create the interactor that removes edges
		_edgeRemover = new EdgeRemover(this);
		_edgeRemover.setMouseFilter(new MouseFilter(InputEvent.BUTTON3_MASK, InputEvent.CTRL_MASK));
		getEdgeInteractor().addInteractor(_edgeRemover);
	}
}

/* 
 * $Log: CustomGraphController.java,v $
 * Revision 1.2  2002/06/27 06:08:35  adamtj
 * Activate BezierConnector edge type.  This turns on the bezier curve control point grab handles.
 *
 * Revision 1.1  2002/01/15 19:44:13  zcox
 * initial commit of interaction package
 *
 * 
 */
