/* $Id: EdgeAdder.java,v 1.2 2002/01/22 23:03:19 zcox Exp $
 *
 *
 */

package fcmodeler.interaction;

import java.awt.geom.Rectangle2D;

import diva.canvas.interactor.AbstractInteractor;
import diva.canvas.event.LayerEvent;
import diva.canvas.Figure;
import diva.graph.model.Node;
import diva.graph.AbstractGraphController;
import diva.graph.GraphView;
import diva.graph.model.GraphModel;
import diva.graph.model.Edge;
import diva.canvas.FigureLayer;
import diva.canvas.Site;
import diva.canvas.connector.Connector;
import diva.canvas.connector.ConnectorManipulator;
import diva.canvas.manipulator.GrabHandle;
import diva.canvas.connector.AutonomousSite;

import fcmodeler.view.BezierEdgeFigure;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class EdgeAdder extends AbstractInteractor {
	private AbstractGraphController _controller = null;
	private Site _tailSite = null;
	
	/** Creates new EdgeAdder */
	public EdgeAdder(AbstractGraphController controller) {
		_controller = controller;
	}
	
	public void mousePressed(LayerEvent layerEvent) {
		Figure source = null;
		double x = 0.0d;
		double y = 0.0d;
		
		//get the node figure
		source = layerEvent.getFigureSource();
		
		//get the (x,y) of the mouse-click (in layer coordinates)
		x = layerEvent.getLayerX();
		y = layerEvent.getLayerY();
		
		//find a suitable site on the node figure for the edge's tail
		_tailSite = _controller.getConnectorTarget().getTailSite(source, x, y);
	}
	
	public void mouseReleased(LayerEvent layerEvent) {
		GraphView view = null;
		GraphModel model = null;
		FigureLayer layer = null;
		double x = 0.0d;
		double y = 0.0d;
		Edge edge = null;
		Site headSite = null;
		Connector connector = null;
		ConnectorManipulator cm = null;
		GrabHandle gh = null;
		double x1 = 0.0d, y1 = 0.0d, x2 = 0.0d, y2 = 0.0d;
		double pickHalo = 0.0d;
		Rectangle2D region = null;
		Figure source = null;
		
		//get the graph model & view
		view = _controller.getGraphView();
		model = _controller.getGraphModel();
		
		//get the layer that the event came from
		layer = (FigureLayer) layerEvent.getLayerSource();
		
		//get the (x,y) of the mouse-click (in layer coordinates)
		x = layerEvent.getLayerX();
		y = layerEvent.getLayerY();
		
		//create a new edge
		edge = model.createEdge("e" + System.currentTimeMillis());
		
		//see if the user released over another node
		//which means they want that node to be the edge's head
		/*pickHalo = layer.getPickHalo();
		region = new Rectangle2D.Double(x - pickHalo, y - pickHalo, pickHalo*2, pickHalo*2);
		source = layer.pick(region);
		if (source != null) {
			//find a suitable site on the node figure for the edge's head
			headSite = _controller.getConnectorTarget().getHeadSite(source, x, y);
			System.out.println("x = " + x + ", y = " + y);
			System.out.println("xs = " + headSite.getX() + ", ys = " + headSite.getY());
			System.out.println("figure = " + headSite.getFigure().getUserObject());
		}
		else {*/
			//the head isn't connected to anything yet - so it's an AutonomousSite - it's floating
			headSite = new AutonomousSite(layer, x, y);
		//}
		
		//create & configure a figure for the new edge
		connector = view.getEdgeRenderer().render(edge, _tailSite, headSite);
		connector.setInteractor(_controller.getEdgeInteractor());
		
		//need to setup the edge figure's EdgeShape so there's an actual line to draw
		//NOTE - don't need to set anything up, because BezierEdgeFigure does it for us!
		if (connector instanceof BezierEdgeFigure) {
			/*x1 = _tailSite.getX() + 0.1*(headSite.getX() - _tailSite.getX());
			y1 = _tailSite.getY() + 0.1*(headSite.getY() - _tailSite.getY());
			x2 = _tailSite.getX() + 0.9*(headSite.getX() - _tailSite.getX());
			y2 = _tailSite.getY() + 0.9*(headSite.getY() - _tailSite.getY());*/
			/*edgeShape = ((EdgeFigure) connector).getEdgeShape();
			edgeShape.resetPath();
			edgeShape.moveTo((float) _tailSite.getX(), (float) _tailSite.getY());
			//edgeShape.lineTo((float) x1, (float) y1);
			//edgeShape.lineTo((float) x2, (float) y2);
			edgeShape.lineTo((float) headSite.getX(), (float) headSite.getY());
			//edgeShape.endPath((float) x2, (float) y2, (float) headSite.getX(), (float) headSite.getY());
			edgeShape.endSimpleLine();*/
		}
		else {
			throw new UnsupportedOperationException("need to fix the case where connector is not a BezierEdgeFigure");
		}
		
		//add edge figure to the view
		view.addEdgeFigureToLayer(connector);
		view.addEdgeMapping(edge, connector);
		
		//add edge to the model
		model.setEdgeTail(edge, (Node) _tailSite.getFigure().getUserObject());
		
		//route the edge figure
		connector.route();
		
		//add it to the selection so it gets a manipulator (which creates grab handles), and
		//make events go to the grab-handle under the mouse
		_controller.getSelectionModel().addSelection(connector);
		//cm = (ConnectorManipulator) connector.getParent();
		//gh = cm.getHeadHandle();
		//layer.grabPointer(layerEvent, gh);
	}
}

/* 
 * $Log: EdgeAdder.java,v $
 * Revision 1.2  2002/01/22 23:03:19  zcox
 * modifications to EdgeFigure classes
 *
 * Revision 1.1  2002/01/15 19:44:14  zcox
 * initial commit of interaction package
 *
 * 
 */
