/* $Id: GeneralConnectorHighlighter.java,v 1.2 2002/06/27 09:06:28 adamtj Exp $
 *
 */

package fcmodeler.interaction.GeneralConnector;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.Paint;
import java.awt.Shape;
import java.util.ListIterator;

import diva.canvas.Figure;
import diva.canvas.connector.ConnectorEnd;
import diva.canvas.FigureDecorator;
import diva.canvas.connector.AbstractConnector;

/**
 * GeneralConnectorHighlighter adds highlighting to a GeneralConnector and draws 
 * grab handles when it is selected.  To do special highlighting, subclass this
 * and override paint().  See BezierConnectorHighlighter for an example.  
 * BezierConnectorHighlighter draws extra lines to connect bezier control point
 * grab handles to segment end grab handles.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class GeneralConnectorHighlighter extends GeneralConnectorManipulator {
	
	/** the color of the line */
	protected Paint _paint = null;
	/** the Stroke to use on the line */
	protected BasicStroke _stroke = null;
	
	
	/** Creates a new instance of GeneralConnectorHighlighter */
	public GeneralConnectorHighlighter() {
		super();
		_stroke = new BasicStroke(2.0f);
		_paint = Color.red;
	}
	
	/** Highlight the figure */
	public void paint(Graphics2D graphics2D) {
		Figure child = null;
		GeneralConnector connector = null;
		Paint origPaint = null;
		float origLineWidth = 0.0f;
		BasicStroke origStroke = null;
		Shape shape = null;
		ConnectorEnd headEnd = null, tailEnd = null;
		
		//let super do its usual painting
		super.paint(graphics2D);
		
		//add highlighting to the child
		if (getChild() == null) {
			return;
		}
		child = getChild();
		if (child instanceof GeneralConnector) {
			connector = (GeneralConnector) child;
			
			shape = connector.getShape();
			tailEnd = connector.getTailEnd();
			headEnd = connector.getHeadEnd();
			
			graphics2D.setStroke(new BasicStroke(2.0f));//_stroke);
			graphics2D.setPaint(Color.red);//_paint);
			graphics2D.draw(shape);
			
			if (tailEnd != null) {
				tailEnd.paint(graphics2D);
			}
			if (headEnd != null) {
				headEnd.paint(graphics2D);
			}
		}
		
		/* On drawing the grab handles:
		 * We can't call super.paint() before highlighting the connector because
		 * the grab handles will then be drawn under the connector.  We can't call
		 * it after we highlight the connector, because then the regular connector
		 * will be drawn on top of the highlighting.  We can't do this:
			child.setVisible(false);
			super.paint(graphics2D);
			child.setVisible(true);	
		 * because super.paint() disregards the visible attribute.
		 * We can't avoid calling super.paint() because that is the only way to
		 * draw the head and tail grab handles.  We don't have access to the private
		 * members of the superclass.  That is, the following doesn't work.
			Iterator i = _grabHandles.iterator();
			while (i.hasNext()) {
				GrabHandle h = (GrabHandle)i.next();
				h.paint(g);
			}
		 * Because of these problems with diva, we have to call super.paint()
		 * then highlight over the connector and grab handles, then draw the extra
		 * grab handles by hand, since we have access to them.
		*/
		
		//Redraw the grabhandles on top of the edge.
		//Draw them in reverse order, so that a handle which is on top of another 
		//handle actually gets drawn above it.
		ListIterator i = connector.getVertices().listIterator();
		//Move the iterator position to the end.
		while (i.hasNext()) {
			i.next();
		}
		while (i.hasPrevious()) {
			GeneralConnectorSite s = (GeneralConnectorSite) i.previous();
			Figure f = s.getFigure();
			if ( f != null ) {
				f.paint(graphics2D);
			}
		}
	}
}

/* 
 * $Log: GeneralConnectorHighlighter.java,v $
 * Revision 1.2  2002/06/27 09:06:28  adamtj
 * Draw the grab handles in reverse order so that a handle which is on top of another is drawn above it.
 *
 * Revision 1.1  2002/06/27 06:03:15  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
