/* $Id: GeneralConnectorManipulator.java,v 1.2 2002/06/28 08:06:03 adamtj Exp $
 *
 */

package fcmodeler.interaction.GeneralConnector;

import java.util.Iterator;

import diva.canvas.Figure;
import diva.canvas.FigureDecorator;
import diva.canvas.interactor.DragInteractor;
import diva.canvas.connector.ConnectorManipulator;
import diva.canvas.manipulator.GrabHandle;

/**
 * GeneralConnectorManipulator is a ConnectorManipulator that is capable of
 * decorating a GeneralConnector with the extra grab handles it needs.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class GeneralConnectorManipulator extends ConnectorManipulator {
	
	/** the grab handle interactor that responds to mouse drag events. */
	protected GeneralConnectorInteractor _vertexInteractor = null;
	
	/** Create a new instance of GeneralConnectorManipulator */
	public GeneralConnectorManipulator() {
		_vertexInteractor = new GeneralConnectorInteractor();
	}
	
	/** Creates a new GeneralConnectorHighlighter that highlights the specified
	 *  figure.  This need not be overridden in subclasses as it automatically
	 *  determines the class of <code>this</code> object, and creates a new object
	 *  of the same class.
	 * @param f the figure to which to attach the new highlighter
	 */	
	public FigureDecorator newInstance(Figure f) {
		try {
			GeneralConnectorHighlighter spawn = (GeneralConnectorHighlighter) this.getClass().newInstance();
			spawn.setGrabHandleFactory(this.getGrabHandleFactory());
			spawn.setHandleInteractor(this.getHandleInteractor());
			return spawn;
		} catch (Exception e) {
			GeneralConnectorHighlighter spawn = new GeneralConnectorHighlighter();
			spawn.setGrabHandleFactory(this.getGrabHandleFactory());
			spawn.setHandleInteractor(this.getHandleInteractor());
			return spawn;
		}
	}
	
	/** Sets the child figure that this object will manipulate.  This is where the
	 *  grab handles get created.
	 * @param f the figure to decorate
	 */
	public void setChild(Figure f) {
		// a null figure. must be anorexic. get it?  hehe. um, yeah.
		if (f==null) {
			super.setChild(null);
			clearGrabHandles();
			return;
		}
		
		//super.setChild(f) calls clearGrabHandles() so we must be called before 
		//we add grab handles.
		super.setChild(f);
		
		if ( !(f instanceof GeneralConnector) ) {
			throw new IllegalArgumentException(
			"GeneralConnector required by GeneralConnectorManipulator");
		}
		
		//Get the sites to attach grab handles to.
		//This is a little tricky.  We rely on the fact that the getVertices returns
		//a list of vertices with the tail as the first element and the head as the
		//last element.
		Iterator i = ((GeneralConnector)f).getVertices().iterator();
		i.next();
		while (i.hasNext()) {
			GeneralConnectorSite s = (GeneralConnectorSite) i.next();
			//if ((s.getID() != BezierShapeFactory.HEAD) && (s.getID() != BezierShapeFactory.TAIL)) {
			if (i.hasNext()) {
				GrabHandle g = ((GeneralConnector)f).getCustomGrabHandleFactory().createGrabHandle(s);
				g.setParent(this);
				g.setInteractor(_vertexInteractor);
				addGrabHandle(g);
			}
		}
	}
	
	/** Sets a custom DragInteractor so that grab handles may be controlled in 
	 *  different ways.
	 */
	public void setCustomHandleInteractor(GeneralConnectorInteractor interactor) {
		_vertexInteractor = interactor;
	}
	
	/** returns the custom DragInteractor */
	public DragInteractor getCustomHandleInteractor() {
		return _vertexInteractor;
	}
}

/* 
 * $Log: GeneralConnectorManipulator.java,v $
 * Revision 1.2  2002/06/28 08:06:03  adamtj
 * Implemented grab handle locking.  Grab handles may be locked to related handles so that dragging it will cause the others to move as well.
 *
 * Revision 1.1  2002/06/27 06:03:15  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
