/* $Id: GeneralConnectorRenderer.java,v 1.2 2002/07/13 07:48:12 adamtj Exp $
 *
 */

package fcmodeler.interaction.GeneralConnector;

import diva.graph.EdgeRenderer;
import diva.graph.model.Edge;
import diva.canvas.Site;
import diva.canvas.connector.Connector;
import diva.canvas.connector.Arrowhead;

/**
 * GeneralConnectorRenderer is the EdgeRenderer that attaches GeneralConnectors
 * to graph Edges.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class GeneralConnectorRenderer implements EdgeRenderer {
	
	/** the ShapeFactory that maps grab handle sites to edge shapes. */
	protected GeneralConnectorShapeFactory _factory=null;
	
	/** Creates a new instance of GeneralConnectorRenderer */
	public GeneralConnectorRenderer(GeneralConnectorShapeFactory factory) {
		_factory = factory;
	}

	/** Creates a <CODE>GeneralConnector/CODE> to be used as a visual representation for the
	 * specified <CODE>Edge</CODE>.
	 *
	 * @param edge the edge to render.
	 * @param tailSite the site on the tail node figure to connect this edge figure to.
	 * @param headSite the site on the head node figure to connect this edge figure to.
	 * @return a visual representation of the specified <CODE>Edge</CODE>, of type <CODE>BezierEdgeFigure</CODE>.
	 */
	public Connector render(Edge edge, Site tailSite, Site headSite) {
		GeneralConnector edgeFigure = null;

		edgeFigure = new GeneralConnector(tailSite, headSite, _factory);
		edgeFigure.setUserObject(edge);
		if (edge.isDirected()) {
			edgeFigure.setHeadEnd(new Arrowhead(headSite.getX(), headSite.getY(), headSite.getNormal()));
		}

		return edgeFigure;
	}
}

/* 
 * $Log: GeneralConnectorRenderer.java,v $
 * Revision 1.2  2002/07/13 07:48:12  adamtj
 * Fixed a problem with edge mappings.  GeneralConnectorEdgeFigure improperly implemented EdgeFigure.  It has been removed and now GeneralConnector extends AbstractEdgeFigure.
 *
 * Revision 1.1  2002/06/27 06:03:15  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
