/* $Id: GeneralConnectorShapeFactory.java,v 1.1 2002/06/27 06:03:16 adamtj Exp $
 *
 */

package fcmodeler.interaction.GeneralConnector;

import java.awt.Shape;
import java.util.List;
import java.awt.geom.Point2D;

import diva.canvas.manipulator.GrabHandleFactory;

/**
 * A ShapeFactory is used by a GeneralConnectorGeometry object to convert a list
 * of user-manipulable Sites into a Shape.
 *
 * Caveat: When implementing this class, the first and last Sites in the Lists
 * used by getShape() and setShape() should be the tail and head sites respectively
 * of the shape.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public interface GeneralConnectorShapeFactory {
	
	/** Return a list of GeneralConnectorSites representing the user-manipulable
	 *  vertices of this shape.  The first element in the list must be the tail
	 *  site and the last element must be the head site.
	 */
	public List setShape(Shape s);
	
	/** Given a list of user vertices, build and return a shape. */
	public Shape getShape(List vertices);
	
	/** Given data in whatever user-defined format is most convenient, build and
	 *  return a List of vertices.  The first element in the returned List of
	 *  vertices must be the tail site and the last element must be the head site.
	 */
	public List fromFile(Object data);
	
	/** Given a List of GeneralConnectorSites, build and return an object to be
	 *  used for saving to a file.  The object may be, for example, a list of
	 *  Point2Ds.  It is left to the implementer to determine what it returned.
	 */
	public Object toFile(List vertices);
	
	/** Return a custom GrabHandleFactory to be used for generating the grab handles
	 *  for this type of shape.
	 */
	public GrabHandleFactory getCustomGrabHandleFactory();
	
	/** Specify the custom GrabHandleFactory. */
	public void setCustomGrabHandleFactory(GrabHandleFactory factory);
	
	/** Given the vertices, compute the normal of the shape's head */
	public double getHeadNormal(Point2D headPoint, List vertices);
	
	/** Given the vertices, compute the normal of the shape's tail */
	public double getTailNormal(Point2D tailPoint, List vertices);
}

/*
 * $Log: GeneralConnectorShapeFactory.java,v $
 * Revision 1.1  2002/06/27 06:03:16  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
