/* $Id: GeneralConnectorSite.java,v 1.1 2002/06/27 06:03:16 adamtj Exp $
 *
 */

package fcmodeler.interaction.GeneralConnector;

import diva.canvas.AbstractSite;
import diva.canvas.Figure;
import diva.canvas.Site;

/**
 * GeneralConnectorSite is the Site that GeneralConnectorGrabHandles use.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class GeneralConnectorSite extends AbstractSite {
	
	/** the ID of this site */
	protected int _id=0;
	/** the x coordinate of the Site */
	protected double _x=0;
	/** the y coordinate of the Site */
	protected double _y=0;
	/** the Figure which contains this Site */
	protected Figure _figure=null;
	
	/** Creates a new instance of GeneralConnectorSite. */
	public GeneralConnectorSite() {
		super();
	}
	
	/** Creates a new instance of GeneralConnectorSite with the specified ID.
	 * @param ID the ID of this site
	 */
	public GeneralConnectorSite(int ID) {
		super();
		_id = ID;
	}
	
	/** Creates a new instance of GeneralConnectorSite with the specified ID and
	 *  coordinates.
	 *  @param x the x coordinate
	 *  @param y the y coordinate
	 *  @param ID the ID of this site
	 */
	public GeneralConnectorSite(double x, double y, int ID) {
		super();
		_x = x;
		_y = y;
		_id = ID;
	}
	
	/** Creates a new instance of GeneralConnectorSite with the same position
	 *  and ID as the parameter Site.
	 * @param s the Site to copy
	 */
	public GeneralConnectorSite(Site s) {
		super();
		_x = s.getX();
		_y = s.getY();
		if (s instanceof GeneralConnectorSite) {
			_id = ((GeneralConnectorSite)s).getID();
		}
		_figure = s.getFigure();
	}
	
	/** Get the figure to which this site is attached. */
	public Figure getFigure() {
		return _figure;
	}
	
	/** This is declared abstract in the superclass, so it must be implemented, yet
	 * the Diva documentation states "Within each figure, the IDs of the sites must
	 * be unique."  I have no idea how to ensure that IDs are unique.  Since there
	 * is no setID function, I cannot assume that somebody else will ensure that IDs
	 * are unique.  I will therefore use it for the "type" of this site.  The type
	 * may be used by grab handles to draw different styles of handles for different
	 * types of sites.
	 * @return the id of this site.
	 */
	public int getID() {
		return _id;
	}
	
	/** Return the x coordinate of this site. */
	public double getX() {
		return _x;
	}
	/** Return the y coordinate of this site. */
	public double getY() {
		return _y;
	}
	
	/** Move this site to the indicated position.
	 *  @param x the new x coordinate
	 *  @param y the new y coordinate
	 */
	public void moveTo(double x, double y) {
		_x = x;
		_y = y;
	}
	
	/** Set the figure to which this site is attached. */
	public void setFigure(Figure f) {
		_figure = f;
	}
	
	/** Set the ID of this site.  See getID() */
	public void setID(int ID) {
		_id = ID;
	}
	
	/** Translate the site by the indicated distance.
	 *  @param x the amount by which to vary the x coordinate
	 *  @param y the amount by which to vary the y coordinate
	 */
	public void translate(double x, double y) {
		_x += x;
		_y += y;
	}
}

/*
 * $Log: GeneralConnectorSite.java,v $
 * Revision 1.1  2002/06/27 06:03:16  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
