/* $Id: NodeRemover.java,v 1.1 2002/01/15 19:44:14 zcox Exp $
 *
 *
 */

package fcmodeler.interaction;

import java.util.Iterator;

import diva.canvas.Figure;
import diva.canvas.interactor.AbstractInteractor;
import diva.canvas.event.LayerEvent;
import diva.graph.GraphController;
import diva.graph.model.GraphModel;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.GraphEvent;
import diva.graph.GraphView;

import fcmodeler.view.NodeFigure;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class NodeRemover extends AbstractInteractor {
	private GraphController _controller = null;
	
	/** Creates new NodeRemover */
	public NodeRemover(GraphController controller) {
		_controller = controller;
	}
	
	public void mousePressed(LayerEvent layerEvent) {
		GraphModel model = _controller.getGraphModel();
		GraphView view = _controller.getGraphView();
		Figure figure = null;
		Node node = null;
		Iterator edges = null;
		
		//get the figure that was hit
		figure = layerEvent.getFigureSource();
		
		//make sure it was a node figure
		if (figure instanceof NodeFigure) {
			//get the node that this figure represents
			node = (Node) figure.getUserObject();
			
			//make sure the node figure and all adjacent edge figures are de-selected
			_controller.getNodeInteractor().getSelectionModel().removeSelection(figure);
			edges = node.outEdges();
			while (edges.hasNext()) {
				_controller.getEdgeInteractor().getSelectionModel().removeSelection(
					view.getEdgeFigure((Edge) edges.next()));
			}
			edges = node.inEdges();
			while (edges.hasNext()) {
				_controller.getEdgeInteractor().getSelectionModel().removeSelection(
					view.getEdgeFigure((Edge) edges.next()));
			}
			
			//remove node figure & all adjacent edge figures from the view
			_controller.nodeRemoved(new GraphEvent(GraphEvent.NODE_REMOVED, this, node, node.getParent()));
			
			//remove node & all adjacent edges from the model
			//have to do this after the view removal, because BasicGraphImpl.remove() clears the BasicNode's
			//edge sets so the node has no in/out edges in AbstractGraphController.nodeRemoved()
			model.removeNode(node);
		}
	}
}

/* 
 * $Log: NodeRemover.java,v $
 * Revision 1.1  2002/01/15 19:44:14  zcox
 * initial commit of interaction package
 *
 * 
 */
