/* $Id: AbstractFileFilter.java,v 1.7 2002/02/15 20:01:06 zcox Exp $
 *
 *
 * Created on April 3, 2001, 7:25 PM
 */

package fcmodeler.io;

import javax.swing.filechooser.FileFilter;
import java.io.File;

/** An abstract convenience class for file filters.  Concrete subclasses representing file filters
 * for certain file types simply subclass this class, override the <CODE>isValidExtension</CODE>
 * method, and create a string description of the file type.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.7 $
 * @since JDK1.3
 */

public abstract class AbstractFileFilter extends FileFilter {
	/** The string description for this file filter.
	 *
	 */	
	protected String _description = null;
	
	/** Creates a new <CODE>AbstractFileFilter</CODE>.
	 *
	 */
	public AbstractFileFilter() {
		
	}
	
	/** Returns <CODE>true</CODE> if the specified file should be shown in the file chooser.
	 *
	 * @param file the file to test for acceptance.
	 * @return <CODE>true</CODE> if the specified file should be shown in the file chooser.
	 */	
	public boolean accept(File file) {
		String extension = null, name = null;
		boolean accept = false;
		
		if (file.isDirectory()) {
			accept =  true;
		}
		else {
			name = file.getName();
			extension = name.substring(name.lastIndexOf('.')+1).toLowerCase();
			if (isValidExtension(extension)) {
				accept = true;
			}
		}

		return accept;
	}
	
	/** Returns the string description of this file filter.
	 *
	 * @return the string description of this file filter.
	 */	
	public String getDescription() {
		return _description;
	}
	
	/** Returns <CODE>true</CODE> if the specified file extension is valid for this file filter.
	 *
	 * @param extension the file extension to test for validity.
	 * @return <CODE>true</CODE> if the specified file extension is valid for this file filter.
	 */	
	public abstract boolean isValidExtension(String extension);
}

/* 
 * $Log: AbstractFileFilter.java,v $
 * Revision 1.7  2002/02/15 20:01:06  zcox
 * big time update
 *
 * Revision 1.6  2002/01/15 20:30:05  zcox
 * some changes
 *
 * Revision 1.5  2001/05/22 16:42:10  zcox
 * code clean-up
 *
 * Revision 1.4  2001/04/04 01:44:16  zcox
 * no message
 *
 * Revision 1.1  2001/04/04 01:21:47  zcox
 * initial commit of some fcmodeler.io classes
 *
 *
 *
 */
