/* $Id: CycleFileReader.java,v 1.1 2002/01/15 20:32:06 zcox Exp $
 *
 *
 */

package fcmodeler.io;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXParseException;

import diva.graph.model.Graph;
import diva.graph.model.Node;
import diva.graph.model.Edge;

import fcmodeler.subgraph.Cycle;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class CycleFileReader extends DefaultHandler {
	private List _cycles = null;
	private Cycle _cycle = null;
	private Map _nodeMap = null, _edgeMap = null;
	private Graph _graph = null;
	private boolean _error = false;
	
	/** Creates new CycleFileReader */
	public CycleFileReader(Graph graph) {
		_graph = graph;
	}
	
	public void endDocument() throws SAXException {
		//delete the maps
		_nodeMap = null;
		_edgeMap = null;
	}
	
	public void endElement(String uri, String localName, String qName) throws SAXException {
		Node first = null;
		
		if (!_error) {
			if (localName.equals("cycle") || qName.equals("cycle")) {
				//finish the cycle & make sure it's valid
				if (_cycle != null) {
					first = (Node) _cycle.get(0);
					_cycle.add(first);
					if (!_cycle.isValid()) {
						_cycles.remove(_cycle);
						_cycle = null;
					}
				}
			}
		}
		_error = false;
	}
	
	public void error(SAXParseException saxParseException) throws SAXException {
		_error = true;
		System.out.println("error: " + saxParseException);
	}
	
	public void fatalError(SAXParseException saxParseException) throws SAXException {
		System.out.println("fatalError: " + saxParseException);
	}
	
	public List getCycles() {
		return _cycles;
	}
	
	public void startDocument() throws SAXException {
		Node node = null;
		Edge edge = null;
		Iterator nodes = null, edges = null;
		
		//create List of cycles
		_cycles = new ArrayList();
		
		//create maps
		_nodeMap = new HashMap();
		_edgeMap = new HashMap();
		
		//put nodes/edges & their semantic objects in maps
		nodes = _graph.nodes();
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			_nodeMap.put(node.getSemanticObject(), node);
			edges = node.outEdges();
			while (edges.hasNext()) {
				edge = (Edge) edges.next();
				_edgeMap.put(edge.getSemanticObject(), edge);
			}
		}
	}
	
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		Node node = null;
		Edge edge = null;
		Cycle last = null;
		
		if (!_error) {
			if (localName.equals("cycle") || qName.equals("cycle")) {
				//create new cycle & add to list of cycles
				_cycle = new Cycle();
				_cycles.add(_cycle);
			}
			else if (localName.equals("node") || qName.equals("node")) {
				if (_cycle != null) {
					node = (Node) _nodeMap.get(attributes.getValue("id"));
					if (node != null) {
						_cycle.add(node);
					}
					else {
						_cycles.remove(_cycles.size()-1);
						_cycle = null;
						System.out.println("node " + attributes.getValue("id") + " not found in FCModeler's graph model - invalid cycle");
					}
				}
			}
			else if (localName.equals("edge") || qName.equals("edge")) {
				if (_cycle != null) {
					edge = (Edge) _edgeMap.get(attributes.getValue("id"));
					if (edge != null) {
						_cycle.add(edge);
					}
					else {
						_cycles.remove(_cycles.size()-1);
						_cycle = null;
						System.out.println("edge " + attributes.getValue("id") + " not found in FCModeler's graph model - invalid cycle");
					}
				}
			}
		}
		_error = false;
	}
	
	public void warning(SAXParseException saxParseException) throws SAXException {
		System.out.println("warning: " + saxParseException);
	}
}

/* 
 * $Log: CycleFileReader.java,v $
 * Revision 1.1  2002/01/15 20:32:06  zcox
 * initial commit of some io classes
 *
 * 
 */
