/* $Id: JGraphToJPEGWriter.java,v 1.5 2002/05/23 17:02:34 adamtj Exp $
 *
 *
 * Created on March 27, 2001, 4:31 PM
 */

package fcmodeler.io;

import javax.swing.JFileChooser;
import java.io.File;
import java.io.FileOutputStream;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Color;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;

import diva.graph.JGraph;

/** 
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */

public class JGraphToJPEGWriter extends Object {
	
	/** Creates new GraphPaneToJPEGWriter */
	public JGraphToJPEGWriter() {
		
	}
	
	public static void writeJGraphToJPEG(JGraph jgraph) {
		writeJGraphToJPEG(jgraph, null);
	}
	
	public static void writeJGraphToJPEG(JGraph jgraph, String path) {
		JFileChooser save = null;
		int returnVal = 0;
		
		//open a file save dialog
		if (path != null) {
			save = new JFileChooser(path);
		}
		else {
			save = new JFileChooser();
		}
		save.addChoosableFileFilter(new JPEGFileFilter());
		save.setSelectedFile(new File("*.jpg"));
		returnVal = save.showSaveDialog(null);
		
		//if the user selected a file
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				//get the selected file, and create an output stream to it
				File file = save.getSelectedFile();
				FileOutputStream out = new FileOutputStream(file);
				
				//create the jpeg encoder
				JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
				
				//create an off-screen buffered image to paint to and get its Graphics2D
				//context to use for painting
				BufferedImage image = new BufferedImage(jgraph.getWidth(), 
					jgraph.getHeight(), BufferedImage.TYPE_INT_RGB);
				Graphics2D imageg2 = image.createGraphics();
				
				//paint the background white
				imageg2.setBackground(Color.white);
				imageg2.clearRect(0, 0, jgraph.getWidth(), jgraph.getHeight());
				
				//ask the graph pane to paint to the Graphics2D context
				jgraph.getGraphPane().paint(imageg2);
				
				//configure the jpeg encoder
				JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(image);
                //This is where we choose the quality of the image.
                //0.1f is very poor quality, 1.0f is max quality.
                //Filesize goes up with increased quality.
                param.setQuality(1.0f, false);
				encoder.setJPEGEncodeParam(param);
				
				//encode the buffered image - write the jpeg to the file
				encoder.encode(image);
				
				//close the FileOutputStream
				out.close();
				
			} catch (java.io.FileNotFoundException e) {
				e.printStackTrace();
			} catch (java.io.IOException e) {
				e.printStackTrace();
			}
		}
	}
}

/* 
 * $Log: JGraphToJPEGWriter.java,v $
 * Revision 1.5  2002/05/23 17:02:34  adamtj
 * turned the jpeg quality to maximum.
 *
 * Revision 1.4  2002/01/15 20:30:05  zcox
 * some changes
 *
 * Revision 1.3  2001/05/29 16:57:35  zcox
 * file chooser starts with the file *.jpg now, and starts in pathToGraphs directory
 *
 * Revision 1.2  2001/05/22 16:42:10  zcox
 * code clean-up
 *
 * Revision 1.1  2001/04/04 01:21:47  zcox
 * initial commit of some fcmodeler.io classes
 *
 *
 *
 */
