/* $Id: DatabaseGraphReader.java,v 1.1 2002/01/15 20:28:41 zcox Exp $
 *
 *
 */

package fcmodeler.io.db;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import diva.graph.model.GraphModel;
import diva.graph.model.Node;
import diva.graph.model.Edge;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class DatabaseGraphReader extends Object {
	private GraphModel _model = null;
	//node tables & columns
	public static final String NODE_TABLE = "Nodes";
	public static final String NODE_ID_COL = "ID";
	public static final String NODE_LABEL_COL = "Node_Label";
	public static final String ORGANELLE_ID_IN_NODE_TABLE_COL = "Organelle_ID";
	public static final String ORGANELLE_TABLE = "OrganelleType";
	public static final String ORGANELLE_ID_COL = "ID";
	public static final String ORGANELLE_CODE_COL = "Organelle_Code";
	//edge talbes & columns
	public static final String LINK_TABLE = "Link";
	public static final String LINK_ID_COL = "ID";
	public static final String LINK_PIECE_TABLE = "LinkPiece";
	public static final String LINK_ID_IN_LINK_PIECE_TABLE_COL = "Link_ID";
	public static final String NODE_ID_IN_LINK_PIECE_TABLE_COL = "Node_ID";
	public static final String ROLE_ID_IN_LINK_PIECE_TABLE_COL = "Role_ID";
	public static final String LINK_ROLE_TYPE_TABLE = "LinkRoleType";
	public static final String LINK_ROLE_TYPE_ID = "ID";
	public static final String LINK_ROLE_TYPE_COL = "Role";
	
	/** Creates new DatabaseGraphReader */
	public DatabaseGraphReader() {
		
	}
	
	public GraphModel getGraphModel() {
		return _model;
	}
	
	public void readGraph(String url) {
		Connection con = null;
		/*Statement nodesStmt = null, linkIDsStmt = null, linkPiecesStmt = null, nodeLabelsStmt = null;
		Statement roleTypesStmt = null, textPropsStmt = null, dbReferencesStmt = null, webDatabasesStmt = null;
		Statement organelleStmt = null;
		ResultSet nodes = null, linkIDs = null, linkPieces = null, nodeLabels = null, roleTypes = null;
		ResultSet organelle = null;
		/*String nodesQuery = "SELECT " + NODE_ID_COL + ", " + NODE_LABEL_COL + ", " +
			ORGANELLE_ID_IN_NODE_TABLE_COL + " FROM " + NODE_TABLE;*/
		/*String nodesQuery = "SELECT " + NODE_ID_COL + ", " + NODE_LABEL_COL + " FROM " + NODE_TABLE;
		String organelleQuery = "SELECT " + ORGANELLE_CODE_COL + " FROM " + ORGANELLE_TABLE + " WHERE " +
		ORGANELLE_ID_COL + " =";
		String linkIDsQuery = "SELECT " + LINK_ID_COL + " FROM " + LINK_TABLE;
		String linkPiecesQuery = "SELECT " + NODE_ID_IN_LINK_PIECE_TABLE_COL + ", " +
		ROLE_ID_IN_LINK_PIECE_TABLE_COL + " FROM " + LINK_PIECE_TABLE + " WHERE " +
		LINK_ID_IN_LINK_PIECE_TABLE_COL + " =";
		String roleTypeQuery = "SELECT " + LINK_ROLE_TYPE_COL + " FROM " + LINK_ROLE_TYPE_TABLE +
		" WHERE " + LINK_ROLE_TYPE_ID + " =";
		int linkID = 0, roleID = 0, organelleID = 0;
		Integer nodeID = null;
		String nodeLabel = null, roleType = null, organelleCode = null;
		Map nodeMap = null, roleTypesMap = null;
		Node node = null, tail = null, head = null;
		Edge edge = null;
		List nodeList = null;
		Iterator lpIterator = null;*/
		//Set linkTypes = new HashSet();
		//Set roles = new HashSet();
		Map linkProperties = new HashMap();
		Map linkPieceProperties = new HashMap();
		Map nodeMap = new HashMap();
		
		try {
			//load the mysql driver
			// The newInstance() call is a work around for some
			// broken Java implementations
			Class.forName("org.gjt.mm.mysql.Driver").newInstance();
		}
		catch (Exception e) {
			System.err.println("Unable to load driver.");
			e.printStackTrace();
		}
		
		try {
			//setup the graph
			_model = new GraphModel();
			List nodeProperties = new ArrayList();
			List edgeProperties = new ArrayList();
			_model.getGraph().setProperty("nodeProperties", nodeProperties);
			_model.getGraph().setProperty("edgeProperties", edgeProperties);
			
			//open the connection to the db
			con = DriverManager.getConnection(url);
			System.out.println("db opened");
			
			//get all of the subsets in the db
			String subsetQuery = "SELECT SubsetType.ID, Submitter.Name_First, Submitter.Name_Last, SubsetType.Description, SubsetType.Date_Created FROM SubsetType, Submitter WHERE Submitter.ID=SubsetType.Submitter_ID";
			Statement subsetStmt = con.createStatement();
			ResultSet subsetResults = subsetStmt.executeQuery(subsetQuery);
			List subsets = new ArrayList();
			Subset subset = null;
			while (subsetResults.next()) {
				subset = new Subset(subsetResults.getInt(1), subsetResults.getString(2) + subsetResults.getString(3), subsetResults.getString(4), subsetResults.getString(5));
				subsets.add(subset);
			}
			
			//ask the user to select a subset
			SubsetSelectionDialog dialog = new SubsetSelectionDialog(subsets);
			int subsetID = 0;
			if (dialog.showDialog() == SubsetSelectionDialog.OK) {
				subsetID = dialog.getSelectedSubset().getID();
				
				//get the links in the subset
				String linksQuery = "SELECT Subset.Link_ID FROM Subset WHERE Subset.Subset_ID = " + subsetID;
				Statement linksStmt = con.createStatement();
				ResultSet linksResults = linksStmt.executeQuery(linksQuery);
				int linkID = 0;
				while (linksResults.next()) {
					linkID = linksResults.getInt(1);
					
					//get properties for the link
					linkProperties.clear();
					String linkPropsQuery = "SELECT Link.ID, LinkType.Link_Type, Submitter.Name_First, Submitter.Name_Last, Link.Date_Submitted, Link.Confidence, Link.Strength, LinkCategoryType.Link_Category FROM Link, LinkType, Submitter, LinkCategoryType WHERE Submitter.ID=Link.Submitter_ID AND Link.ID=" + linkID + " AND Link.Link_Type_ID=LinkType.ID AND LinkCategoryType.ID=Link.LinkCategory_ID";
					Statement linkPropsStmt = con.createStatement();
					ResultSet linkPropsResults = linkPropsStmt.executeQuery(linkPropsQuery);
					String linkType=null, firstName, lastName, dateSubmitted, confidence, strength, categoryType;
					while (linkPropsResults.next()) {
						linkType = linkPropsResults.getString(2);
						//linkTypes.add(linkType);
						firstName = linkPropsResults.getString(3);
						lastName = linkPropsResults.getString(4);
						dateSubmitted = linkPropsResults.getString(5);
						confidence = linkPropsResults.getString(6);
						strength = linkPropsResults.getString(7);
						categoryType = linkPropsResults.getString(8);
					}
					
					//create a new link
					Link link = LinkFactory.getInstance().createLink(linkType, linkProperties, _model);
					
					//get the link pieces of the link
					String linkPiecesQuery = "SELECT LinkPiece.Link_ID, LinkPiece.Node_ID, Nodes.Node_Label, LinkRoleType.Role FROM LinkPiece, Nodes, LinkRoleType WHERE LinkPiece.Link_ID=" + linkID + " AND LinkPiece.Role_ID=LinkRoleType.ID AND Nodes.ID=LinkPiece.Node_ID";
					Statement linkPiecesStmt = con.createStatement();
					ResultSet linkPiecesResults = linkPiecesStmt.executeQuery(linkPiecesQuery);
					int nodeID = 0;
					String nodeLabel, role;
					while (linkPiecesResults.next()) {
						linkPieceProperties.clear();
						nodeID = linkPiecesResults.getInt(2);
						nodeLabel = linkPiecesResults.getString(3);
						role = linkPiecesResults.getString(4);
						//roles.add(role);
						
						linkPieceProperties.put("label", nodeLabel);
						
						//get properties for the node
						String nodeInfoQuery = "SELECT Nodes.Node_Label, Nodes.NodeDescription, Submitter.Name_First, Submitter.Name_Last, Nodes.Date_Submitted, WebDatabase.Database_Name, DatabaseReference.Accession_Code FROM Nodes, Submitter, WebDatabase, DatabaseReference WHERE Nodes.ID=" + nodeID + " AND Submitter.ID=Nodes.Submitter_ID AND Nodes.ExternalRef_ID=DatabaseReference.ExtReference_ID AND DatabaseReference.Database_ID=WebDatabase.ID";
						Statement nodeInfoStmt = con.createStatement();
						ResultSet nodeInfoResults = nodeInfoStmt.executeQuery(nodeInfoQuery);
						String nodeDesc, nFirstName, nLastName, nDateSubmitted, dbName, accessionCode;
						while (nodeInfoResults.next()) {
							nodeDesc = nodeInfoResults.getString(2);
							nFirstName = nodeInfoResults.getString(3);
							nLastName = nodeInfoResults.getString(4);
							nDateSubmitted = nodeInfoResults.getString(5);
							dbName = nodeInfoResults.getString(6);
							accessionCode = nodeInfoResults.getString(7);
						}
						
						//add this link piece to the link
						link.addLinkPiece(role, linkPieceProperties, nodeMap);
					}
				}
				
				//System.out.println("linkTypes = " + linkTypes);
				//System.out.println("roles = " + roles);
			}
			
			
			//create the nodes
			/*nodeMap = new HashMap();
			nodesStmt = con.createStatement();
			nodes = nodesStmt.executeQuery(nodesQuery);
			while (nodes.next()) {
				//DEBUG
				//java.sql.ResultSetMetaData md = nodes.getMetaData();
				//System.out.println("" + md.getColumnCount() + " columns");
				//for (int i=1; i<=md.getColumnCount(); i++) {
				//	System.out.println("column " + i + ": " + md.getColumnName(i));
				//}
				//END DEBUG
			 
				//get the node ID, node label, and organelle ID
				nodeID = new Integer(nodes.getInt(1));
				nodeLabel = nodes.getString(2);
				//organelleID = nodes.getInt(3);
			 
				//get the organelle code
				//organelleStmt = con.createStatement();
				//organelle = organelleStmt.executeQuery(organelleQuery + " " + organelleID);
				//organelle.next();
				//organelleCode = organelle.getString(1);
			 
				//create the node
				if (organelleCode != null) {
					node = _model.createNode(nodeLabel + "_" + organelleCode);
				}
				else {
					node = _model.createNode(nodeLabel);
				}
			 
				//add node to the graph model
				_model.addNode(node);
			 
				//map the node ID to the node object
				nodeMap.put(nodeID, node);
			}
			System.out.println("nodes finished");
			 
			//create the edges
			nodeList = new ArrayList();
			roleTypesMap = new HashMap();
			linkIDsStmt = con.createStatement();
			linkIDs = linkIDsStmt.executeQuery(linkIDsQuery);
			while (linkIDs.next()) {
				//get the link ID
				linkID = linkIDs.getInt(1);
				//get the link pieces for the corresponding link ID
				linkPiecesStmt = con.createStatement();
				linkPieces = linkPiecesStmt.executeQuery(linkPiecesQuery + " " + linkID);
				nodeList.clear();
				roleTypesMap.clear();
				while (linkPieces.next()) {
					//get the node ID & role type ID for each link piece
					nodeID = new Integer(linkPieces.getInt(1));
					roleID = linkPieces.getInt(2);
					//add the node ID to the list of nodes
					nodeList.add(nodeID);
			 
					//get the role type for this node
					roleTypesStmt = con.createStatement();
					roleTypes = roleTypesStmt.executeQuery(roleTypeQuery + " " + roleID);
					roleTypes.next();
					roleType = roleTypes.getString(1);
					roleTypesMap.put(nodeID, roleType);
				}
			 
				//build the edges from the link pieces
				//NOTE: ONLY source AND sink ROLE TYPES ARE SUPPORTED HERE
				if (nodeList.size() > 2) {
					//System.out.println("only 2-node link pieces are currently supported");
					System.out.println("" + nodeList.size() + "-node link pieces not supported");
					continue;
				}
				tail = null;
				head = null;
				lpIterator = nodeList.iterator();
				while (lpIterator.hasNext()) {
					nodeID = (Integer) lpIterator.next();
					roleType = (String) roleTypesMap.get(nodeID);
					if (roleType.equals("source")) {
						tail = (Node) nodeMap.get(nodeID);
					}
					else if (roleType.equals("sink")) {
						head = (Node) nodeMap.get(nodeID);
					}
					else {
					//	System.out.println("only source & sink role types are currently supported");
					}
				}
				if (tail != null && head != null) {
					//create the edge & connect it
					edge = _model.createEdge("" + linkID);
					_model.connectEdge(edge, tail, head);
				}
				else {
				//	System.out.println("couldn't create edge " + linkID);
				}
			}
			System.out.println("edges finished");*/
			
			//close the connection to the db
			con.close();
			System.out.println("db closed");
		}
		catch (SQLException e) {
			System.out.println("SQLException: " + e.getMessage());
			System.out.println("SQLState:     " + e.getSQLState());
			System.out.println("VendorError:  " + e.getErrorCode());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}

/*
 * $Log: DatabaseGraphReader.java,v $
 * Revision 1.1  2002/01/15 20:28:41  zcox
 * initial commit of fcmodeler.io.db package
 *
 *
 *
 */
