/* $Id: DotGraphReader.java,v 1.1 2002/02/15 20:06:46 zcox Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.io.Reader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

import diva.graph.model.Graph;
import diva.graph.layout.LayoutTarget;

import fcmodeler.FCModeler;
import fcmodeler.io.AbstractGraphReader;

/** 
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class DotGraphReader extends AbstractGraphReader {
	private DotGraphHandler _handler = null;
	
	/** Creates new DotGraphReader */
	public DotGraphReader(DotGraphHandler handler) {
		_handler = handler;
	}
	
	public void configure(FCModeler fcmodeler) {
		_handler.configure(fcmodeler);
	}
	
	private void getAttributes(String string, Map map) {
		int comma = -1, quote1 = 0, quote2 = 0, equals = 0;
		String property = null, value = null;
		
		//clear the map
		map.clear();
		
		//extract all prop & value pairs and fill the map
		while (comma < string.length()) {
			//find the equals sign & extract the property - are there ever quotes around a property?
			equals = string.indexOf('=', comma+1);
			property = removeQuotes(string.substring(comma+1, equals).trim());
			
			//test if value is quoted
			if (string.charAt(equals+1) == '\"' || (string.charAt(equals+1) == ' ' && string.charAt(equals+2) == '\"')) {
				//find the closing quote
				if (string.charAt(equals+1) == '\"') {
					quote1 = equals + 1;
				}
				else {
					quote1 = equals + 2;
				}
				quote2 = string.indexOf('\"', quote1+1);
				
				//the next char after closing quote should be a comma
				comma = string.length();
				if (quote2+1 < comma && string.charAt(quote2+1) == ',') {
					comma = quote2+1;
				}
				
				//extract the value
				value = string.substring(quote1+1, quote2).trim();
			}
			else {
				//find the next comma
				comma = string.indexOf(',', equals+1);
				if (comma == -1) {
					comma = string.length();
				}
				
				//extract the property
				value = string.substring(equals+1, comma).trim();
			}
			
			//put the prop & value in the map
			map.put(property, value);
		}
	}
	
	private String getBeforeBrackets(String string) {
		int left = string.indexOf('[');
		String before = null;
		
		if (left > -1) {
			before = string.substring(0, left).trim();
		}
		
		return before;
	}
	
	/*public Rectangle2D getBoundingBox() {
		return _handler.getBoundingBox();
	}*/
	
	private String getBracketContents(String string, BufferedReader reader) {
		int left = string.indexOf('[');
		int right = string.indexOf(']');
		String brackets = null;
		
		//test if dot put the bracket content on multiple lines
		if (right == -1) {
			brackets = "";
			try {
				//read lines until we find the one with the closing bracket
				while (right == -1) {
					brackets += string.substring(left+1, string.length()).trim();
					left = -1;
					string = reader.readLine();
					string = string.trim();
					right = string.indexOf(']');
				}
				brackets += string.substring(0, right);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else {
			brackets = string.substring(left+1, right).trim();
		}
		
		return brackets;
	}
	
	public DotGraphHandler getHandler() {
		return _handler;
	}
	
	/** Reads the graph from the specified input source.
	 *
	 * @param reader the input source containing the graph in text format.
	 */
	public void readGraph(Reader reader) {
		List subgraphStack = new ArrayList();
		Map map = new HashMap();
		BufferedReader input = new BufferedReader(reader);
		try {
			String line = null;
			while ((line = input.readLine()) != null) {
				//remove the tabs (if any) from the beginning of line
				line = line.trim();
				
				//System.out.println("read line from dot: " + line);
				//find out what type of line it is
				if (line.startsWith("warning")) {
					//print out the warning
					System.out.println(line);
				}
				else if (line.endsWith("{")) {
					//extract the label
					String label = "";
					if (line.length() > 1) {
						//should be "subgraph LABEL {"
						label = line.substring(line.indexOf(' ')+1, line.lastIndexOf(' '));
					}
					
					//add subgraph to top of stack
					subgraphStack.add(label);
					
					//notify handler
					_handler.subgraphDefinition(label);
				}
				else if (line.startsWith("}")) {
					//pop subgraph off top of stack
					String label = (String) subgraphStack.remove(subgraphStack.size()-1);
					
					//notify handler
					_handler.subgraphTermination(label);
				}
				else if (line.startsWith("graph")) {
					//get the attributes
					getAttributes(getBracketContents(line, input), map);
					
					//notify the handler
					_handler.subgraphAttribute(map);
				}
				else if (line.startsWith("node")) {
					//get the attributes
					getAttributes(getBracketContents(line, input), map);
					
					//notify the handler
					_handler.nodeAttribute(map);
				}
				else if (line.startsWith("edge")) {
					//get the attributes
					getAttributes(getBracketContents(line, input), map);
					
					//notify the handler
					_handler.edgeAttribute(map);
				}
				else if (line.indexOf("->") > 0) {
					//split up the line
					String before = getBeforeBrackets(line);
					String after = getBracketContents(line, input);
					
					//get node labels
					String tail = removeQuotes(before.substring(0, before.indexOf(' ')));
					String head = removeQuotes(before.substring(before.lastIndexOf(' ')+1, before.length()));
					
					//get the node attributes
					getAttributes(after, map);
					
					//notify the handler
					_handler.edgeDefinition(tail, head, map);
				}
				else if (line.indexOf("[") > 0) { //must be a node definition
					//split up the line
					String before = getBeforeBrackets(line);
					String after = getBracketContents(line, input);
					
					//get the node label
					String label = removeQuotes(before);
					
					//get the node attributes
					getAttributes(after, map);
					
					//notify the handler
					_handler.nodeDefinition(label, map);
				}
			}
		} catch (IOException e) {
			System.out.println("error reading back from dot");
			e.printStackTrace();
		}/* catch (NullPointerException e) {
			System.out.println("error reading back from dot");
			e.printStackTrace();
		}*/
	}
	
	private String removeQuotes(String quoted) {
		String noQuotes = quoted;
		
		if (quoted.startsWith("\"") && quoted.endsWith("\"")) {
			noQuotes = quoted.substring(1, quoted.length()-1);
		}
		
		return noQuotes;
	}
	
}

/* 
 * $Log: DotGraphReader.java,v $
 * Revision 1.1  2002/02/15 20:06:46  zcox
 * big time update
 *
 *
 *
 */
