/* $Id: DotUtilities.java,v 1.2 2002/05/16 21:11:12 zcox Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import diva.graph.model.Node;
import diva.graph.model.Edge;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class DotUtilities {
	
	/** Creates new DotUtilities */
	private DotUtilities() {
		
	}
	
	public static String addQuotes(String string) {
		String quotes = string;
		
		//needs quotes - note that there could be other criteria for needing quotes!
		if ((quotes.indexOf(" ") != -1 || quotes.indexOf("_") != -1 || quotes.indexOf("-") != -1) || Character.isDigit(quotes.charAt(0)) && (!quotes.startsWith("\"") && !quotes.endsWith("\""))) {
			quotes = "\"" + string + "\"";
		}
		
		return quotes;
	}
	
	public static String cleanCoordinates(String coordinates) {
		StringBuffer sb = new StringBuffer(coordinates);
		int i = 0, comma = -1, lastComma = -1, space = -1, slash = -1;
		char current = 0;
		
		//step through each char in sb
		while (i < sb.length()) {
			//get the current char
			current = sb.charAt(i);
			
			//see if it's meaningful
			if (current == ' ') {
				//save the position
				space = i;
			}
			else if (current == '\\') {
				//save the position
				slash = i;
			}
			else if (current == ',') {
				//save the position
				comma = i;
				
				//if this isn't the first comma in the string
				if (lastComma > -1) {
					//if there was a slash between lastComma and this comma
					if (slash > -1) {
						//if there was no space between lastComma and this comma
						if (space < 0) {
							//replace the slash with a space
							sb.setCharAt(slash, ' ');
						}
						else {
							//delete the slash
							sb.deleteCharAt(slash);
							
							//that shortened the string by one, so update the comma's position
							comma--;
							
							//also update the counter so we look at the correct char next time
							i--;
						}
					}
				}
				
				//reset variables
				space = -1;
				slash = -1;
				lastComma = comma;
			}
			
			//look at the next char
			i++;
			
			//make sure there's not a slash at the end
			if (i == sb.length() && slash > -1) {
				sb.deleteCharAt(slash);
			}
		}
		
		return sb.toString();
	}
	
	public static String cleanNumber(String string) {
		StringBuffer sb = new StringBuffer(string);
		int i = 0;
		char current = 0;
		
		while (i < sb.length()) {
			current = sb.charAt(i);
			if (current == '\\') {
				sb.deleteCharAt(i);
			}
			else {
				i++;
			}
		}
		
		return sb.toString();
	}
	
	public static List getEdgeCoordinates(String coords) {
		String xy = null;
		char arrowhead = 0;
		List pts = new ArrayList();
		StringTokenizer tknzr = null;
		double x = 0.0d, y = 0.0d;
		Point2D pt1 = null, pt2 = null, pt3 = null;
		
		coords = cleanCoordinates(coords);
		
		//get the arrowhead indicator
		//'s' means the first x,y is for the arrowhead - ignore first x,y
		//'e' means the last x,y is for the arrowhead - ignore last x,y
		arrowhead = coords.charAt(0);
		
		//get the control points
		//(xs|e,ys|e) x0,y0 x1,y1 x2,y2 x3,y3 ... x3n,y3n
		tknzr = new StringTokenizer(coords);
		while (tknzr.hasMoreTokens()) {
			xy = tknzr.nextToken();
			
			//ignore the first control pt
			if (xy.charAt(0) == arrowhead) {
				continue;
			}
			
			//parse & add the control point
			/*try {
				x = Double.parseDouble(xy.substring(0, xy.indexOf(',')));
				y = Double.parseDouble(xy.substring(xy.indexOf(',')+1, xy.length()));
			} catch (NumberFormatException e) {
				System.out.println("edge = " + tail + " -> " + head);
				System.out.println("pos = " + pos);
				System.out.println("xy = " + xy);
				e.printStackTrace();
			}*/
			pts.add(getPoint(xy, null));
		}
		
		//reverse the control pts if needed
		if (arrowhead == 's') {
			Collections.reverse(pts);
		}
		
		return pts;
	}
	
	public static String getEdgeName(Edge edge) {
		//return getNodeName(edge.getTail()) + " -> " + getNodeName(edge.getHead());
		String name = null;
		try {
			name = getNodeName(edge.getTail()) + " -> " + getNodeName(edge.getHead());
		} catch (NullPointerException e) {
			System.out.println(edge.getSemanticObject());
		}
		
		return name;
	}
	
	public static String getNodeName(Node node) {
		return (String) node.getSemanticObject();
	}
	
	public static Point2D getPoint(String xy, Point2D point) {
		Point2D p = null;
		double x = 0.0d, y = 0.0d;
		
		x = Double.parseDouble(xy.substring(0, xy.indexOf(',')));
		y = Double.parseDouble(xy.substring(xy.indexOf(',')+1, xy.length()));
		
		if (point == null) {
			p = new Point2D.Double(x, y);
			//p.setLocation(x, y);
			return p;
		}
		else {
			point.setLocation(x, y);
			return point;
		}
	}
	
	public static String getQuotedNodeName(Node node) {
		return addQuotes((String) node.getSemanticObject());
	}
	
	public static String getQuotedEdgeName(Edge edge) {
		//return addQuotes(getNodeName(edge.getTail())) + " -> " + addQuotes(getNodeName(edge.getHead()));
		String name = null;
		try {
			name = addQuotes(getNodeName(edge.getTail())) + " -> " + addQuotes(getNodeName(edge.getHead()));
		} catch (NullPointerException e) {
			System.out.println(edge.getSemanticObject());
		}
		
		return name;
	}
	
	public static Rectangle2D getRectangle(String coords, Rectangle2D rect) {
		Rectangle2D r = null;
		double x = 0.0d, y = 0.0d, width = 0.0d, height = 0.0d;
		int comma = -1, lastComma = -1;
		
		//get the x
		comma = coords.indexOf(',');
		x = Double.parseDouble(coords.substring(0, comma));
		lastComma = comma;
		
		//get the y
		comma = coords.indexOf(',', comma+1);
		y = Double.parseDouble(coords.substring(lastComma+1, comma));
		lastComma = comma;
		
		//get the width
		comma = coords.indexOf(',', comma+1);
		width = Double.parseDouble(coords.substring(lastComma+1, comma)) - x;
		lastComma = comma;
		
		//get the height
		comma = coords.indexOf(',', comma+1);
		height = Double.parseDouble(coords.substring(lastComma+1, coords.length())) - y;
		
		if (rect == null) {
			r = new Rectangle2D.Double(x, y, width, height);
			return r;
		}
		else {
			rect.setRect(x, y, width, height);
			return rect;
		}
	}
}

/* 
 * $Log: DotUtilities.java,v $
 * Revision 1.2  2002/05/16 21:11:12  zcox
 * not sure...
 *
 * Revision 1.1  2002/01/15 20:29:28  zcox
 * initial commit of fcmodeler.io.dot package
 *
 * 
 */
