/* $Id: RankClusterDotHandler.java,v 1.4 2002/02/15 20:06:46 zcox Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.Iterator;
import java.awt.Font;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import diva.graph.GraphView;
import diva.canvas.Figure;
import diva.canvas.FigureLayer;
import diva.canvas.toolbox.LabelFigure;
import diva.canvas.connector.PerimeterSite;
import diva.canvas.connector.Arrowhead;

import fcmodeler.FCModeler;
import fcmodeler.view.NodeFigure;
import fcmodeler.view.BezierEdgeFigure;
import fcmodeler.view.ClusterFigure;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public class RankClusterDotHandler extends AbstractDotHandler {
	private SimpleDotHandler _handler = null;
	private boolean _ranks = false;
	private boolean _cluster = false;
	private String _clusterLabel = null;
	private Map _rankMap = null;
	private Set _figures = null;
	
	/** Creates new RankClusterDotHandler */
	public RankClusterDotHandler(GraphView graphView) {
		super(graphView);
		
		//_handler = (SimpleDotHandler) SimpleDotFactory.getInstance().createDotHandler(graphView);
		_handler = new SimpleDotHandler(graphView);
		_rankMap = new HashMap();
		_figures = new HashSet();
	}
	
	public void configure(FCModeler fcmodeler) {
		GraphView view = fcmodeler.getGraphView();
		FigureLayer layer = view.getGraphicsPane().getForegroundLayer();
		
		for (Iterator iterator = _figures.iterator(); iterator.hasNext();) {
			Figure figure = (Figure) iterator.next();
			
			//add the figure to the FigureLayer
			layer.add(layer.getFigureCount(), figure);
		}
		
		_handler.configure(fcmodeler);
	}
	
	public void edgeAttribute(Map attributes) {
		
	}
	
	public void edgeDefinition(String tail, String head, Map attributes) {
		//see if this is a rank edge
		if (_ranks) {
			//get the figures for the tail & head nodes
			NodeFigure tailFigure = (NodeFigure) _rankMap.get(tail);
			NodeFigure headFigure = (NodeFigure) _rankMap.get(head);
			
			//make sure they exist
			if (tailFigure != null && headFigure != null) {
				//create sites on tail & head figures
				PerimeterSite tailSite = new PerimeterSite(tailFigure, 0, 1.0);
				PerimeterSite headSite = new PerimeterSite(headFigure, 0, 1.0);
				
				//create the rank edge figure & configure it
				BezierEdgeFigure edgeFigure = new BezierEdgeFigure(tailSite, headSite);
				edgeFigure.setUserObject(null);
				
				//position it appropriately
				List pts = DotUtilities.getEdgeCoordinates((String) attributes.get("pos"));
				edgeFigure.setControlPoints(pts);
				
				//put an arrowhead on it
				edgeFigure.setHeadEnd(new Arrowhead(headSite.getX(), headSite.getY(), headSite.getNormal()));
				
				_figures.add(edgeFigure);
			}
		}
		else {
			//it wasn't a rank edge, so let _handler deal with it
			_handler.edgeDefinition(tail, head, attributes);
		}
	}
	
	public void nodeAttribute(Map attributes) {
		
	}
	
	public void nodeDefinition(String label, Map attributes) {
		//see if this is a rank node
		if (_ranks) {
			//create a figure for the rank node & configure it
			LabelFigure labelFigure = new LabelFigure(label);
			labelFigure.setFont(new Font("Serif", Font.ITALIC, 16));
			NodeFigure figure = new NodeFigure(labelFigure);
			figure.setUserObject(null);
			figure.getFigure().setStrokePaint(Color.white);
			/*NodeFigure figure = new NodeFigure(new LabelFigure(label));
			figure.setUserObject(null);
			figure.getLabelFigure().setFont(new Font("Serif", Font.ITALIC, 16));
			figure.getFigure().setStrokePaint(Color.white);*/
			
			//position the figure appropriately
			Point2D pt = DotUtilities.getPoint((String) attributes.get("pos"), null);
			Rectangle2D bounds = figure.getBounds();
			figure.translate(pt.getX() - bounds.getWidth()/2 - bounds.getX(), _boundingBox.getHeight() - pt.getY() - bounds.getHeight()/2 - bounds.getY());
			
			_figures.add(figure);
			//add the figure to the FigureLayer
			//FigureLayer layer = _graphView.getGraphicsPane().getForegroundLayer();
			//layer.add(layer.getFigureCount(), figure);
			
			//put the figure in the map so rank edges can get it
			_rankMap.put(label, figure);
		}
		else {
			//it's not a rank node, so let _handler deal with it
			_handler.nodeDefinition(label, attributes);
		}
	}
	
	public void setEdgeMap(Map edgeMap) {
		super.setEdgeMap(edgeMap);
		_handler.setEdgeMap(edgeMap);
	}
	
	public void setNodeMap(Map nodeMap) {
		super.setNodeMap(nodeMap);
		_handler.setNodeMap(nodeMap);
	}
	
	public void subgraphAttribute(Map attributes) {
		String bb = (String) attributes.get("bb");
		String label = (String) attributes.get("label");
		
		_handler.subgraphAttribute(attributes);
		
		if (bb != null && _boundingBox == null) {
			_boundingBox = _handler.getBoundingBox();
		}
		else if (_cluster && bb != null) {
			//create a new cluster figure
			Rectangle2D rect = DotUtilities.getRectangle(bb, null);
			rect.setRect(rect.getX(), _boundingBox.getHeight() - (rect.getY() + rect.getHeight()), rect.getWidth(), rect.getHeight());
			ClusterFigure figure = new ClusterFigure(rect, _clusterLabel);
			figure.getLabel().setFont(new Font("Serif", Font.ITALIC, 16));
			
			_figures.add(figure);
			//add the figure to the FigureLayer
			//FigureLayer layer = _graphView.getGraphicsPane().getForegroundLayer();
			//layer.add(layer.getFigureCount(), figure);
		}
		//label should always come before bb
		else if (_cluster && label != null) {
			_clusterLabel = label;
		}
	}
	
	public void subgraphDefinition(String label) {
		_handler.subgraphDefinition(label);
		
		if (label.equals("ranks")) {
			_ranks = true;
		}
		else if (label.startsWith("cluster")) {
			_cluster = true;
		}
	}
	
	public void subgraphTermination(String label) {
		_handler.subgraphTermination(label);
		
		if (label.equals("ranks")) {
			_ranks = false;
		}
		else if (label.startsWith("cluster")) {
			_cluster = false;
		}
	}
}

/* 
 * $Log: RankClusterDotHandler.java,v $
 * Revision 1.4  2002/02/15 20:06:46  zcox
 * big time update
 *
 * Revision 1.3  2002/01/22 23:02:58  zcox
 * modifications to EdgeFigure classes
 *
 * Revision 1.2  2002/01/18 19:53:50  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.1  2002/01/15 20:29:28  zcox
 * initial commit of fcmodeler.io.dot package
 *
 * 
 */
