/* $Id: RankClusterDotWriter.java,v 1.2 2002/02/15 20:06:46 zcox Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.io.Reader;
import java.io.Writer;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;

import diva.graph.model.Graph;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.BasicGraph;

import fcmodeler.FCModeler;
import fcmodeler.io.AbstractGraphWriter;
import fcmodeler.layout.Rank;
import fcmodeler.layout.Cluster;
import fcmodeler.mapping.Sentence;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class RankClusterDotWriter extends AbstractGraphWriter implements DotGraphWriter {
	private List _ranks = null;
	private List _clusters = null;
	private int _clusterNum = 0;
	private Map _nodeMap = null;
	private Map _edgeMap = null;
	
	/** Creates new RankClusterDotWriter */
	public RankClusterDotWriter(Reader layoutXML) {
		RankClusterReader reader = new RankClusterReader(layoutXML);
		_ranks = reader.getRanks();
		_clusters = reader.getClusters();
	}
	
	private boolean evaluate(Node node, Set set) {
		boolean evaluation = false;
		
		for (Iterator iterator = set.iterator(); iterator.hasNext();) {
			Sentence sentence = (Sentence) iterator.next();
			if ((evaluation = evaluation || sentence.evaluate(node))) { break; }
		}
		
		return evaluation;
	}
	
	public Map getEdgeMap() {
		return _edgeMap;
	}
	
	public Map getNodeMap() {
		return _nodeMap;
	}
	
	private void writeCluster(Cluster cluster, Graph graph, PrintWriter out, Map clusterNodes) {
		out.println("subgraph cluster" + _clusterNum++ + " {");
		out.println("label = " + DotUtilities.addQuotes(cluster.getLabel()) + ";");
		
		for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
			Node node = (Node) nodes.next();
			
			if (clusterNodes.get(node) == null && evaluate(node, cluster.getSentences())) {
				out.print(DotUtilities.getQuotedNodeName(node) + "; ");
				clusterNodes.put(node, cluster);
			}
		}
		
		for (Iterator clusters = cluster.getSubclusters().iterator(); clusters.hasNext();) {
			writeCluster((Cluster) clusters.next(), graph, out, clusterNodes);
		}
		
		out.println("}");
	}
	
	/** Writes the specified graph to the specified output.
	 *
	 * @param graph the graph to write.
	 * @param writer the output where the graph in text format will go.
	 */
	public void writeGraph(FCModeler fcmodeler, Writer writer) {
		if (isWritingTopology()) {
			PrintWriter output = new PrintWriter(writer);
			Graph graph = fcmodeler.getGraph();
			Map rankNodes = new HashMap(((BasicGraph) graph).getNodeCount());
			Map clusterNodes = new HashMap(((BasicGraph) graph).getNodeCount());
			_nodeMap = new HashMap();
			_edgeMap = new HashMap();

			//System.out.println("RankClusterDotWriter.writeGraph(), output = " + output + ", graph = " + graph);
			//System.out.println("writing beginning of graph");
			//print beginning of graph
			output.println("digraph G {");
			output.println("ratio = auto;");
			output.println("center = true;");
			output.println("ranksep = 2.5;");
			output.println("rotate = 90;");

			//System.out.println("writing the rank definitions");
			//print the ranks
			output.println();
			output.println("subgraph ranks {");
			output.println("node [shape = plaintext];");
			for (int i=0, size=_ranks.size(); i<size; i++) {
				output.print(_ranks.get(i).toString());
				if (i < _ranks.size()-1) {
					output.print(" -> ");
				}
			}
			output.println(";");
			output.println("}");
			output.println();

			//System.out.println("writing the clusters");
			//print the clusters
			_clusterNum = 0;
			for (Iterator clusterIterator = _clusters.iterator(); clusterIterator.hasNext();) {
				writeCluster((Cluster) clusterIterator.next(), graph, output, clusterNodes);
			}

			//System.out.println("writing the rank=same's");
			//print the rank = same's
			for (Iterator rankIterator = _ranks.iterator(); rankIterator.hasNext();) {
				Rank rank = (Rank) rankIterator.next();
				output.print("{ rank = same; " + DotUtilities.addQuotes(rank.toString()) + "; ");
				for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
					Node node = (Node) nodes.next();
					if (rankNodes.get(node) == null && evaluate(node, rank.getSentences())) {
						output.print(DotUtilities.getQuotedNodeName(node) + "; ");
						rankNodes.put(node, rank);
					}
				}
				output.println("}");
			}
			output.println("");

			//System.out.println("writing nodes that weren't in a rank=same statement or a cluster");
			//print the nodes that weren't in a {rank = same} statement or a cluster
			_nodeMap.clear();
			for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
				Node node = (Node) nodes.next();
				String nodeName = DotUtilities.getNodeName(node);
				_nodeMap.put(nodeName, node);
				if (rankNodes.get(node) == null && clusterNodes.get(node) == null) {
					output.println(DotUtilities.addQuotes(nodeName) + ";");
				}
			}
			output.println("");

			//System.out.println("writing the edges");
			//print the edges
			_edgeMap.clear();
			for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
				for (Iterator outEdges = ((Node) nodes.next()).outEdges(); outEdges.hasNext();) {
					Edge edge = (Edge) outEdges.next();
					String edgeName = DotUtilities.getEdgeName(edge);
					_edgeMap.put(edgeName, edge);
					output.println(DotUtilities.getQuotedEdgeName(edge) + ";");
				}
			}

			//print the closing
			output.println("}");

			//System.out.println("finishing writing the graph");
			//finish the output
			output.flush();
			output.close();
		}
	}
	
}

/* 
 * $Log: RankClusterDotWriter.java,v $
 * Revision 1.2  2002/02/15 20:06:46  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 20:29:28  zcox
 * initial commit of fcmodeler.io.dot package
 *
 * 
 */
