/* $Id: RankClusterReader.java,v 1.1 2002/02/15 20:06:46 zcox Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.io.Reader;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import fcmodeler.mapping.Sentence;
import fcmodeler.mapping.AtomicSentence;
import fcmodeler.mapping.CompositeSentence;
import fcmodeler.mapping.Connective;
import fcmodeler.mapping.AndConnective;
import fcmodeler.mapping.OrConnective;
import fcmodeler.mapping.NodeMapping;
import fcmodeler.mapping.EdgeMapping;
import fcmodeler.util.NoTextNodeList;
import fcmodeler.util.ChildNodeList;
import fcmodeler.layout.Rank;
import fcmodeler.layout.Cluster;

/** 
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class RankClusterReader extends Object {
	private List _ranks = null;
	private List _clusters = null;
	
	/** Creates new RankClusterReader */
	public RankClusterReader(Reader reader) {
		createRanksAndClusters(reader);
	}
	
	private void createRanksAndClusters(Reader reader) {
		try {
			DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = builderFactory.newDocumentBuilder();
			Document document = builder.parse(new InputSource(reader));
			Element root = document.getDocumentElement();
			
			_ranks = new ArrayList();
			NodeList nodeList = new ChildNodeList(root, "rank");
			if (nodeList.getLength() > 0) {
				createRanks(nodeList);
			}
			
			_clusters = new ArrayList();
			nodeList = new ChildNodeList(root, "cluster");
			if (nodeList.getLength() > 0) {
				_clusters.addAll(createClusters(nodeList));
			}
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void createRanks(NodeList rankList) {
		for (int i=0, length=rankList.getLength(); i<length; i++) {
			Element rankElement = (Element) rankList.item(i);
			Set sentences = new HashSet();
			NodeList atomList = new ChildNodeList(rankElement, "atom");
			for (int j=0, length2=atomList.getLength(); j<length2; j++) {
				Element atomElement = (Element) atomList.item(j);
				sentences.add(buildSentence(atomElement));
			}
			NodeList compositeList = new ChildNodeList(rankElement, "composite");
			for (int j=0, length2=compositeList.getLength(); j<length2; j++) {
				Element compositeElement = (Element) compositeList.item(j);
				sentences.add(buildSentence(compositeElement));
			}
			Rank rank = new Rank(rankElement.getAttribute("label"), sentences);
			_ranks.add(rank);
		}
	}
	
	private Sentence buildSentence(Element sentenceElement) {
		Sentence sentence = null;
		if (sentenceElement.getTagName().equals("atom")) {
			sentence = new AtomicSentence(sentenceElement.getAttribute("property"), sentenceElement.getAttribute("value"));
		}
		else if (sentenceElement.getTagName().equals("composite")) {
			NodeList children = new NoTextNodeList(sentenceElement.getChildNodes());
			Element firstElement = (Element) children.item(0);
			Sentence first = buildSentence(firstElement);
			Element connectiveElement = (Element) children.item(1);
			Connective connective = connectiveElement.getAttribute("type").equals("and") ? (Connective) AndConnective.getInstance() : (Connective) OrConnective.getInstance();
			Element secondElement = (Element) children.item(2);
			Sentence second = buildSentence(secondElement);
			sentence = new CompositeSentence(first, connective, second);
		}
		return sentence;
	}
	
	private List createClusters(NodeList clusterList) {
		List clusters = new ArrayList();
		for (int i=0, length=clusterList.getLength(); i<length; i++) {
			Element clusterElement = (Element) clusterList.item(i);
			Set sentences = new HashSet();
			NodeList atomList = new ChildNodeList(clusterElement, "atom");
			for (int j=0, length2=atomList.getLength(); j<length2; j++) {
				Element atomElement = (Element) atomList.item(j);
				sentences.add(buildSentence(atomElement));
			}
			NodeList compositeList = new ChildNodeList(clusterElement, "composite");
			for (int j=0, length2=compositeList.getLength(); j<length2; j++) {
				Element compositeElement = (Element) compositeList.item(j);
				sentences.add(buildSentence(compositeElement));
			}
			NodeList subclusterList = clusterElement.getElementsByTagName("cluster");
			List subclusters = createClusters(subclusterList);
			Cluster cluster = new Cluster(clusterElement.getAttribute("label"), sentences, subclusters);
			clusters.add(cluster);
		}
		return clusters;
	}
	
	public List getClusters() {
		return _clusters;
	}
	
	public List getRanks() {
		return _ranks;
	}
	
}

/* 
 * $Log: RankClusterReader.java,v $
 * Revision 1.1  2002/02/15 20:06:46  zcox
 * big time update
 *
 *
 *
 */
