/* $Id: SimpleDotHandler.java,v 1.5 2002/07/13 07:48:12 adamtj Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.util.StringTokenizer;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import diva.graph.GraphView;
import diva.graph.layout.LayoutUtilities;
import diva.graph.model.Node;
import diva.graph.model.Edge;

import fcmodeler.FCModeler;
import fcmodeler.view.EdgeFigure;
import fcmodeler.view.BezierEdgeFigure;
import fcmodeler.interaction.GeneralConnector.GeneralConnector;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */

public class SimpleDotHandler extends AbstractDotHandler {
	private String _digraph = null;
	private Map _nodePts = null;
	private Map _edgePts = null;
	
	/** Creates new SimpleDotReader */
	public SimpleDotHandler(GraphView graphView) {
		super(graphView);
		_nodePts = new HashMap();
		_edgePts = new HashMap();
	}
	
	public void configure(FCModeler fcmodeler) {
		GraphView view = fcmodeler.getGraphView();
		
		for (Iterator iterator = _nodePts.entrySet().iterator(); iterator.hasNext();) {
			Map.Entry entry = (Map.Entry) iterator.next();
			Node node = (Node) entry.getKey();
			Point2D pt = (Point2D) entry.getValue();
			
			//place the node at (x,y)
			LayoutUtilities.placeNoReroute(view, node, pt.getX(), _boundingBox.getHeight() - pt.getY());
		}
		
		for (Iterator iterator = _edgePts.entrySet().iterator(); iterator.hasNext();) {
			Map.Entry entry = (Map.Entry) iterator.next();
			Edge edge = (Edge) entry.getKey();
			List pts = (List) entry.getValue();
			
			//set the control points for the edge figure
			EdgeFigure edgeFigure = (EdgeFigure) view.getEdgeFigure(edge);
			if (edgeFigure instanceof BezierEdgeFigure) {
				((BezierEdgeFigure) edgeFigure).setControlPoints(pts);
			} else if (edgeFigure instanceof GeneralConnector) {
				((GeneralConnector) edgeFigure).fromFile(pts);
			} else {
				throw new UnsupportedOperationException("SimpleDotHandler.configure(): EdgeFigure not of the right type.");
			}
		}
		
		LayoutUtilities.routeVisibleEdges(view.getGraphModel().getGraph(), view);
	}
	
	public void edgeAttribute(Map attributes) {
		
	}
	
	public void edgeDefinition(String tail, String head, Map attributes) {
		//get the edge from the edge map
		Edge edge = (Edge) _edgeMap.get(tail + " -> " + head);
		
		if (edge != null) {
			List temp = DotUtilities.getEdgeCoordinates((String) attributes.get("pos"));
			List pts = new ArrayList(temp.size());
			for (Iterator iterator = temp.iterator(); iterator.hasNext();) {
				Point2D pt1 = (Point2D) iterator.next();
				pts.add(new Point2D.Double(pt1.getX(), _boundingBox.getHeight() - pt1.getY()));
			}
			
			_edgePts.put(edge, pts);
		}
	}
	
	public void nodeAttribute(Map attributes) {
		
	}
	
	public void nodeDefinition(String label, Map attributes) {
		double x = 0.0d, y = 0.0d;
		Node node = null;
		String xy = null;
		Point2D pt = null;
		
		//get the node from the node map
		node = (Node) _nodeMap.get(label);
		
		if (node != null) {
			//get the (x,y) coordinate - it's in a String as "x,y", which is the value for key "pos"
			xy = (String) attributes.get("pos");
			pt = DotUtilities.getPoint(xy, null);
			
			_nodePts.put(node, pt);
		}
	}
	
	public void subgraphAttribute(Map attributes) {
		String bb = (String) attributes.get("bb");
		
		if (bb != null && _boundingBox == null) {
			_boundingBox = DotUtilities.getRectangle(bb, null);
		}
	}
	
	public void subgraphDefinition(String label) {
		if (_digraph == null) {
			_digraph = label;
		}
	}
	
	public void subgraphTermination(String label) {
		if (label == _digraph) {
			//LayoutUtilities.routeVisibleEdges(_graphView.getGraphModel().getGraph(), _graphView);
		}
	}
}

/*
 * $Log: SimpleDotHandler.java,v $
 * Revision 1.5  2002/07/13 07:48:12  adamtj
 * Fixed a problem with edge mappings.  GeneralConnectorEdgeFigure improperly implemented EdgeFigure.  It has been removed and now GeneralConnector extends AbstractEdgeFigure.
 *
 * Revision 1.4  2002/06/27 06:08:35  adamtj
 * Activate BezierConnector edge type.  This turns on the bezier curve control point grab handles.
 *
 * Revision 1.3  2002/02/15 20:06:46  zcox
 * big time update
 *
 * Revision 1.2  2002/01/22 23:02:58  zcox
 * modifications to EdgeFigure classes
 *
 * Revision 1.1  2002/01/15 20:29:28  zcox
 * initial commit of fcmodeler.io.dot package
 *
 *
 */
