/* $Id: XMLFileFilter.java,v 1.1 2002/02/15 20:08:54 zcox Exp $
 *
 *
 * Created on January 8, 2001, 1:56 PM
 */

package fcmodeler.io.xml;

import java.io.File;

import fcmodeler.io.AbstractFileFilter;
import fcmodeler.io.GraphIOFactory;
import fcmodeler.io.GraphReader;
import fcmodeler.io.GraphWriter;

/** A file filter for XML files.
 *
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class XMLFileFilter extends AbstractFileFilter implements GraphIOFactory {
	private String _xmlExtension = "xml";

	/** Creates a new <CODE>XMLFileFilter</CODE>.
	 *
	 */
	public XMLFileFilter() {
		_description = "FCModeler Graph (*." + _xmlExtension + ")";
	}
	
	/** Returns an XML graph reader.
	 *
	 * @return an XML graph reader.
	 */	
	public GraphReader getGraphReader() {
		return new XMLGraphReader();
	}
	
	/** Returns an XML graph writer.
	 *
	 * @return an XML graph writer.
	 */	
	public GraphWriter getGraphWriter() {
		return new XMLGraphWriter();
	}
	
	/** Returns <CODE>true</CODE> if the specified file extension is for an XML file.
	 *
	 * @param extension the file extension.
	 * @return <CODE>true</CODE> if the specified file extension is for an XML file.
	 */	
	public boolean isValidExtension(String extension) {
		boolean isValid = false;
		
		if (extension.equals(_xmlExtension)) {
			isValid = true;
		}
		
		return isValid;
	}
}

/* 
 * $Log: XMLFileFilter.java,v $
 * Revision 1.1  2002/02/15 20:08:54  zcox
 * big time update
 *
 * Revision 1.4  2002/01/15 20:30:06  zcox
 * some changes
 *
 * Revision 1.3  2001/05/22 16:42:10  zcox
 * code clean-up
 *
 * Revision 1.2  2001/04/04 01:21:20  zcox
 * huge architectural changes
 *
 *
 *
 */
