/* $Id: XMLGraphWriter.java,v 1.6 2002/07/31 17:19:54 zcox Exp $
 *
 *
 */

package fcmodeler.io.xml;

import java.io.Writer;
import java.io.BufferedWriter;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.IOException;
import java.util.Set;
import java.util.Iterator;
import java.util.List;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import diva.canvas.Figure;
import diva.graph.GraphView;
import diva.graph.model.Graph;
import diva.graph.model.Edge;
import diva.graph.model.GraphUtilities.EdgeIterator;

import fcmodeler.FCModeler;
import fcmodeler.interaction.GeneralConnector.GeneralConnector;
import fcmodeler.io.AbstractGraphWriter;
import fcmodeler.mapping.NodeMapping;
import fcmodeler.mapping.EdgeMapping;
import fcmodeler.mapping.Sentence;
import fcmodeler.mapping.AtomicSentence;
import fcmodeler.mapping.CompositeSentence;
import fcmodeler.attributes.VisualAttributeValue;
import fcmodeler.view.BezierEdgeFigure;

/**
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.6 $
 * @since JDK1.3
 */

public class XMLGraphWriter extends AbstractGraphWriter {
	private FCModeler _fcmodeler = null;
	private PrintWriter _writer = null;
	
	/** Creates new XMLGraphWriter */
	public XMLGraphWriter() {
		
	}
	
	public XMLGraphWriter(boolean writeCoordinates, boolean writeMappings, boolean writeTopology) {
		writeCoordinates(writeCoordinates);
		writeMappings(writeMappings);
		writeTopology(writeTopology);
	}
	
	private void writeCoordinates(Document document) {
		Element root = document.getDocumentElement();
		Element coordsElement = document.createElement("coordinates");
		root.appendChild(coordsElement);
		
		Graph graph = _fcmodeler.getGraph();
		GraphView view = _fcmodeler.getGraphView();
		for (Iterator iterator = graph.nodes(); iterator.hasNext();) {
			diva.graph.model.Node node = (diva.graph.model.Node) iterator.next();
			Figure nodeFigure = view.getNodeFigure(node);
			Element nodeFigureElement = document.createElement("nodeFigure");
			nodeFigureElement.setAttribute("node", node.getSemanticObject().toString());
			Rectangle2D bounds = nodeFigure.getBounds();
			nodeFigureElement.setAttribute("coord", "" + bounds.getCenterX() + "," + bounds.getCenterY());
			coordsElement.appendChild(nodeFigureElement);
		}
		
		for (Iterator iterator = graph.nodes(); iterator.hasNext();) {
			diva.graph.model.Node node = (diva.graph.model.Node) iterator.next();
			for (Iterator iterator2 = node.outEdges(); iterator2.hasNext();) {
				Edge edge = (Edge) iterator2.next();
				GeneralConnector edgeFigure = (GeneralConnector) view.getEdgeFigure(edge);
				//BezierEdgeFigure edgeFigure = (BezierEdgeFigure) view.getEdgeFigure(edge);
				Element edgeFigureElement = document.createElement("edgeFigure");
				edgeFigureElement.setAttribute("edge", edge.getSemanticObject().toString());
				List pts = (List) edgeFigure.toFile();
				//List pts = edgeFigure.getControlPoints();
				StringBuffer sb = new StringBuffer();
				for (Iterator iterator3 = pts.iterator(); iterator3.hasNext();) {
					Point2D pt = (Point2D) iterator3.next();
					sb.append("" + pt.getX() + "," + pt.getY() + " ");
				}
				sb.deleteCharAt(sb.length()-1);
				edgeFigureElement.setAttribute("coords", sb.toString());
				coordsElement.appendChild(edgeFigureElement);
			}
		}
	}
	
	private void writeDeclaration() {
		_writer.println("<?xml version=\"1.0\" standalone=\"yes\"?>");
		_writer.println("<!DOCTYPE graphFile [");
		_writer.println("<!ELEMENT graphFile (graph*, mappings*, coordinates*)>");
		
		if (isWritingCoordinates()) {
			_writer.println("<!ELEMENT coordinates (nodeFigure*, edgeFigure*)>");
			_writer.println("<!ELEMENT nodeFigure (EMPTY) >");
			_writer.println("<!ATTLIST nodeFigure");
			_writer.println("node CDATA #REQUIRED");
			_writer.println("coord CDATA #REQUIRED>");
			_writer.println("<!ELEMENT edgeFigure (EMPTY) >");
			_writer.println("<!ATTLIST edgeFigure");
			_writer.println("edge CDATA #REQUIRED");
			_writer.println("coords CDATA #REQUIRED>");
		}
		
		if (isWritingMappings()) {
			_writer.println("<!ELEMENT mappings (mapping*)>");
			_writer.println("<!ELEMENT mapping ((atom | composite), attributeValue)>");
			_writer.println("<!ATTLIST mapping");
			_writer.println("type (node | edge) #REQUIRED>");
			_writer.println("<!ELEMENT atom EMPTY>");
			_writer.println("<!ATTLIST atom");
			_writer.println("property CDATA #REQUIRED");
			_writer.println("value CDATA #REQUIRED>");
			_writer.println("<!ELEMENT composite ((atom | composite), connective, (atom | composite))>");
			_writer.println("<!ELEMENT connective EMPTY>");
			_writer.println("<!ATTLIST connective");
			_writer.println("type (and | or) #REQUIRED>");
			_writer.println("<!ELEMENT attributeValue EMPTY>");
			_writer.println("<!ATTLIST attributeValue");
			_writer.println("attribute CDATA #REQUIRED");
			_writer.println("value CDATA #REQUIRED>");
		}
		
		if (isWritingTopology()) {
			_writer.println("<!ELEMENT graph (nodeProperties*, node*, compositeNode*, edgeProperties*, edge*)>");
			_writer.println("<!ELEMENT nodeProperties (#PCDATA)>");
			_writer.println("<!ELEMENT node (#PCDATA)>");
			_writer.println("<!ATTLIST node");
			_writer.println("id ID #REQUIRED>");
			_writer.println("<!ELEMENT edgeProperties (#PCDATA)>");
			_writer.println("<!ELEMENT edge (#PCDATA)>");
			_writer.println("<!ATTLIST edge");
			_writer.println("id ID #REQUIRED");
			_writer.println("head IDREF #REQUIRED");
			_writer.println("tail IDREF #REQUIRED");
			_writer.println("directed (true|false) \"false\">");
		}
		
		_writer.println("]>");
	}
	
	/** Writes the specified graph to the specified output.
	 *
	 * @param graph the graph to write.
	 * @param writer the output where the graph in text format will go.
	 */
	public void writeGraph(FCModeler fcmodeler, Writer writer) {
		_fcmodeler = fcmodeler;
		_writer = new PrintWriter(new BufferedWriter(writer));
		
		if (isWritingCoordinates() || isWritingMappings() || isWritingTopology()) {
			try {
				DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = builderFactory.newDocumentBuilder();
				Document document = builder.newDocument();
				Element root = document.createElement("graphFile");
				document.appendChild(root);
				
				writeDeclaration();
				
				if (isWritingCoordinates()) {
					writeCoordinates(document);
				}
				
				if (isWritingMappings()) {
					writeMappings(document);
				}
				
				if (isWritingTopology()) {
					writeTopology(document);
				}
				
				TransformerFactory txfmFactory = TransformerFactory.newInstance();
				Transformer transformer = txfmFactory.newTransformer();
				transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
				transformer.setOutputProperty(OutputKeys.INDENT, "yes");
				transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
				DOMSource source = new DOMSource(document);
				StringWriter xml = new StringWriter();
				StreamResult result = new StreamResult(xml);
				transformer.transform(source, result);
				
				xml.flush();
				_writer.flush();
				_writer.print(xml.toString());
				
				_writer.close();
				writer.close();
				xml.close();
			} catch (TransformerConfigurationException e) {
				e.printStackTrace();
			} catch (TransformerException e) {
				e.printStackTrace();
			} catch (ParserConfigurationException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	private void writeMappings(Document document) {
		Element root = document.getDocumentElement();
		Element mappingsElement = document.createElement("mappings");
		root.appendChild(mappingsElement);
		
		for (Iterator iterator = _fcmodeler.getNodeMappings().iterator(); iterator.hasNext();) {
			NodeMapping mapping = (NodeMapping) iterator.next();
			Element mappingElement = document.createElement("mapping");
			mappingElement.setAttribute("type", "node");
			writeSentence(mapping.getSentence(), mappingElement, document);
			Element attributeValueElement = document.createElement("attributeValue");
			VisualAttributeValue value = (VisualAttributeValue) mapping.getValue();
			attributeValueElement.setAttribute("attribute", value.getAttribute().toString());
			attributeValueElement.setAttribute("value", value.toString());
			mappingElement.appendChild(attributeValueElement);
			mappingsElement.appendChild(mappingElement);
		}
		
		for (Iterator iterator = _fcmodeler.getEdgeMappings().iterator(); iterator.hasNext();) {
			EdgeMapping mapping = (EdgeMapping) iterator.next();
			Element mappingElement = document.createElement("mapping");
			mappingElement.setAttribute("type", "edge");
			writeSentence(mapping.getSentence(), mappingElement, document);
			Element attributeValueElement = document.createElement("attributeValue");
			VisualAttributeValue value = (VisualAttributeValue) mapping.getValue();
			attributeValueElement.setAttribute("attribute", value.getAttribute().toString());
			attributeValueElement.setAttribute("value", value.toString());
			mappingElement.appendChild(attributeValueElement);
			mappingsElement.appendChild(mappingElement);
		}
	}
	
	private void writeSentence(Sentence sentence, Element element, Document document) {
		if (sentence instanceof AtomicSentence) {
			AtomicSentence atom = (AtomicSentence) sentence;
			Element atomElement = document.createElement("atom");
			atomElement.setAttribute("property", atom.getProperty());
			atomElement.setAttribute("value", atom.getPropertyValue().toString());
			element.appendChild(atomElement);
		}
		else if (sentence instanceof CompositeSentence) {
			CompositeSentence composite = (CompositeSentence) sentence;
			Element compositeElement = document.createElement("composite");
			writeSentence(composite.getFirstSentence(), compositeElement, document);
			Element connectiveElement = document.createElement("connective");
			connectiveElement.setAttribute("type", composite.getConnective().toString());
			compositeElement.appendChild(connectiveElement);
			writeSentence(composite.getSecondSentence(), compositeElement, document);
			element.appendChild(compositeElement);
		}
	}
	
	private void writeTopology(Document document) {
		Element root = document.getDocumentElement();
		Element graphElement = document.createElement("graph");
		root.appendChild(graphElement);
		
		Graph graph = _fcmodeler.getGraph();
		List nodeProperties = (List) graph.getProperty("nodeProperties");
		StringBuffer sb = new StringBuffer();
		for (Iterator iterator = nodeProperties.iterator(); iterator.hasNext();) {
			sb.append("\"" + iterator.next().toString() + "\" ");
		}
		Element nodePropertiesElement = document.createElement("nodeProperties");
		nodePropertiesElement.appendChild(document.createTextNode(sb.toString()));
		graphElement.appendChild(nodePropertiesElement);
		for (Iterator iterator = graph.nodes(); iterator.hasNext();) {
			diva.graph.model.Node node = (diva.graph.model.Node) iterator.next();
			Element nodeElement = document.createElement("node");
			nodeElement.setAttribute("id", node.getSemanticObject().toString());
			sb = new StringBuffer();
			for (Iterator iterator2 = nodeProperties.iterator(); iterator2.hasNext();) {
				Object o = node.getProperty(iterator2.next().toString());
				String value = o == null ? "null" : o.toString();
				sb.append("\"" + value + "\" ");
			}
			nodeElement.appendChild(document.createTextNode(sb.toString()));
			graphElement.appendChild(nodeElement);
		}
		
		List edgeProperties = (List) graph.getProperty("edgeProperties");
		sb = new StringBuffer();
		for (Iterator iterator = edgeProperties.iterator(); iterator.hasNext();) {
			sb.append("\"" + iterator.next().toString() + "\" ");
		}
		Element edgePropertiesElement = document.createElement("edgeProperties");
		edgePropertiesElement.appendChild(document.createTextNode(sb.toString()));
		graphElement.appendChild(edgePropertiesElement);
		for (Iterator iterator = new EdgeIterator(graph); iterator.hasNext();) {
			Edge edge = (Edge) iterator.next();
			Element edgeElement = document.createElement("edge");
			edgeElement.setAttribute("id", edge.getSemanticObject().toString());
			edgeElement.setAttribute("tail", edge.getTail().getSemanticObject().toString());
			edgeElement.setAttribute("head", edge.getHead().getSemanticObject().toString());
			edgeElement.setAttribute("directed", edge.isDirected() ? "true" : "false");
			sb = new StringBuffer();
			for (Iterator iterator2 = edgeProperties.iterator(); iterator2.hasNext();) {
				Object o = edge.getProperty(iterator2.next().toString());
				String value = o == null ? "null" : o.toString();
				sb.append("\"" + value + "\" ");
			}
			edgeElement.appendChild(document.createTextNode(sb.toString()));
			graphElement.appendChild(edgeElement);
		}
	}
	
}

/*
 * $Log: XMLGraphWriter.java,v $
 * Revision 1.6  2002/07/31 17:19:54  zcox
 * graphFile dtd declaration needed to have the coordinates element
 *
 * Revision 1.5  2002/07/13 07:48:12  adamtj
 * Fixed a problem with edge mappings.  GeneralConnectorEdgeFigure improperly implemented EdgeFigure.  It has been removed and now GeneralConnector extends AbstractEdgeFigure.
 *
 * Revision 1.4  2002/06/27 06:08:35  adamtj
 * Activate BezierConnector edge type.  This turns on the bezier curve control point grab handles.
 *
 * Revision 1.3  2002/05/29 04:29:11  adamtj
 * Fixed the coordinates DTD.
 *
 * Revision 1.2  2002/03/04 05:40:07  zcox
 * added support for writing coordinates
 *
 * Revision 1.1  2002/02/15 20:08:54  zcox
 * big time update
 *
 *
 *
 */
