/*
 * BatchWGUpdater.java
 *
 * Created on May 2, 2002, 12:56 PM
 */

package fcmodeler.jsomap;

import java.util.List;
import java.util.Set;
import java.util.Iterator;

import jsomap.map.Map;
import jsomap.map.Model;
import jsomap.algorithm.parameters.Kernel;
import jsomap.algorithm.parameters.BatchUpdater;

/**
 *
 * @author  default
 * @version
 */
public class BatchWGUpdater extends Object implements BatchUpdater {
	
	/** Creates new BatchWGUpdater */
	public BatchWGUpdater() {
		
	}
	
	/** Updates the specified model using the distribution of patterns and the other parameters.  Each
	 * element of the <CODE>List</CODE> is a <CODE>Set</CODE>, containing the patterns assigned to the
	 * node at that index.
	 *
	 * @param model the model to update.
	 * @param map the map.
	 * @param kernel the kernel function.
	 * @param patterns the distribution of patterns.
	 * @param iteration the current iteration number.
	 * @throws NullPointerException if any of the parameters are <CODE>null</CODE>.
	 * @throws IllegalArgumentException if <CODE>iteration < 0</CODE>.
	 */
	public void update(Model model, Map map, Kernel kernel, List patterns, int iteration) {
		if (model == null) { throw new NullPointerException("model was null"); }
		if (map == null) { throw new NullPointerException("map was null"); }
		if (kernel == null) { throw new NullPointerException("kernel was null"); }
		if (patterns == null) { throw new NullPointerException("patterns was null"); }
		if (iteration < 0) { throw new IllegalArgumentException("iteration must be >= 0: " + iteration); }
		
		//double[] m = new double[((DoublePattern) model.getPattern()).get().length];
		WeightedGraph m = new WeightedGraph();
		
		//double[] temp = new double[m.length];
		double[] k = new double[patterns.size()];
		int[] size = new int[k.length];
		int index = map.getIndex(model);
		
		for (int j=0, size2=patterns.size(); j<size2; j++) {
			Set set = (Set) patterns.get(j);
			size[j] = set.size();
			k[j] = kernel.calculate(map.getNode(j), map.getNode(index), iteration);
			//for (int d=0, length=temp.length; d<length; d++) { temp[d] = 0.0d; }
			WeightedGraph temp = new WeightedGraph();
			
			for (Iterator iterator = set.iterator(); iterator.hasNext();) {
				//double[] x = ((DoublePattern) iterator.next()).get();
				WeightedGraph x = (WeightedGraph) iterator.next();
				//for (int d=0, length=temp.length; d<length; d++) { temp[d] += x[d]; }
				WGMath.addTo(temp, x);
			}
			
			//for (int d=0, length=temp.length; d<length; d++) { m[d] += k[j] * temp[d]; }
			//m += k[j]*temp
			WGMath.addTo(m, WGMath.multiplyBy(temp, k[j]));
		}
		
		double denominator = 0.0d;
		for (int j=0, size2=patterns.size(); j<size2; j++) {
			denominator += k[j] * size[j];
		}
		
		//for (int d=0, length=m.length; d<length; d++) { m[d] /= denominator; }
		WGMath.divideBy(m, denominator);
		
		//model.setPattern(new DoublePattern(m));
		model.setPattern(m);
	}
	
}
