/*
 * Edge.java
 *
 * Created on April 12, 2002, 8:31 AM
 */

package fcmodeler.jsomap;

/** A simple implementation of an edge in a graph.  An edge consists of two objects: the tail and the head.
 *
 *
 * @author default
 * @version
 */
public class Edge implements Comparable {
	private Object _tail = null;
	private Object _head = null;
	
	/** Creates a new <CODE>Edge</CODE> using the specfied tail and head nodes.
	 *
	 * @param tail the tail node.
	 * @param head the head node.
	 */
	public Edge(Object tail, Object head) {
		if (tail == null) { throw new NullPointerException("tail was null"); }
		if (head == null) { throw new NullPointerException("head was null"); }
		
		_tail = tail;
		_head = head;
	}
	
	/** Compares the specified object to this edge.  The object must be an edge.  Compares the tail nodes, and if
	 * the tail nodes are equal compares the head nodes.
	 *
	 * @param obj the object to compare to this edge.
	 * @return the result of the comparison.
	 */	
	public int compareTo(Object obj) {
		Edge other = (Edge) obj;
		Comparable tail = (Comparable) _tail;
		Comparable head = (Comparable) _head;
		int result = tail.compareTo(other.getTail());
		return result == 0 ? head.compareTo(other.getHead()) : result;
	}
	
	/** Returns <CODE>true</CODE> if the specified object is an edge and its tail and head nodes are equal to this
	 * edge's tail and head nodes.
	 *
	 * @param obj the object to test for equality with this edge.
	 * @return <CODE>true</CODE> if the specified object is equal to this edge.
	 */	
	public boolean equals(Object obj) {
		if (obj == this) { return true; }
		if (!(obj instanceof Edge)) { return false; }
		Edge other = (Edge) obj;
		return _tail.equals(other.getTail()) && _head.equals(other.getHead());
	}
	
	/** Returns the head node of this edge.
	 *
	 * @return the head node of this edge.
	 */	
	public Object getHead() {
		return _head;
	}
	
	/** Returns the tail node of this edge.
	 *
	 * @return the tail node of this edge.
	 */	
	public Object getTail() {
		return _tail;
	}
	
	/** Returns the hash code of this edge.
	 *
	 * @return the hash code of this edge.
	 */	
	public int hashCode() {
		int result = 17;
		result = 37*result + _tail.hashCode();
		result = 37*result + _head.hashCode();
		return result;
	}
	
	/** Returns the string representation of this edge.
	 *
	 * @return the string representation of this edge.
	 */	
	public String toString() {
		return "(" + _tail.toString() + ", " + _head.toString() + ")";
	}
	
}
