/*
 * FileWGData.java
 *
 * Created on May 3, 2002, 11:40 AM
 */

package fcmodeler.jsomap;

import java.util.List;
import java.util.ArrayList;
import java.io.Reader;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import jsomap.data.Pattern;

import fcmodeler.util.NoTextNodeList;

/**
 *
 * @author  default
 * @version
 */
public class FileWGData extends Object implements WGData {
	private List _graphs = null;
	
	/** Creates new FileWGData */
	public FileWGData(Reader reader) {
		if (reader == null) { throw new NullPointerException("reader was null"); }
		
		try {
			//open the xml file
			DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = builderFactory.newDocumentBuilder();
			Document document = builder.parse(new InputSource(reader));
			
			//create the cycle graphs
			_graphs = new ArrayList();
			NodeList nodeList = document.getElementsByTagName("cycle");
			for (int i=0, length=nodeList.getLength(); i<length; i++) {
				WeightedSet nodes = new WeightedSet();
				WeightedSet edges = new WeightedSet();
				Element cycleElement = (Element) nodeList.item(i);
				NodeList nodesEdges = new NoTextNodeList(cycleElement.getChildNodes());
				Element firstNode = (Element) nodesEdges.item(0);
				for (int j=0, length2=nodesEdges.getLength(); j<length2; j++) {
					//j is even for node elements
					if (j % 2 == 0) {
						Element node = (Element) nodesEdges.item(j);
						nodes.add(node.getAttribute("id"));
					}
					//j is odd for edge elements
					else {
						Element tail = (Element) nodesEdges.item(j-1);
						Element head = j+1 < nodesEdges.getLength() ? (Element) nodesEdges.item(j+1) : firstNode;
						edges.add(new Edge(tail.getAttribute("id"), head.getAttribute("id")));
					}
				}
				_graphs.add(new WeightedGraph(nodes, edges));
			}
		} 
		catch (ParserConfigurationException e) { e.printStackTrace(); } 
		catch (SAXException e) { e.printStackTrace(); } 
		catch (IOException e) { e.printStackTrace(); }
	}
	
	/** Returns the number of patterns being stored in this data object.
	 *
	 * @return the number of patterns in this data object.
	 */
	public int count() {
		return _graphs.size();
	}
	
	/** Returns the pattern at the specified position.
	 *
	 * @param index the index of the desired pattern.
	 * @return the pattern indexed by <CODE>index</CODE>.
	 * @throws IndexOutOfBoundsException if <CODE>index</CODE> is out of range.
	 */
	public Pattern get(int index) {
		return (Pattern) _graphs.get(index);
	}
	
	public WeightedGraph getWG(int index) {
		return (WeightedGraph) _graphs.get(index);
	}
	
	/** Sets the pattern at the specified index to the specified pattern (optional
	 * operation).
	 *
	 * @param index the index of the pattern.
	 * @param point the new pattern.
	 * @throws NullPointerException if <CODE>pattern</CODE> is null.
	 * @throws IndexOutOfBoundsException if <CODE>index</CODE> is out of range.
	 * @throws UnsupportedOperationException if this <CODE>set</CODE> method is not supported by this data object.
	 * @throws IllegalArgumentException if the specified pattern cannot be added to this data object.
	 */
	public void set(int index, Pattern pattern) {
		
	}
	
	public void setWG(int index, WeightedGraph graph) {
		
	}
	
}
