/*
 * MapPanel.java
 *
 * Created on May 9, 2002, 5:22 PM
 */

package fcmodeler.jsomap;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import jsomap.map.Map;

import fcmodeler.FCModelerFrame;
import fcmodeler.view.ScrollableJGraph;
import fcmodeler.layout.SimpleDotLayout;

/**
 *
 * @author  default
 * @version
 */
public class MapPanel extends JPanel implements MouseListener {
	private JWeightedGraph[][] _graphs = null;
	private FCModelerFrame _frame = null;
	
	/** Creates new MapPanel */
	public MapPanel(Map map, int rows, int cols) {
		if (map == null) { throw new NullPointerException("map was null"); }
		if (rows*cols != map.count()) { throw new IllegalArgumentException("rows*cols must be equal to map.count()"); }
		
		//create the model views
		JPanel panel = new JPanel();
		setLayout(new GridLayout(rows, cols));
		_graphs = new JWeightedGraph[cols][rows];
		for (int i=0; i<cols; i++) {
			for (int j=0; j<rows; j++) {
				WeightedGraph wg = (WeightedGraph) map.getModel(i*rows+j).getPattern();
				_graphs[i][j] = new JWeightedGraph(wg);
			}
		}
		
		//add the model views to this panel
		//map starts at lower-left corner and increases up
		for (int i=_graphs[0].length-1; i>=0; i--) {
			for (int j=0; j<_graphs.length; j++) {
				JScrollPane sp = new JScrollPane(_graphs[j][i].getScrollableJGraph());
				//add(_graphs[j][i].getScrollableJGraph());
				add(sp);
				_graphs[j][i].getScrollableJGraph().addMouseListener(this);
				//add(_graphs[j][j].getJScrollPane());
			}
		}
		
		//JScrollPane sp = new JScrollPane(panel);
		//add(sp);
		//addMouseListener(this);
		
		//_frame = new FCModelerFrame(_graphs[0][0]);
		_frame = new FCModelerFrame("FCModelerConfig.txt");
		_frame.show();
		_frame.setTitle("node0");
		WeightedGraphRenderer renderer = new WeightedGraphRenderer();
		_frame.getFCModeler().setNodeRenderer(renderer);
		_frame.getFCModeler().setEdgeRenderer(renderer);
	}
	
	public void mouseClicked(MouseEvent event) {
		ScrollableJGraph graph = (ScrollableJGraph) event.getComponent();
		_frame.getFCModeler().setGraphModel(graph.getGraphModel());
		
		SimpleDotLayout layout = new SimpleDotLayout(_frame.getFCModeler());
		layout.layout(_frame.getFCModeler().getGraphView(), _frame.getFCModeler().getGraph());
		Rectangle2D box = layout.getBoundingBox();
		_frame.getFCModeler().getScrollableJGraph().setPreferredSize(new Dimension((int) box.getWidth(), (int) box.getHeight()));
		_frame.getFCModeler().getScrollableJGraph().setSize(new Dimension((int) box.getWidth(), (int) box.getHeight()));
		_frame.getFCModeler().getJScrollPane().revalidate();
		
		for (int i=0; i<_graphs.length; i++) {
			for (int j=0; j<_graphs[0].length; j++) {
				if (_graphs[i][j].getScrollableJGraph() == graph) {
					_frame.setTitle("node" + (i*_graphs.length+j));
					break;
				}
			}
		}
	}
	
	public void mouseEntered(MouseEvent event) {
		
	}
	
	public void mouseExited(MouseEvent event) {
		
	}
	
	public void mousePressed(MouseEvent event) {
		
	}
	
	public void mouseReleased(MouseEvent event) {
		
	}
	
	/*protected void paintComponent(Graphics graphics) {
		super.paintComponent(graphics);
		Graphics2D g = (Graphics2D) graphics;
	}*/
	
}
