/*
 * OnlineWGUpdater.java
 *
 * Created on May 2, 2002, 12:41 PM
 */

package fcmodeler.jsomap;

import jsomap.map.Map;
import jsomap.map.Node;
import jsomap.map.Model;
import jsomap.data.Pattern;
import jsomap.algorithm.parameters.Kernel;
import jsomap.algorithm.parameters.OnlineUpdater;

/**
 *
 * @author  default
 * @version
 */
public class OnlineWGUpdater extends Object implements OnlineUpdater {
	
	/** Creates new OnlineWGUpdater */
	public OnlineWGUpdater() {
		
	}
	
	/** Updates the specified model using the specified pattern and the other parameters.  In general, the
	 * model becomes "more like" the pattern, dependant on the result of the kernel function.
	 *
	 * @param model the model to update.
	 * @param winner the winning node.
	 * @param map the map.
	 * @param kernel the kernel function.
	 * @param pattern the pattern used to update the model.
	 * @param iteration the current iteration number.
	 * @throws NullPointerException if any of the parameters are <CODE>null</CODE>.
	 * @throws IllegalArgumentException if <CODE>iteration < 0</CODE>.
	 */
	public void update(Model model, Node winner, Map map, Kernel kernel, Pattern pattern, int iteration) {
		if (model == null) { throw new NullPointerException("model was null"); }
		if (map == null) { throw new NullPointerException("map was null"); }
		if (kernel == null) { throw new NullPointerException("kernel was null"); }
		if (pattern == null) { throw new NullPointerException("pattern was null"); }
		if (iteration < 0) { throw new IllegalArgumentException("iteration must be >= 0: " + iteration); }
		
		//double[] m = ((DoublePattern) model.getPattern()).get();
		WeightedGraph m = (WeightedGraph) model.getPattern();
		
		//double[] mNew = new double[m.length];
		//WeightedGraph mNew = new WeightedGraph();
		
		//double[] x = ((DoublePattern) pattern).get();
		WeightedGraph x = (WeightedGraph) pattern;
		
		double k = kernel.calculate(winner, model.getNode(), iteration);
		
		//for (int d=0, length=m.length; d<length; d++) {
		//	mNew[d] = m[d] + k*(x[d] - m[d]);
		//}
		//model.setPattern(new DoublePattern(mNew));
		
		//mNew = m + k*(x - m)
		WeightedGraph mNew = WGMath.addTo(WGMath.multiplyBy(WGMath.subtract(x, m), k), m);
		model.setPattern(mNew);
	}
	
}
