/*
 * WGMath.java
 *
 * Created on April 11, 2002, 5:02 PM
 */

package fcmodeler.jsomap;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 *
 * @author  default
 * @version
 */
public final class WGMath {
	
	/** Creates new WGMath */
	private WGMath() {
		
	}
	
	public static WeightedGraph add(WeightedGraph g1, double a) {
		WeightedSet nodes = WSMath.add(g1.getNodes(), a);
		WeightedSet edges = WSMath.add(g1.getEdges(), a);
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph add(WeightedGraph g1, WeightedGraph g2) {
		WeightedSet nodes = WSMath.add(g1.getNodes(), g2.getNodes());
		WeightedSet edges = WSMath.add(g1.getEdges(), g2.getEdges());
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph addTo(WeightedGraph g1, double a) {
		WSMath.addTo(g1.getNodes(), a);
		WSMath.addTo(g1.getEdges(), a);
		return g1;
	}
	
	public static WeightedGraph addTo(WeightedGraph g1, WeightedGraph g2) {
		WSMath.addTo(g1.getNodes(), g2.getNodes());
		WSMath.addTo(g1.getEdges(), g2.getEdges());
		return g1;
	}
	
	public static double distance(WeightedGraph g1, WeightedGraph g2) {
		double nodeDist = WSMath.distance(g1.getNodes(), g2.getNodes());
		double edgeDist = WSMath.distance(g1.getEdges(), g2.getEdges());
		return nodeDist + edgeDist;
	}
	
	public static WeightedGraph divide(WeightedGraph g1, double a) {
		WeightedSet nodes = WSMath.divide(g1.getNodes(), a);
		WeightedSet edges = WSMath.divide(g1.getEdges(), a);
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph divide(WeightedGraph g1, WeightedGraph g2) {
		WeightedSet nodes = WSMath.divide(g1.getNodes(), g2.getNodes());
		WeightedSet edges = WSMath.divide(g1.getEdges(), g2.getEdges());
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph divideBy(WeightedGraph g1, double a) {
		WSMath.divideBy(g1.getNodes(), a);
		WSMath.divideBy(g1.getEdges(), a);
		return g1;
	}
	
	public static WeightedGraph divideBy(WeightedGraph g1, WeightedGraph g2) {
		WSMath.divideBy(g1.getNodes(), g2.getNodes());
		WSMath.divideBy(g1.getEdges(), g2.getEdges());
		return g1;
	}
	
	public static WeightedGraph median(Set graphs) {
		if (graphs == null) { throw new NullPointerException("graphs was null"); }
		
		Set nodesets = new HashSet();
		Set edgesets = new HashSet();
		for (Iterator iterator = graphs.iterator(); iterator.hasNext();) {
			WeightedGraph graph = (WeightedGraph) iterator.next();
			nodesets.add(graph.getNodes());
			edgesets.add(graph.getEdges());
		}
		
		WeightedSet nodes = WSMath.median(nodesets);
		WeightedSet edges = WSMath.median(edgesets);
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph multiply(WeightedGraph g1, double a) {
		WeightedSet nodes = WSMath.multiply(g1.getNodes(), a);
		WeightedSet edges = WSMath.multiply(g1.getEdges(), a);
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph multiply(WeightedGraph g1, WeightedGraph g2) {
		WeightedSet nodes = WSMath.multiply(g1.getNodes(), g2.getNodes());
		WeightedSet edges = WSMath.multiply(g1.getEdges(), g2.getEdges());
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph multiplyBy(WeightedGraph g1, double a) {
		WSMath.multiplyBy(g1.getNodes(), a);
		WSMath.multiplyBy(g1.getEdges(), a);
		return g1;
	}
	
	public static WeightedGraph multiplyBy(WeightedGraph g1, WeightedGraph g2) {
		WSMath.multiplyBy(g1.getNodes(), g2.getNodes());
		WSMath.multiplyBy(g1.getEdges(), g2.getEdges());
		return g1;
	}
	
	public static WeightedGraph subtract(WeightedGraph g1, double a) {
		WeightedSet nodes = WSMath.subtract(g1.getNodes(), a);
		WeightedSet edges = WSMath.subtract(g1.getEdges(), a);
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph subtract(WeightedGraph g1, WeightedGraph g2) {
		WeightedSet nodes = WSMath.subtract(g1.getNodes(), g2.getNodes());
		WeightedSet edges = WSMath.subtract(g1.getEdges(), g2.getEdges());
		return new WeightedGraph(nodes, edges);
	}
	
	public static WeightedGraph subtractFrom(WeightedGraph g1, double a) {
		WSMath.subtractFrom(g1.getNodes(), a);
		WSMath.subtractFrom(g1.getEdges(), a);
		return g1;
	}
	
	public static WeightedGraph subtractFrom(WeightedGraph g1, WeightedGraph g2) {
		WSMath.subtractFrom(g1.getNodes(), g2.getNodes());
		WSMath.subtractFrom(g1.getEdges(), g2.getEdges());
		return g1;
	}
	
}
