/*
 * WSMath.java
 *
 * Created on April 11, 2002, 4:28 PM
 */

package fcmodeler.jsomap;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 *
 * @author  default
 * @version
 */
public final class WSMath {
	
	/** Creates new WSMath */
	private WSMath() {
		
	}
	
	public static WeightedSet add(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		WeightedSet ws = new WeightedSet();
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) + a);
		}
		return ws;
	}
	
	public static WeightedSet add(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		WeightedSet ws = new WeightedSet();
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) + s2.get(obj));
		}
		return ws;
	}
	
	public static WeightedSet addTo(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) + a);
		}
		return s1;
	}
	
	public static WeightedSet addTo(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) + s2.get(obj));
		}
		return s1;
	}
	
	public static double distance(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		double distance = 0.0d;
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			double a = s1.get(obj);
			double b = s2.get(obj);
			distance += (a-b) * (a-b);
		}
		return Math.sqrt(distance);
	}
	
	public static WeightedSet divide(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		WeightedSet ws = new WeightedSet();
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) / a);
		}
		return ws;
	}
	
	public static WeightedSet divide(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		WeightedSet ws = new WeightedSet();
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) / s2.get(obj));
		}
		return ws;
	}
	
	public static WeightedSet divideBy(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) / a);
		}
		return s1;
	}

	public static WeightedSet divideBy(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) / s2.get(obj));
		}
		return s1;
	}
	
	public static WeightedSet median(Set sets) {
		if (sets == null) { throw new NullPointerException("sets was null"); }
		
		WeightedSet median = new WeightedSet();
		for (Iterator iterator = sets.iterator(); iterator.hasNext();) {
			WSMath.addTo(median, (WeightedSet) iterator.next());
		}
		WSMath.divideBy(median, sets.size());
		return median;
	}
	
	public static WeightedSet multiply(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		WeightedSet ws = new WeightedSet();
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) * a);
		}
		return ws;
	}
	
	public static WeightedSet multiply(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		WeightedSet ws = new WeightedSet();
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) * s2.get(obj));
		}
		return ws;
	}
	
	public static WeightedSet multiplyBy(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) * a);
		}
		return s1;
	}
	
	public static WeightedSet multiplyBy(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) * s2.get(obj));
		}
		return s1;
	}
	
	public static WeightedSet subtract(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		WeightedSet ws = new WeightedSet();
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) - a);
		}
		return ws;
	}
	
	public static WeightedSet subtract(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		WeightedSet ws = new WeightedSet();
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			ws.set(obj, s1.get(obj) - s2.get(obj));
		}
		return ws;
	}
	
	public static WeightedSet subtractFrom(WeightedSet s1, double a) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		
		for (Iterator iterator = s1.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) - a);
		}
		return s1;
	}
	
	public static WeightedSet subtractFrom(WeightedSet s1, WeightedSet s2) {
		if (s1 == null) { throw new NullPointerException("s1 was null"); }
		if (s2 == null) { throw new NullPointerException("s2 was null"); }
		
		Set alphabet = new HashSet(s1);
		alphabet.addAll(s2);
		for (Iterator iterator = alphabet.iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			s1.set(obj, s1.get(obj) - s2.get(obj));
		}
		return s1;
	}
	
}
