/*
 * WeightedGraph.java
 *
 * Created on April 11, 2002, 4:47 PM
 */

package fcmodeler.jsomap;

import jsomap.data.Pattern;

/** A graph whose nodes and edges are weighted.  This graph is composed of two sets: a node set and an
 * edge set.  The node set is a set of objects, and the edge set is a set of relations between pairs of
 * elements of the node set.
 *
 * @author default
 * @version
 */
public class WeightedGraph implements Pattern {
	private WeightedSet _edges = null;
	private WeightedSet _nodes = null;
	
	/** Creates a new empty <CODE>WeightedGraph</CODE>.
	 *
	 */
	public WeightedGraph() {
		_nodes = new WeightedSet();
		_edges = new WeightedSet();
	}
	
	/** Creates a new <CODE>WeightedGraph</CODE> that is a copy of the specified graph.
	 *
	 * @param graph the graph to copy.
	 */	
	public WeightedGraph(WeightedGraph graph) {
		if (graph == null) { throw new NullPointerException("graph was null"); }
		
		_nodes = new WeightedSet(graph.getNodes());
		_edges = new WeightedSet(graph.getEdges());
	}
	
	/** Creates a new <CODE>WeightedGraph</CODE> from copies of the specified node and edge sets.
	 *
	 * @param nodes the node set.
	 * @param edges the edge set.
	 */	
	public WeightedGraph(WeightedSet nodes, WeightedSet edges) {
		if (nodes == null) { throw new NullPointerException("nodes was null"); }
		if (edges == null) { throw new NullPointerException("edges was null"); }
		
		_nodes = new WeightedSet(nodes);
		_edges = new WeightedSet(edges);
	}
	
	/** Returns the edge set.
	 *
	 * @return the edge set.
	 */	
	public WeightedSet getEdges() {
		return _edges;
	}
	
	/** Returns the node set.
	 *
	 * @return the node set.
	 */	
	public WeightedSet getNodes() {
		return _nodes;
	}
	
	/** Returns the string representation of this graph.
	 *
	 * @return the string representation of this graph.
	 */	
	public String toString() {
		return "(" + _nodes + ", " + _edges + ")";
	}
	
}
