/*
 * WeightedGraphRenderer.java
 *
 * Created on May 13, 2002, 10:19 AM
 */

package fcmodeler.jsomap;

import java.awt.Color;

import diva.graph.EdgeRenderer;
import diva.graph.NodeRenderer;
import diva.canvas.Site;
import diva.canvas.Figure;
import diva.canvas.toolbox.BasicFigure;
import diva.canvas.connector.Connector;
import diva.graph.model.Edge;
import diva.graph.model.Node;

import fcmodeler.view.NodeFigure;
import fcmodeler.view.BezierEdgeFigure;
import fcmodeler.view.NodeFigureRenderer;
import fcmodeler.view.BezierEdgeFigureRenderer;

/**
 *
 * @author  default
 * @version
 */
public class WeightedGraphRenderer extends Object implements NodeRenderer, EdgeRenderer {
	private NodeFigureRenderer _nodeRenderer = null;
	private BezierEdgeFigureRenderer _edgeRenderer = null;
	
	/** Creates new WeightedGraphRenderer */
	public WeightedGraphRenderer() {
		_nodeRenderer = new NodeFigureRenderer();
		_edgeRenderer = new BezierEdgeFigureRenderer();
	}
	
	public Connector render(Edge edge, Site tailSite, Site headSite) {
		BezierEdgeFigure edgeFigure = (BezierEdgeFigure) _edgeRenderer.render(edge, tailSite, headSite);
		double weight = Double.parseDouble(edge.getProperty("weight").toString());
		float width = (float) (5.0d * weight);
		edgeFigure.getFigure().setLineWidth(width);
		edgeFigure.getPermanentFigure().setLineWidth(width);
		return edgeFigure;
	}
	
	public Figure render(Node node) {
		NodeFigure nodeFigure = (NodeFigure)_nodeRenderer.render(node);
		double weight = Double.parseDouble(node.getProperty("weight").toString());
		float inverse = 1.0f - (float) weight;
		BasicFigure figure = nodeFigure.getFigure();
		figure.setFillPaint(new Color(1.0f, inverse, inverse));
		figure = nodeFigure.getPermanentFigure();
		figure.setFillPaint(new Color(1.0f, inverse, inverse));
		return nodeFigure;
	}
	
}
