/* $Id: AbstractLayout.java,v 1.1 2002/07/26 23:09:05 adamtj Exp $
 *
 *
 */
package fcmodeler.layout;

import java.util.List;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.Dimension;
import java.lang.Double;

import diva.canvas.selection.SelectionModel;
import diva.graph.GraphView;
import diva.graph.layout.GlobalLayout;
import diva.graph.layout.LayoutTarget;
import diva.graph.layout.LayoutUtilities;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.Graph;

import fcmodeler.FCModeler;
//import fcmodeler.interaction.GeneralConnector.GeneralConnector;
//import fcmodeler.view.EdgeFigure;
//import fcmodeler.view.BezierEdgeFigure;

/**
 * ... is ...
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */
public abstract class AbstractLayout extends Object implements GlobalLayout {
	protected FCModeler _fcmodeler = null;
	
	protected List _nodes = null;
	protected Map _edgesToEdgeFigures = null;
	
	
	private LayoutTarget _target = null;
	private Graph _graph = null;
	
	/** Creates a new instance of AbstractLayout */
	public AbstractLayout(FCModeler fcmodeler) {
		setFCModeler(fcmodeler);
		_nodes = new Vector();
		_edgesToEdgeFigures = new HashMap();
	}
	public FCModeler getFCModeler() {
		return _fcmodeler;
	}
	public Graph getGraph() {
		return _graph;
	}
	public LayoutTarget getLayoutTarget() {
		return _target;
	}
	public void setFCModeler(FCModeler fcmodeler) {
		_fcmodeler = fcmodeler;
	}
	public void layout(LayoutTarget target, Graph graph) {
		GraphView view = _fcmodeler.getGraphView();
		
		//Build a list of edges.
		//I think it's silly that diva can't give us one.
		Iterator i = graph.nodes();
		while (i.hasNext()) {
			Node node = (Node) i.next();
			_nodes.add(node);
			Iterator edges = node.inEdges();
			while (edges.hasNext()) {
				Edge edge = (Edge) edges.next();
				_edgesToEdgeFigures.put(edge, view.getEdgeFigure(edge));
			}
			//If we do the outEdges(), we'll count every edge twice: unnecessary.
		}

		//This is necessary for placeNode(...) to work.
		_target = target;
		
		//This is where the subclass takes over and does the actual layout work.
		computeLayout();
		
		//Route the edges.
		LayoutUtilities.routeVisibleEdges(view.getGraphModel().getGraph(), view);
		
		//Refresh the selections
		//If the selections are refreshed, then grab handles could be in the wrong
		//spot after re-laying out the graph.
		//I think the correct way to refresh the selections is from 
		//GeneralConnectorManipulator.refresh(), however there are problems with
		//making the grab handles go away.
		SelectionModel sm = ((GraphView)target).getGraphController().getEdgeInteractor().getSelectionModel();
		/*Iterator*/ i = sm.getSelection();
		//We have to build a list of selected objects because if we start removing
		//objects from the selection, we'll change the contents of the iterator
		//which will throw an exception.
		List selectedObjs = new Vector();
		while (i.hasNext()) {
			selectedObjs.add(i.next());
		}
		i = selectedObjs.iterator();
		while (i.hasNext()) {
			Object o = i.next();
			sm.removeSelection(o);
			sm.addSelection(o);	
		}
	}
	
	public void placeNode(Node node, double x, double y) {
		LayoutUtilities.placeNoReroute(_target, node, x, y);
	}
	/** Call placeNode(...) to position each node.  Do whatever is necessary to
	 *  whatever type of  edgeFigure you are using to position them.
	 */
	public abstract void computeLayout ();
	
	public void layout() {
		layout(_fcmodeler.getGraphView(), _fcmodeler.getGraph());
		Rectangle2D r = getBoundingBox();
		Dimension size = new Dimension((int)r.getMaxX(), (int)r.getMaxY());
		_fcmodeler.getScrollableJGraph().setPreferredSize(size);
		_fcmodeler.getScrollableJGraph().setSize(size);
		_fcmodeler.getJScrollPane().revalidate();
		_fcmodeler.getJScrollPane().repaint();
	}
	
	public Rectangle2D getBoundingBox() {
		
		//Get the node coordinates
		Iterator nodes = _fcmodeler.getGraph().nodes();
		GraphView gv = _fcmodeler.getGraphView();
		Set xSizes = new HashSet();
		Set ySizes = new HashSet();
		while (nodes.hasNext()) {
			Node node = (Node) nodes.next();
			Rectangle2D bounds = gv.getBounds(node);
			xSizes.add(new Double((double)bounds.getMaxX()));
			ySizes.add(new Double((double)bounds.getMaxY()));
		}
		
		//Get the edge coordinates.
		//If we don't do this, there may be some edges that swoop out of view.
		Iterator edges = _edgesToEdgeFigures.keySet().iterator();
		while (edges.hasNext()) {
			Edge edge = (Edge) edges.next();
			Rectangle2D bounds = gv.getEdgeFigure(edge).getBounds();
			xSizes.add(new Double((double)bounds.getMaxX()));
			ySizes.add(new Double((double)bounds.getMaxY()));
		}
		
		//Find the maxima.
		Double maxX = new Double(0.0d);
		Double maxY = new Double(0.0d);
		Iterator sizes = xSizes.iterator();
		while(sizes.hasNext()) {
			Double size = (Double) sizes.next();
			if (size.compareTo(maxX)>0) {
				maxX = size;
			}
		}
		sizes = ySizes.iterator();
		while(sizes.hasNext()) {
			Double size = (Double) sizes.next();
			if (size.compareTo(maxY)>0) {
				maxY = size;
			}
		}
		
		double d1 = maxX.doubleValue();
		double d2 = maxY.doubleValue();
		return new Rectangle2D.Double(0,0,d1,d2);
	}
}

/* 
 * $Log: AbstractLayout.java,v $
 * Revision 1.1  2002/07/26 23:09:05  adamtj
 * Added the files necessary for the GEM layout algorithm.
 *
 *
 */
