/* $Id: InitialLayout.java,v 1.5 2002/01/15 20:33:52 zcox Exp $
 *
 *
 * Created on April 5, 2001, 3:51 PM
 */

package fcmodeler.layout;

import java.awt.geom.Rectangle2D;
import java.util.Iterator;

import diva.graph.layout.RandomIncrLayout;
import diva.graph.layout.LayoutTarget;
import diva.graph.layout.LayoutUtilities;
import diva.graph.model.Node;
import diva.graph.model.Graph;
import diva.graph.model.GraphEvent;

/** This class was needed because Diva's RandomIncrLayout is stupid when an entire graph
 * needs to be layed out randomly, as in the initial display of a graph after loading from
 * a file.  It took RandomIncrLayout approximately 24 seconds to perform the layout of a 
 * 200 node, 200 edge graph running on a 1GHz Athlon w/ 256MB of RAM.  This class performs
 * the same layout in approximately 0.8 seconds.
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */

public class InitialLayout extends RandomIncrLayout {
	private LayoutTarget _target = null;
	
	/** Creates new InitialLayout */
	public InitialLayout(LayoutTarget target) {
		super(target);
		_target = target;
	}
	
	public void structureChanged(GraphEvent graphEvent) {
		Graph graph = (Graph) graphEvent.getTarget();
		Node node = null;
		Iterator nodes = graph.nodes();
		Rectangle2D viewport = _target.getViewport(graph);
		double width = viewport.getWidth(), height = viewport.getHeight();
		double x = 0.0d, y = 0.0d;
		
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			x = Math.random() * width + 50;
			y = Math.random() * height + 50;
			LayoutUtilities.placeNoReroute(_target, node, x, y);
		}
	}
}

/*
 * $Log: InitialLayout.java,v $
 * Revision 1.5  2002/01/15 20:33:52  zcox
 * some changes
 *
 * Revision 1.4  2001/05/22 16:42:45  zcox
 * code clean-up
 *
 * Revision 1.3  2001/04/05 21:23:53  zcox
 * wincvs screwed up again
 *
 * Revision 1.2  2001/04/05 21:22:43  zcox
 * wincvs screwed up
 *
 * Revision 1.1  2001/04/05 21:20:21  zcox
 * initial commit of InitialLayout
 *
 *
 *
 *
 */
