/* $Id: RankClusterDotLayout.java,v 1.1 2002/02/15 20:12:05 zcox Exp $
 *
 *
 */

package fcmodeler.layout;

import java.util.Iterator;
import java.io.Reader;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.awt.geom.Rectangle2D;

import diva.canvas.Figure;
import diva.canvas.FigureLayer;
import diva.canvas.FigureDecorator;
import diva.graph.GraphView;
import diva.graph.layout.GlobalLayout;
import diva.graph.layout.LayoutTarget;
import diva.graph.model.Graph;

import fcmodeler.FCModeler;
import fcmodeler.io.dot.DotGraphWriter;
import fcmodeler.io.dot.RankClusterDotWriter;
import fcmodeler.io.dot.DotGraphReader;
import fcmodeler.io.dot.RankClusterDotHandler;

/** 
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class RankClusterDotLayout extends Object implements GlobalLayout {
	private DotGraphWriter _writer = null;
	private DotGraphReader _reader = null;
	private FCModeler _fcmodeler = null;
	
	/** Creates new RankClusterDotLayout */
	public RankClusterDotLayout(FCModeler fcmodeler, Reader layoutXML) {
		_fcmodeler = fcmodeler;
		_writer = new RankClusterDotWriter(layoutXML);
		_writer.writeTopology(true);
	}
	
	public Rectangle2D getBoundingBox() {
		return _reader.getHandler().getBoundingBox();
	}
	
	public void layout(LayoutTarget layoutTarget, Graph graph) {
		if (!(layoutTarget instanceof GraphView)) { throw new IllegalArgumentException("layoutTarget must be of type GraphView"); }
		
		GraphView view = (GraphView) layoutTarget;
		_reader = new DotGraphReader(new RankClusterDotHandler(view));
		
		//remove any non-graph element figures first
		//System.out.println("removing non-graph element figures");
		//get the FigureLayer & iterate over all of its figures
		FigureLayer layer = view.getGraphicsPane().getForegroundLayer();
		for (Iterator figures = layer.figures(); figures.hasNext();) {
			Figure figure = (Figure) figures.next();
			
			//if the figure has no user object, it's not a graph element figure - so remove it
			//NOTE - the first condition of the following if-statement is a temporary fix for a bigger problem,
			//namely deciding what Figures should be removed from the graph view.
			//right now, we're just trying to remove rank & cluster figures, but there could be more in the future
			//should probably identify those figures with some interface, and test for that for removal
			if (!(figure instanceof FigureDecorator) && figure.getUserObject() == null) {
				figure.setParent(null);
				figures.remove();
			}
		}
		
		//create the dot process
		//System.out.println("creating Dot object with _pathToDot = " + _pathToDot);
		Dot dot = new Dot(_fcmodeler.getPathToDot(), null);
		//System.out.println("Dot object created, dot = " + dot);
		
		//System.out.println("writing the graph to dot");
		//write the graph to dot
		_writer.writeGraph(_fcmodeler, new BufferedWriter(new OutputStreamWriter(dot.getProcess().getOutputStream())));
		
		//System.out.println("reading graph back from dot");
		//read the graph back from dot
		//parse it and send events to dot handler
		_reader.getHandler().setEdgeMap(_writer.getEdgeMap());
		_reader.getHandler().setNodeMap(_writer.getNodeMap());
		_reader.readGraph(new InputStreamReader(dot.getProcess().getInputStream()));
		_reader.configure(_fcmodeler);
	}
	
}

/* 
 * $Log: RankClusterDotLayout.java,v $
 * Revision 1.1  2002/02/15 20:12:05  zcox
 * big time update
 *
 *
 *
 */
