/* $Id: SpecifiedLayout.java,v 1.5 2002/07/13 07:48:12 adamtj Exp $
 *
 *
 */

package fcmodeler.layout;

import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.Dimension;
import java.lang.Double;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import diva.canvas.selection.SelectionModel;
import diva.graph.GraphView;
import diva.graph.layout.GlobalLayout;
import diva.graph.layout.LayoutTarget;
import diva.graph.layout.LayoutUtilities;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.Graph;


import fcmodeler.FCModeler;
import fcmodeler.interaction.GeneralConnector.GeneralConnector;
import fcmodeler.view.EdgeFigure;
import fcmodeler.view.BezierEdgeFigure;

/**
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */
public class SpecifiedLayout extends Object implements GlobalLayout {
	private FCModeler _fcmodeler = null;
	private Map _nodeCoordinates = null;	
	private Map _edgeCoordinates = null;
	private Set _edges = null;
	
	/** Creates a new instance of SpecifiedLayout */
	public SpecifiedLayout(FCModeler fcmodeler, Map nodeCoordinates, Map edgeCoordinates) {
		_fcmodeler = fcmodeler;
		_nodeCoordinates = nodeCoordinates;
		_edgeCoordinates = edgeCoordinates;
	}
	
	public Rectangle2D getBoundingBox() {
		
		//Get the node coordinates
		Iterator nodes = _fcmodeler.getGraph().nodes();
		GraphView gv = _fcmodeler.getGraphView();
		Set xSizes = new HashSet();
		Set ySizes = new HashSet();
		while (nodes.hasNext()) {
			Node node = (Node) nodes.next();
			Rectangle2D bounds = gv.getBounds(node);
			xSizes.add(new Double((double)bounds.getMaxX()));
			ySizes.add(new Double((double)bounds.getMaxY()));
		}
		
		//Get the edge coordinates.
		//If we don't do this, there may be some edges that swoop out of view.
		Iterator edges = _edges.iterator();
		while (edges.hasNext()) {
			Edge edge = (Edge) edges.next();
			Rectangle2D bounds = gv.getEdgeFigure(edge).getBounds();
			xSizes.add(new Double((double)bounds.getMaxX()));
			ySizes.add(new Double((double)bounds.getMaxY()));
		}
		
		//Find the maxima.
		Double maxX = new Double(0.0d);
		Double maxY = new Double(0.0d);
		Iterator sizes = xSizes.iterator();
		while(sizes.hasNext()) {
			Double size = (Double) sizes.next();
			if (size.compareTo(maxX)>0) {
				maxX = size;
			}
		}
		sizes = ySizes.iterator();
		while(sizes.hasNext()) {
			Double size = (Double) sizes.next();
			if (size.compareTo(maxY)>0) {
				maxY = size;
			}
		}
		
		double d1 = maxX.doubleValue();
		double d2 = maxY.doubleValue();
		return new Rectangle2D.Double(0,0,d1,d2);
	}
	
	public void layout(LayoutTarget target, Graph graph) {
		//Place the nodes
		Node node = null;
        
		Iterator nodes = graph.nodes();    
		while (nodes.hasNext()) {
			node = (Node) nodes.next();        
            
			if (_nodeCoordinates != null) {
				//This will only touch the nodes that had coordinates specified in 
				//the input file.  If the node wasn't listed in the file, we
				//shouldn't mess with it.
				Point2D coord = (Point2D)_nodeCoordinates.get(node.getSemanticObject());
				if (coord!=null) {
					LayoutUtilities.placeNoReroute(target, node, coord.getX(), coord.getY());
				}
			}
		}
        
		//Place the edges
		Edge edge = null;
		GraphView view = _fcmodeler.getGraphView();
        
		Map edgeNamesToEdges = new HashMap();

		//Build a list of edges.
		//I think it's silly that diva can't give us one.
		nodes = graph.nodes();
		_edges = new HashSet();
		while (nodes.hasNext()) {
			node = (Node) nodes.next();
			Iterator edges = node.inEdges();
			while (edges.hasNext()) {
				edge = (Edge) edges.next();
				edgeNamesToEdges.put(edge.getSemanticObject(), edge);
				_edges.add(edge);
			}
			//If we do the outEdges(), we'll count every edge twice: unnecessary.
		}
    
		//The fact that values() returns a Collection as opposed to a Set shouldn't
		//matter since the mapping should be one to one.
		Iterator edgeNames = edgeNamesToEdges.keySet().iterator();
		while (edgeNames.hasNext()) {
			String edgeName = (String) edgeNames.next();
			edge = (Edge) edgeNamesToEdges.get(edgeName);
			EdgeFigure edgeFigure = (EdgeFigure) view.getEdgeFigure(edge);
			List coords = (List)_edgeCoordinates.get(edgeName);
            
			if (edgeFigure instanceof BezierEdgeFigure) {
				if (coords != null) {
					((BezierEdgeFigure) edgeFigure).setControlPoints(coords);
				} else {
					System.out.println("SpecifiedLayout.layout(): Coordinates not specified.");
				}
			} else if (edgeFigure instanceof GeneralConnector){
				if (coords != null) {
					((GeneralConnector) edgeFigure).fromFile(coords);
				} else {
					System.out.println("SpecifiedLayout.layout(): Coordinates not specified.");
				}			
			} else {
				throw new UnsupportedOperationException("SpecifiedLayout.layout(): EdgeFigure not supported!");
			}   
		}
		LayoutUtilities.routeVisibleEdges(view.getGraphModel().getGraph(), view);
		
		//Refresh the selections
		//If the selections are refreshed, then grab handles could be in the wrong
		//spot after re-laying out the graph.
		//I think the correct way to refresh the selections is from 
		//GeneralConnectorManipulator.refresh(), however there are problems with
		//making the grab handles go away.
		SelectionModel sm = ((GraphView)target).getGraphController().getEdgeInteractor().getSelectionModel();
		Iterator i = sm.getSelection();
		//We have to build a list of selected objects because if we start removing
		//objects from the selection, we'll change the contents of the iterator
		//which will throw an exception.
		List selectedObjs = new Vector();
		while (i.hasNext()) {
			selectedObjs.add(i.next());
		}
		i = selectedObjs.iterator();
		while (i.hasNext()) {
			Object o = i.next();
			sm.removeSelection(o);
			sm.addSelection(o);	
		}
	}
	
	public void layout() {
		layout(_fcmodeler.getGraphView(), _fcmodeler.getGraph());
		Rectangle2D r = getBoundingBox();
		Dimension size = new Dimension((int)r.getMaxX(), (int)r.getMaxY());
		_fcmodeler.getScrollableJGraph().setPreferredSize(size);
		_fcmodeler.getScrollableJGraph().setSize(size);
		_fcmodeler.getJScrollPane().revalidate();
		_fcmodeler.getJScrollPane().repaint();
	}	
}

/* 
 * $Log: SpecifiedLayout.java,v $
 * Revision 1.5  2002/07/13 07:48:12  adamtj
 * Fixed a problem with edge mappings.  GeneralConnectorEdgeFigure improperly implemented EdgeFigure.  It has been removed and now GeneralConnector extends AbstractEdgeFigure.
 *
 * Revision 1.4  2002/06/27 08:11:08  adamtj
 * Added the cvs headers that I forgot to include with the initial commit.
 *
 *
 */
