/* $Id: AndConnective.java,v 1.1 2002/02/15 20:23:56 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

/** A <CODE>Connective</CODE> representing the AND logical operator.  This class is a Singleton;
 * the <CODE>getInstance</CODE> method provides access to the only <CODE>AndConnective</CODE> instance.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class AndConnective extends Object implements Connective {
	private static final AndConnective _instance = new AndConnective();
	private static final String _string = "and";
	
	/** Creates a new AndConnective.  Since this constructor is private, it cannot 
	 * be called outside of this class.
	 *
	 */
	private AndConnective() {
		
	}
	
	/** Evaluates the sentence <CODE>first</CODE> AND <CODE>second</CODE>.
	 *
	 * @param first the first term in the sentence.
	 * @param second the second term in the sentence.
	 * @return <CODE>true</CODE> if <CODE>first && second</CODE> evaluates to true; <CODE>false</CODE> otherwise.
	 */	
	public boolean evaluate(boolean first, boolean second) {
		return first && second;
	}
	
	/** Returns the only instance of <CODE>AndConnective</CODE>.
	 *
	 * @return the only instance of <CODE>AndConnective</CODE>.
	 */	
	public static AndConnective getInstance() {
		return _instance;
	}
	
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: AndConnective.java,v $
 * Revision 1.1  2002/02/15 20:23:56  zcox
 * big time update
 *
 *
 *
 */
