/* $Id: MappingEditor.java,v 1.1 2002/02/15 20:23:57 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

import javax.swing.JPanel;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.Set;
import java.util.Iterator;

import diva.graph.GraphView;
import diva.graph.model.Graph;
import diva.graph.model.GraphUtilities.EdgeIterator;

import fcmodeler.attributes.VisualAttribute;
import fcmodeler.attributes.EditorDialog;
import fcmodeler.attributes.EdgeVisualAttributeValue;
import fcmodeler.attributes.NodeVisualAttributeValue;

/** 
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class MappingEditor extends JPanel {
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.ButtonGroup _nConnectivesGroup;
	private javax.swing.ButtonGroup _eConnectivesGroup;
	private javax.swing.JSplitPane _splitPane;
	private javax.swing.JPanel _nodesPanel;
	private javax.swing.JPanel _npMappings;
	private javax.swing.JScrollPane _nspMappings;
	private javax.swing.JList _nMappings;
	private javax.swing.JPanel _npPropertyValues;
	private javax.swing.JScrollPane _nspPropertyValues;
	private javax.swing.JTable _nPropertyValues;
	private javax.swing.JPanel _npConnectives;
	private javax.swing.JRadioButton _nAnd;
	private javax.swing.JRadioButton _nOr;
	private javax.swing.JPanel _npAttributes;
	private javax.swing.JScrollPane _nspAttributes;
	private javax.swing.JList _nAttributes;
	private javax.swing.JPanel _edgesPanel;
	private javax.swing.JPanel _epMappings;
	private javax.swing.JScrollPane _espMappings;
	private javax.swing.JList _eMappings;
	private javax.swing.JPanel _epPropertyValues;
	private javax.swing.JScrollPane _espPropertyValues;
	private javax.swing.JTable _ePropertyValues;
	private javax.swing.JPanel _epConnectives;
	private javax.swing.JRadioButton _eAnd;
	private javax.swing.JRadioButton _eOr;
	private javax.swing.JPanel _epAttributes;
	private javax.swing.JScrollPane _espAttributes;
	private javax.swing.JList _eAttributes;
	// End of variables declaration//GEN-END:variables
	private MappingListModel _nodeMappings = null;
	private MappingListModel _edgeMappings = null;
	private NodePropertiesTableModel _nodeProperties = null;
	private EdgePropertiesTableModel _edgeProperties = null;
	private AttributesListModel _nodeAttributes = null;
	private AttributesListModel _edgeAttributes = null;
	private BasicTableSelectionModel _nodeSelection = null;
	private BasicTableSelectionModel _edgeSelection = null;
	private Graph _graph = null;
	private GraphView _view = null;
	
	/** Creates a new <CODE>MappingEditor</CODE> using the specified parameters.
	 *
	 * @param graph the graph containing the nodes and edges.
	 * @param nodeAttributes the node visual attributes.
	 * @param edgeAttributes the edge visual attributes.
	 * @param nodeMappings the node mappings to edit.
	 * @param edgeMappings the edge mapping to edit.
	 * @param view the graph view containing the node and edge figures.
	 */
	public MappingEditor(Graph graph, Collection nodeAttributes, Collection edgeAttributes, Set nodeMappings, Set edgeMappings, GraphView view) {
		_nodeMappings = new MappingListModel(nodeMappings);
		_edgeMappings = new MappingListModel(edgeMappings);
		_nodeProperties = new NodePropertiesTableModel(graph);
		_edgeProperties = new EdgePropertiesTableModel(graph);
		_nodeAttributes = new AttributesListModel(nodeAttributes);
		_edgeAttributes = new AttributesListModel(edgeAttributes);
		_graph = graph;
		_view = view;
		
		initComponents();
		
		_nodeSelection = new BasicTableSelectionModel(_nPropertyValues);
		_edgeSelection = new BasicTableSelectionModel(_ePropertyValues);
		_nPropertyValues.addMouseListener(_nodeSelection);
		_ePropertyValues.addMouseListener(_edgeSelection);
		
		BasicTableCellRenderer nodeRenderer = new BasicTableCellRenderer(_nodeSelection);
		BasicTableCellRenderer edgeRenderer = new BasicTableCellRenderer(_edgeSelection);
		_nPropertyValues.setDefaultRenderer((new Object()).getClass(), nodeRenderer);
		_ePropertyValues.setDefaultRenderer((new Object()).getClass(), edgeRenderer);
		
		_nConnectivesGroup.add(_nAnd);
		_nConnectivesGroup.add(_nOr);
		_eConnectivesGroup.add(_eAnd);
		_eConnectivesGroup.add(_eOr);
		
		_splitPane.setDividerLocation(0.5d);
	}
	
	public Connective getEdgeConnective() {
		if (_eAnd.isSelected()) { return AndConnective.getInstance(); }
		else if (_eOr.isSelected()) { return OrConnective.getInstance(); }
		else { return null; }
	}
	
	public String getEdgeProperty(int col) {
		return _ePropertyValues.getColumnName(col);
	}
	
	public Object getEdgeValue(int row, int col) {
		return _ePropertyValues.getValueAt(row, col);
	}
	
	public Connective getNodeConnective() {
		if (_nAnd.isSelected()) { return AndConnective.getInstance(); }
		else if (_nOr.isSelected()) { return OrConnective.getInstance(); }
		else { return null; }
	}
	
	public String getNodeProperty(int col) {
		return _nPropertyValues.getColumnName(col);
	}
	
	public Object getNodeValue(int row, int col) {
		return _nPropertyValues.getValueAt(row, col);
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	private void initComponents() {//GEN-BEGIN:initComponents
		_nConnectivesGroup = new javax.swing.ButtonGroup();
		_eConnectivesGroup = new javax.swing.ButtonGroup();
		_splitPane = new javax.swing.JSplitPane();
		_nodesPanel = new javax.swing.JPanel();
		_npMappings = new javax.swing.JPanel();
		_nspMappings = new javax.swing.JScrollPane();
		_nMappings = new javax.swing.JList();
		_npPropertyValues = new javax.swing.JPanel();
		_nspPropertyValues = new javax.swing.JScrollPane();
		_nPropertyValues = new javax.swing.JTable();
		_npConnectives = new javax.swing.JPanel();
		_nAnd = new javax.swing.JRadioButton();
		_nOr = new javax.swing.JRadioButton();
		_npAttributes = new javax.swing.JPanel();
		_nspAttributes = new javax.swing.JScrollPane();
		_nAttributes = new javax.swing.JList();
		_edgesPanel = new javax.swing.JPanel();
		_epMappings = new javax.swing.JPanel();
		_espMappings = new javax.swing.JScrollPane();
		_eMappings = new javax.swing.JList();
		_epPropertyValues = new javax.swing.JPanel();
		_espPropertyValues = new javax.swing.JScrollPane();
		_ePropertyValues = new javax.swing.JTable();
		_epConnectives = new javax.swing.JPanel();
		_eAnd = new javax.swing.JRadioButton();
		_eOr = new javax.swing.JRadioButton();
		_epAttributes = new javax.swing.JPanel();
		_espAttributes = new javax.swing.JScrollPane();
		_eAttributes = new javax.swing.JList();
		
		
		setLayout(new java.awt.BorderLayout());
		
		_splitPane.setResizeWeight(0.5);
		_splitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		_nodesPanel.setLayout(new javax.swing.BoxLayout(_nodesPanel, javax.swing.BoxLayout.X_AXIS));
		
		_nodesPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "nodes"));
		_npMappings.setLayout(new java.awt.BorderLayout());
		
		_npMappings.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "mapping rules"));
		_nspMappings.setPreferredSize(new java.awt.Dimension(200, 100));
		_nMappings.setModel(_nodeMappings);
		_nMappings.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyReleased(java.awt.event.KeyEvent evt) {
				_nMappingsKeyReleased(evt);
			}
		});
		
		_nspMappings.setViewportView(_nMappings);
		
		_npMappings.add(_nspMappings, java.awt.BorderLayout.CENTER);
		
		_nodesPanel.add(_npMappings);
		
		_npPropertyValues.setLayout(new javax.swing.BoxLayout(_npPropertyValues, javax.swing.BoxLayout.X_AXIS));
		
		_npPropertyValues.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "property values"));
		_nPropertyValues.setModel(_nodeProperties);
		_nPropertyValues.setPreferredScrollableViewportSize(new java.awt.Dimension(300, 100));
		_nspPropertyValues.setViewportView(_nPropertyValues);
		
		_npPropertyValues.add(_nspPropertyValues);
		
		_npConnectives.setLayout(new javax.swing.BoxLayout(_npConnectives, javax.swing.BoxLayout.Y_AXIS));
		
		_nAnd.setSelected(true);
		_nAnd.setText("and");
		_npConnectives.add(_nAnd);
		
		_nOr.setText("or");
		_npConnectives.add(_nOr);
		
		_npPropertyValues.add(_npConnectives);
		
		_nodesPanel.add(_npPropertyValues);
		
		_npAttributes.setLayout(new java.awt.BorderLayout());
		
		_npAttributes.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "visual attributes"));
		_nspAttributes.setPreferredSize(new java.awt.Dimension(100, 100));
		_nAttributes.setModel(_nodeAttributes);
		_nAttributes.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		_nAttributes.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				_nAttributesMouseClicked(evt);
			}
		});
		
		_nspAttributes.setViewportView(_nAttributes);
		
		_npAttributes.add(_nspAttributes, java.awt.BorderLayout.CENTER);
		
		_nodesPanel.add(_npAttributes);
		
		_splitPane.setLeftComponent(_nodesPanel);
		
		_edgesPanel.setLayout(new javax.swing.BoxLayout(_edgesPanel, javax.swing.BoxLayout.X_AXIS));
		
		_edgesPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "edges"));
		_epMappings.setLayout(new java.awt.BorderLayout());
		
		_epMappings.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "mapping rules"));
		_espMappings.setPreferredSize(new java.awt.Dimension(200, 100));
		_eMappings.setModel(_edgeMappings);
		_eMappings.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyReleased(java.awt.event.KeyEvent evt) {
				_eMappingsKeyReleased(evt);
			}
		});
		
		_espMappings.setViewportView(_eMappings);
		
		_epMappings.add(_espMappings, java.awt.BorderLayout.CENTER);
		
		_edgesPanel.add(_epMappings);
		
		_epPropertyValues.setLayout(new javax.swing.BoxLayout(_epPropertyValues, javax.swing.BoxLayout.X_AXIS));
		
		_epPropertyValues.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "property values"));
		_ePropertyValues.setModel(_edgeProperties);
		_ePropertyValues.setPreferredScrollableViewportSize(new java.awt.Dimension(300, 100));
		_espPropertyValues.setViewportView(_ePropertyValues);
		
		_epPropertyValues.add(_espPropertyValues);
		
		_epConnectives.setLayout(new javax.swing.BoxLayout(_epConnectives, javax.swing.BoxLayout.Y_AXIS));
		
		_eAnd.setSelected(true);
		_eAnd.setText("and");
		_epConnectives.add(_eAnd);
		
		_eOr.setText("or");
		_epConnectives.add(_eOr);
		
		_epPropertyValues.add(_epConnectives);
		
		_edgesPanel.add(_epPropertyValues);
		
		_epAttributes.setLayout(new java.awt.BorderLayout());
		
		_epAttributes.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "visual attributes"));
		_espAttributes.setPreferredSize(new java.awt.Dimension(100, 100));
		_eAttributes.setModel(_edgeAttributes);
		_eAttributes.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		_eAttributes.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				_eAttributesMouseClicked(evt);
			}
		});
		
		_espAttributes.setViewportView(_eAttributes);
		
		_epAttributes.add(_espAttributes, java.awt.BorderLayout.CENTER);
		
		_edgesPanel.add(_epAttributes);
		
		_splitPane.setRightComponent(_edgesPanel);
		
		add(_splitPane, java.awt.BorderLayout.CENTER);
		
	}//GEN-END:initComponents

	private void _eMappingsKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event__eMappingsKeyReleased
		if (evt.getKeyCode() == KeyEvent.VK_DELETE) {
			Object[] mappings = _eMappings.getSelectedValues();
			for (int i=0, length=mappings.length; i<length; i++) {
				_edgeMappings.remove(mappings[i]);
			}
		}
	}//GEN-LAST:event__eMappingsKeyReleased

	private void _nMappingsKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event__nMappingsKeyReleased
		if (evt.getKeyCode() == KeyEvent.VK_DELETE) {
			Object[] mappings = _nMappings.getSelectedValues();
			for (int i=0, length=mappings.length; i<length; i++) {
				_nodeMappings.remove(mappings[i]);
			}
		}
	}//GEN-LAST:event__nMappingsKeyReleased

	private void _eAttributesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event__eAttributesMouseClicked
		if (evt.getClickCount() == 2) {
			int index = _eAttributes.locationToIndex(evt.getPoint());
			VisualAttribute attribute = (VisualAttribute) _eAttributes.getModel().getElementAt(index);
			EditorDialog dialog = new EditorDialog(attribute.getEditor());
			dialog.setLocationRelativeTo(this);
			EdgeVisualAttributeValue value = (EdgeVisualAttributeValue) dialog.showEditor();
			if (value != null) {
				Set selection = _edgeSelection.getSelection();
				if (selection.size() > 0) {
					Iterator iterator = selection.iterator();
					Point cell = (Point) iterator.next();
					String property = getEdgeProperty(cell.y);
					Object propertyValue = getEdgeValue(cell.x, cell.y);
					Sentence sentence = new AtomicSentence(property, propertyValue);
					while (iterator.hasNext()) {
						cell = (Point) iterator.next();
						property = getEdgeProperty(cell.y);
						propertyValue = getEdgeValue(cell.x, cell.y);
						Sentence atom = new AtomicSentence(property, propertyValue);
						Connective connective = getEdgeConnective();
						sentence = new CompositeSentence(sentence, connective, atom);
					}
					EdgeMapping mapping = new EdgeMapping(sentence, value);
					_edgeMappings.add(mapping);
				}
			}
		}
	}//GEN-LAST:event__eAttributesMouseClicked

	private void _nAttributesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event__nAttributesMouseClicked
		if (evt.getClickCount() == 2) {
			int index = _nAttributes.locationToIndex(evt.getPoint());
			VisualAttribute attribute = (VisualAttribute) _nAttributes.getModel().getElementAt(index);
			EditorDialog dialog = new EditorDialog(attribute.getEditor());
			dialog.setLocationRelativeTo(this);
			NodeVisualAttributeValue value = (NodeVisualAttributeValue) dialog.showEditor();
			if (value != null) {
				Set selection = _nodeSelection.getSelection();
				if (selection.size() > 0) {
					Iterator iterator = selection.iterator();
					Point cell = (Point) iterator.next();
					String property = getNodeProperty(cell.y);
					Object propertyValue = getNodeValue(cell.x, cell.y);
					Sentence sentence = new AtomicSentence(property, propertyValue);
					while (iterator.hasNext()) {
						cell = (Point) iterator.next();
						property = getNodeProperty(cell.y);
						propertyValue = getNodeValue(cell.x, cell.y);
						Sentence atom = new AtomicSentence(property, propertyValue);
						Connective connective = getNodeConnective();
						sentence = new CompositeSentence(sentence, connective, atom);
					}
					NodeMapping mapping = new NodeMapping(sentence, value);
					_nodeMappings.add(mapping);
				}
			}
		}
	}//GEN-LAST:event__nAttributesMouseClicked
	
}

/* 
 * $Log: MappingEditor.java,v $
 * Revision 1.1  2002/02/15 20:23:57  zcox
 * big time update
 *
 *
 *
 */
