/* $Id: MappingListModel.java,v 1.1 2002/02/15 20:23:57 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

import javax.swing.AbstractListModel;
import java.util.Set;
import java.util.Iterator;

/** 
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class MappingListModel extends AbstractListModel {
	private Set _mappings = null;
	
	/** Creates new MappingListModel */
	public MappingListModel(Set mappings) {
		super();
		_mappings = mappings;
	}
	
	public boolean add(Object mapping) {
		if (!(mapping instanceof NodeMapping || mapping instanceof EdgeMapping)) {
			throw new IllegalArgumentException("mapping not of type NodeMapping or EdgeMapping: " + mapping.getClass());
		}
		
		boolean added = _mappings.add(mapping);
		fireContentsChanged(this, 0, _mappings.size());
		
		return added;
	}
	
	public Object getElementAt(int index) {
		int i = 0;
		Object mapping = null;
		
		for (Iterator iterator = _mappings.iterator(); iterator.hasNext() && i <= index; i++) {
			mapping = iterator.next();
		}
		
		return mapping;
	}
	
	public Set getMappings() {
		return _mappings;
	}
	
	public int getSize() {
		return _mappings.size();
	}
	
	public boolean remove(Object mapping) {
		if (!(mapping instanceof NodeMapping || mapping instanceof EdgeMapping)) {
			throw new IllegalArgumentException("mapping not of type NodeMapping or EdgeMapping: " + mapping.getClass());
		}
		
		boolean removed = _mappings.remove(mapping);
		fireContentsChanged(this, 0, _mappings.size());
		
		return removed;
	}
	
}

/* 
 * $Log: MappingListModel.java,v $
 * Revision 1.1  2002/02/15 20:23:57  zcox
 * big time update
 *
 *
 *
 */
