/* $Id: OrConnective.java,v 1.1 2002/02/15 20:23:58 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

/** A <CODE>Connective</CODE> representing the OR logical connective.  This class is a Singleton;
 * the <CODE>getInstance</CODE> method provides access to the only <CODE>OrConnective</CODE> instance.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class OrConnective extends Object implements Connective {
	private static final OrConnective _instance = new OrConnective();
	private static final String _string = "or";
	
	/** Creates a new OrConnective.  Since this constructor is private, it cannot 
	 * be called outside of this class.
	 *
	 */
	private OrConnective() {
		
	}
	
	/** Evaluates the sentence <CODE>first</CODE> OR <CODE>second</CODE>.
	 *
	 * @param first the first term in the sentence.
	 * @param second the second term in the sentence.
	 * @return <CODE>true</CODE> if <CODE>first || second</CODE> evaluates to <CODE>true</CODE>; <CODE>false</CODE> otherwise.
	 */	
	public boolean evaluate(boolean first, boolean second) {
		return first || second;
	}
	
	/** Returns the only instance of <CODE>OrConnective</CODE>.
	 * @return the only instance of <CODE>OrConnective</CODE>.
	 */	
	public static OrConnective getInstance() {
		return _instance;
	}
	
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: OrConnective.java,v $
 * Revision 1.1  2002/02/15 20:23:58  zcox
 * big time update
 *
 *
 *
 */
