/* $Id: PropertyTableFilter.java,v 1.1 2002/01/15 20:37:00 zcox Exp $
 *
 *
 */

package fcmodeler.properties;

import javax.swing.table.DefaultTableModel;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import diva.canvas.selection.SelectionListener;
import diva.canvas.selection.SelectionEvent;
import diva.canvas.Figure;
import diva.canvas.connector.Connector;
import diva.util.SemanticObjectContainer;
import diva.util.PropertyContainer;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class PropertyTableFilter extends Object implements SelectionListener {
	private DefaultTableModel _nodeModel = null;
	private DefaultTableModel _edgeModel = null;
	
	/** Creates new PropertyTableFilter */
	public PropertyTableFilter(DefaultTableModel nodeModel, DefaultTableModel edgeModel) {
		_nodeModel = nodeModel;
		_edgeModel = edgeModel;
	}
	
	public void add(Object obj) {
		if (obj instanceof Connector) {
			addEdge((Connector) obj);
		}
		else if (obj instanceof Figure) {
			addNode((Figure) obj);
		}
	}
	
	private void add(Object obj, DefaultTableModel model) {
		SemanticObjectContainer soc = null;
		PropertyContainer pc = null;
		List newRow = null;
		Object semObj = null;
		
		if (obj instanceof SemanticObjectContainer) {
			soc = (SemanticObjectContainer) obj;
			semObj = soc.getSemanticObject();
			if (!containsRow(model, semObj)) {
				newRow = new ArrayList();
				newRow.add(semObj);
				if (obj instanceof PropertyContainer) {
					pc = (PropertyContainer) obj;
					for (int i=1; i<model.getColumnCount(); i++) {
						//model.setValueAt(pc.getProperty(model.getColumnName(i)), model.getRowCount()-1, i);
						newRow.add(pc.getProperty(model.getColumnName(i)));
					}
				}
				model.addRow(newRow.toArray());
			}
		}
	}
	
	public void addEdge(Connector edge) {
		add(edge.getUserObject(), _edgeModel);
	}
	
	public void addNode(Figure node) {
		add(node.getUserObject(), _nodeModel);
	}
	
	private boolean containsRow(DefaultTableModel model, Object obj) {
		boolean contains = false;
		
		for (int i=0; i<model.getRowCount(); i++) {
			if (model.getValueAt(i, 0).equals(obj)) {
				contains = true;
				break;
			}
		}
		
		return contains;
	}
	
	public void remove(Object obj) {
		if (obj instanceof Connector) {
			removeEdge((Connector) obj);

		}
		else if (obj instanceof Figure) {
			removeNode((Figure) obj);
		}
	}
	
	private void remove(Object obj, DefaultTableModel model) {
		SemanticObjectContainer soc = null;
		Object semanticObj = null;
		
		if (obj instanceof SemanticObjectContainer) {
			soc = (SemanticObjectContainer) obj;
			semanticObj = soc.getSemanticObject();
			for (int i=0; i<model.getRowCount(); i++) {
				if (model.getValueAt(i, 0).equals(semanticObj)) {
					model.removeRow(i);
					break;
				}
			}
		}
	}
	
	public void removeEdge(Connector edge) {
		remove(edge.getUserObject(), _edgeModel);
	}
	
	public void removeNode(Figure node) {
		remove(node.getUserObject(), _nodeModel);
	}
	
	public void selectionChanged(SelectionEvent evt) {
		Iterator adds = evt.getSelectionAdditions();
		Iterator rems = evt.getSelectionRemovals();
		Object add = null, rem = null;
		
		while (adds.hasNext()) {
			add = adds.next();
			add(add);
		}
		
		while (rems.hasNext()) {
			rem = rems.next();
			remove(rem);
		}
	}
}

/* 
 * $Log: PropertyTableFilter.java,v $
 * Revision 1.1  2002/01/15 20:37:00  zcox
 * initial commit of fcmodeler.properties package
 *
 *
 *
 */
