/* $Id: PropertyViewer.java,v 1.1 2002/01/15 20:37:00 zcox Exp $
 *
 *
 */

package fcmodeler.properties;

import java.util.List;
import java.util.Iterator;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import javax.swing.table.DefaultTableModel;

import diva.canvas.selection.SelectionModel;
import diva.canvas.selection.SelectionListener;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class PropertyViewer extends javax.swing.JFrame implements WindowListener {
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JSplitPane _splitPane;
	private javax.swing.JScrollPane _nodeScrollPane;
	private javax.swing.JTable _nodeTable;
	private javax.swing.JScrollPane _edgeScrollPane;
	private javax.swing.JTable _edgeTable;
	// End of variables declaration//GEN-END:variables
	private SelectionModel _selectionModel = null;
	private PropertyTableFilter _propertyFilter = null;
	
	public PropertyViewer() {
		DefaultTableModel nodeModel = new DefaultTableModel(new Object[1][1], new Object[1]) {
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		DefaultTableModel edgeModel = new DefaultTableModel(new Object[1][1], new Object[1]) {
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		_propertyFilter = new PropertyTableFilter(nodeModel, edgeModel);
		
		initComponents();
		addWindowListener(this);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		_nodeTable.setModel(nodeModel);
		_edgeTable.setModel(edgeModel);
		pack();
		_splitPane.setDividerLocation(0.5d);
	}
	
	/** Creates new form PropertyViewer */
	public PropertyViewer(SelectionModel selectionModel, List nodeLabels, List edgeLabels) {
		DefaultTableModel nodeModel = new DefaultTableModel(new Object[1][1], nodeLabels.toArray()) {
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		DefaultTableModel edgeModel = new DefaultTableModel(new Object[1][1], edgeLabels.toArray()) {
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		_propertyFilter = new PropertyTableFilter(nodeModel, edgeModel);
		setSelectionModel(selectionModel);
		
		initComponents();
		addWindowListener(this);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		_nodeTable.setModel(nodeModel);
		_edgeTable.setModel(edgeModel);
		pack();
		_splitPane.setDividerLocation(0.5d);
	}
	
	public void clearTables() {
		DefaultTableModel model = (DefaultTableModel) _nodeTable.getModel();
		for (int i=model.getRowCount()-1; i>=0; i--) {
			model.removeRow(i);
		}
		model = (DefaultTableModel) _edgeTable.getModel();
		for (int i=model.getRowCount()-1; i>=0; i--) {
			model.removeRow(i);
		}
	}
	
	public PropertyTableFilter getPropertyTableFilter() {
		return _propertyFilter;
	}
	
	public SelectionModel getSelectionModel() {
		return _selectionModel;
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the FormEditor.
	 */
	private void initComponents() {//GEN-BEGIN:initComponents
		_splitPane = new javax.swing.JSplitPane();
		_nodeScrollPane = new javax.swing.JScrollPane();
		_nodeTable = new javax.swing.JTable();
		_edgeScrollPane = new javax.swing.JScrollPane();
		_edgeTable = new javax.swing.JTable();
		setTitle("property viewer");
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		}
		);
		
		_splitPane.setOrientation(javax.swing.SwingConstants.HORIZONTAL);
		
		
  		_nodeTable.setPreferredScrollableViewportSize(new java.awt.Dimension(400, 100));
    		_nodeScrollPane.setViewportView(_nodeTable);
    		
    		_splitPane.setTopComponent(_nodeScrollPane);
  		
  		
		
  		_edgeTable.setPreferredScrollableViewportSize(new java.awt.Dimension(400, 100));
    		_edgeScrollPane.setViewportView(_edgeTable);
    		
    		_splitPane.setBottomComponent(_edgeScrollPane);
  		
  		
		getContentPane().add(_splitPane, java.awt.BorderLayout.CENTER);
		
	}//GEN-END:initComponents
	
	/** Exit the Application */
	private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
		//System.exit (0);
	}//GEN-LAST:event_exitForm
	
	public void setColumnLabels(List nodeLabels, List edgeLabels) {
		DefaultTableModel model = (DefaultTableModel) _nodeTable.getModel();
		model.setDataVector(new Object[1][1], nodeLabels.toArray());
		model = (DefaultTableModel) _edgeTable.getModel();
		model.setDataVector(new Object[1][1], edgeLabels.toArray());
	}
	
	public void setSelectionModel(SelectionModel selectionModel) {
		Iterator selections = null;
		
		clearTables();
		if (_selectionModel != null) {
			_selectionModel.removeSelectionListener(_propertyFilter);
		}
		_selectionModel = selectionModel;
		_selectionModel.addSelectionListener(_propertyFilter);
		selections = _selectionModel.getSelection();
		while (selections.hasNext()) {
			_propertyFilter.add(selections.next());
		}
	}
	
	/*public void show() {
		_selectionModel.addSelectionListener(_propertyFilter);
		super.show();
	}*/
	
	public void windowActivated(WindowEvent evt) {
		
	}
	
	public void windowClosed(WindowEvent evt) {
		
	}
	
	public void windowClosing(WindowEvent evt) {
		clearTables();
		_selectionModel.removeSelectionListener(_propertyFilter);
	}
	
	public void windowDeactivated(WindowEvent evt) {
		
	}
	
	public void windowDeiconified(WindowEvent evt) {
		
	}
	
	public void windowIconified(WindowEvent evt) {
		
	}
	
	public void windowOpened(WindowEvent evt) {
		
	}
}

/* 
 * $Log: PropertyViewer.java,v $
 * Revision 1.1  2002/01/15 20:37:00  zcox
 * initial commit of fcmodeler.properties package
 *
 *
 *
 */
