/* $Id: Cycle.java,v 1.1 2002/01/15 20:38:35 zcox Exp $
 *
 *
 */

package fcmodeler.subgraph;

import java.util.Collection;
import java.util.Iterator;
import java.util.ListIterator;

import diva.util.SemanticObjectContainer;
import diva.graph.model.Node;
import diva.graph.model.Edge;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class Cycle extends Path {
	
	/** Creates new Cycle */
	public Cycle() {
		super();
	}
	
	public Cycle(Collection collection) {
		super(collection);
	}
	
	public boolean equals(Object obj) {
		boolean equals = false;
		Cycle other = null;
		int index = -1;
		Object thisLast = null, otherLast = null;
		
		//make sure the other object is a cycle
		if (obj instanceof Cycle) {
			other = (Cycle) obj;
			
			//is they're not the same size they're obviously not equal
			if (size() == other.size()) {
				//find the other cycle's index of this cycle's first element
				index = other.indexOf(get(0));
				
				//if other cycle doesn't contain it, they're not equal
				if (index >= 0) {
					equals = true;
					
					//need to temporarily remove the last elements, because they're repeated nodes
					thisLast = remove(size()-1);
					otherLast = other.remove(other.size()-1);
					
					//for each of the other elements in this cycle
					for (int i=1; i<size(); i++) {
						//compare this element to the other cycle's next element circularly
						if (other.indexOf(get(i)) != (index + i) % size()) {
							equals = false;
							break;
						}
					}
					
					//restore the last elements
					add(thisLast);
					other.add(otherLast);
				}
			}
		}
		else {
			equals = super.equals(obj);
		}
		
		return equals;
	}
	
	public int hashCode() {
		int hashCode = 0;
		Iterator iterator = null;
		
		iterator = iterator();
		while (iterator.hasNext()) {
			hashCode += iterator.next().hashCode();
		}
		
		return hashCode;
	}
	
	public boolean isValid() {
		boolean valid = true;
		ListIterator elements = listIterator();
		Node tail = null, head = null, first = null;
		Edge edge = null;
		Object obj = null;
		
		while (elements.hasNext()) {
			obj = elements.next();
			if (first == null) {
				first = (Node) obj;
			}
			else if (obj instanceof Edge) {
				elements.previous();
				tail = (Node) elements.previous();
				elements.next();
				edge = (Edge) elements.next();
				head = (Node) elements.next();
				if (!edge.getTail().equals(tail) || !edge.getHead().equals(head)) {
					valid = false;
					System.out.println("edge " + edge.getSemanticObject() + " should have tail " + 
						edge.getTail().getSemanticObject() + " and head " + edge.getHead().
						getSemanticObject());
					System.out.println("instead, tail " + tail.getSemanticObject() + " and head " + 
						head.getSemanticObject() + " were specified");
					break;
				}
			}
		}
		
		if (!elements.previous().equals(first)) {
			valid = false;
		}
		
		return valid;
	}
	
	public String toString() {
		String s = "[";
		SemanticObjectContainer soc = null;
		
		if (!isEmpty()) {
			s = (size()-1)/2 + "-cycle: [";
			for (int i=0; i<size(); i++) {
				soc = (SemanticObjectContainer) get(i);
				s += soc.getSemanticObject().toString();
				if (i < size()-1) {
					s += ", ";
				}
			}
		}
		s += "]";
		
		return s;
	}
}

/* 
 * $Log: Cycle.java,v $
 * Revision 1.1  2002/01/15 20:38:35  zcox
 * initial commit of some subgraph classes
 *
 * 
 */
