/* $Id: SubgraphFactory.java,v 1.2 2002/01/15 20:38:01 zcox Exp $
 *
 *
 */

package fcmodeler.subgraph;

import java.util.Iterator;
import java.util.Collection;
import java.util.HashSet;

import diva.graph.model.GraphModel;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.BasicGraph;
import diva.graph.model.BasicNode;
import diva.graph.model.BasicEdge;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class SubgraphFactory extends Object {
	private static SubgraphFactory _instance = null;
	
	/** Creates new SubgraphFactory */
	private SubgraphFactory() {
		
	}
	
	public GraphModel createFromEdges(Collection edges) {
		GraphModel model = null;
		Iterator iterator = edges.iterator();
		Edge edge = null;
		Node tail = null, head = null;
		SubgraphEdge subEdge = null;
		SubgraphNode subTail = null, subHead = null;
		
		//create the new graph model to hold the subgraph
		model = new GraphModel(new SubgraphImpl());
		
		//for each edge in the collection
		while (iterator.hasNext()) {
			edge = (Edge) iterator.next();
			
			//get the edge's head & tail nodes
			tail = edge.getTail();
			head = edge.getHead();
			
			//this only needs to be done once, but...
			((Subgraph) model.getGraph()).setOriginalGraph((BasicGraph) tail.getParent());
			
			//if subTail hasn't been created & added, do it
			subTail = ((Subgraph) model.getGraph()).getNode((BasicNode) tail);
			if (subTail == null) {
				subTail = (SubgraphNode) model.createNode(tail);
				model.addNode(subTail);
			}
			
			//if subHead hasn't been created & added, do it
			subHead = ((Subgraph) model.getGraph()).getNode((BasicNode) head);
			if (subHead == null) {
				subHead = (SubgraphNode) model.createNode(head);
				model.addNode(subHead);
			}
			
			//create & connect subEdge
			subEdge = (SubgraphEdge) model.createEdge(edge);
			model.connectEdge(subEdge, subTail, subHead);
		}
		
		return model;
	}
	
	public GraphModel createFromEdgesAndNodes(Collection edges, Collection nodes) {
		throw new UnsupportedOperationException("SubgraphFactory.createFromEdgesAndNodes() not supported");
	}
	
	public GraphModel createFromNodes(Collection nodes) {
		GraphModel model = null;
		Iterator iterator = nodes.iterator(), outEdges = null;
		Node tail = null, head = null;
		Edge edge = null;
		SubgraphNode subTail = null, subHead = null;
		SubgraphEdge subEdge = null;
		
		//create the new graph model to hold the subgraph
		model = new GraphModel(new SubgraphImpl());
		
		//for each node in the collection
		while (iterator.hasNext()) {
			tail = (Node) iterator.next();
			subTail = null;
			
			//this only needs to be done once, but...
			((Subgraph) model.getGraph()).setOriginalGraph((BasicGraph) tail.getParent());
			
			//for each out-edge of tail
			outEdges = tail.outEdges();
			while (outEdges.hasNext()) {
				edge = (Edge) outEdges.next();
				
				//check if this is a valid tail-edge-head combination
				head = edge.getHead();
				if (nodes.contains(head)) {
					//if subTail hasn't been created & added, do it
					subTail = ((Subgraph) model.getGraph()).getNode((BasicNode) tail);
					if (subTail == null) {
						subTail = (SubgraphNode) model.createNode(tail);
						model.addNode(subTail);
					}
					
					//if subHead hasn't been created & added, do it
					subHead = ((Subgraph) model.getGraph()).getNode((BasicNode) head);
					if (subHead == null) {
						subHead = (SubgraphNode) model.createNode(head);
						model.addNode(subHead);
					}
					
					//create & connect subEdge
					subEdge = (SubgraphEdge) model.createEdge(edge);
					model.connectEdge(subEdge, subTail, subHead);
				}
			}
		}
		
		return model;
	}
	
	public GraphModel createFromNodes(Collection nodes, int dist) {
		GraphModel model = null;
		Collection fullSet = null, lastSet = null, newSet = null;
		Node tail = null, head = null;
		Edge edge = null;
		Iterator iterator = null, outEdges = null;
		
		//initialize the collections
		fullSet = new HashSet(nodes);
		lastSet = new HashSet(nodes);
		newSet = new HashSet();
		
		//go to a depth of dist away from the initial set of nodes
		for (int i=0; i<dist; i++) {
			//for each node in lastSet
			iterator = lastSet.iterator();
			while (iterator.hasNext()) {
				tail = (Node) iterator.next();
				
				//for each out-edge of tail
				outEdges = tail.outEdges();
				while (outEdges.hasNext()) {
					edge = (Edge) outEdges.next();
					
					//if head hasn't already been visited
					head = edge.getHead();
					if (!fullSet.contains(head)) {
						//add head to fullSet and newSet
						fullSet.add(head);
						newSet.add(head);
					}
				}
			}
			
			//swap lastSet & newSet
			lastSet.clear();
			lastSet.addAll(newSet);
			newSet.clear();
		}
		
		//pass the full node collection to createFromNode() and get back a populated GraphModel
		model = createFromNodes(fullSet);
		
		return model;
	}
	
	public GraphModel createFromNodesAndEdges(Collection nodes, Collection edges) {
		GraphModel model = null;
		Iterator iterator = null;
		Node tail = null, head = null;
		Edge edge = null;
		SubgraphNode subTail = null, subHead = null;
		SubgraphEdge subEdge = null;
		
		//create the new graph model to hold the subgraph
		model = new GraphModel(new SubgraphImpl());
		
		//add all of the nodes to the subgraph
		iterator = nodes.iterator();
		while (iterator.hasNext()) {
			tail = (Node) iterator.next();
			
			//this only needs to be done once, but...
			((Subgraph) model.getGraph()).setOriginalGraph((BasicGraph) tail.getParent());
			
			//create & add the node to subgraph
			subTail = (SubgraphNode) model.createNode(tail);
			model.addNode(subTail);
		}
		
		//add only the edges with both end-vertices in the collection to the subgraph
		iterator = edges.iterator();
		while (iterator.hasNext()) {
			edge = (Edge) iterator.next();
			tail = edge.getTail();
			head = edge.getHead();
			if (nodes.contains(tail) && nodes.contains(head)) {
				subTail = ((Subgraph) model.getGraph()).getNode((BasicNode) tail);
				subHead = ((Subgraph) model.getGraph()).getNode((BasicNode) head);
				subEdge = (SubgraphEdge) model.createEdge(edge);
				model.connectEdge(subEdge, subTail, subHead);
			}
		}
		
		return model;
	}
	
	public static SubgraphFactory getInstance() {
		if (_instance == null) {
			_instance = new SubgraphFactory();
		}
		
		return _instance;
	}
}

/* 
 * $Log: SubgraphFactory.java,v $
 * Revision 1.2  2002/01/15 20:38:01  zcox
 * some changes
 *
 *
 *
 */
