/* $Id: SubgraphImpl.java,v 1.1 2002/01/15 20:38:35 zcox Exp $
 *
 *
 */

package fcmodeler.subgraph;

import diva.graph.model.GraphImpl;
import diva.graph.model.CompositeNode;
import diva.graph.model.Edge;
import diva.graph.model.Graph;
import diva.graph.model.Node;
import diva.graph.model.BasicEdge;
import diva.graph.model.BasicGraph;
import diva.graph.model.BasicNode;
import diva.graph.model.GraphException;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class SubgraphImpl extends Object implements GraphImpl {
	
	/** Creates new SubgraphImpl */
	public SubgraphImpl() {
		
	}
	
	public void addNode(Node node, Graph graph) {
		SubgraphNode subNode = (SubgraphNode) node;
		Subgraph subgraph = (Subgraph) graph;
		
		if (subgraph.getOriginalGraph().contains(subNode.getOriginalNode())) {
			subgraph.add(subNode);
		}
	}
	
	public CompositeNode createCompositeNode(Object semanticObject) {
		System.out.println("SubgraphImpl doesn't support CompositeNode's");
		
		return null;
	}
	
	public Edge createEdge(Object semanticObject) {
		SubgraphEdge subEdge = null;
		
		if (semanticObject instanceof BasicEdge) {
			subEdge = new SubgraphEdge((BasicEdge) semanticObject);
		}
		else {
			System.out.println("can't create SubgraphEdge without a BasicEdge");
		}
		
		return subEdge;
	}
	
	public Graph createGraph(Object semanticObject) {
		Subgraph subgraph = null;
		
		if (semanticObject instanceof BasicGraph) {
			subgraph = new Subgraph((BasicGraph) semanticObject);
		}
		else if (semanticObject == null) {
			subgraph = new Subgraph(null);
		}
		else {
			System.out.println("can't create Subgraph without a BasicGraph");
		}
		
		return subgraph;
	}
	
	public Node createNode(Object semanticObject) {
		SubgraphNode subNode = null;
		
		if (semanticObject instanceof BasicNode) {
			subNode = new SubgraphNode((BasicNode) semanticObject);
		}
		else {
			System.out.println("can't create SubgraphNode without a BasicNode");
		}
		
		return subNode;
	}
	
	public void removeNode(Node node) {
		SubgraphNode subNode = (SubgraphNode) node;
		Subgraph parent = (Subgraph) subNode.getParent();
		
		if (parent == null) {
			String err = "Trying to remove node with no parent";
			throw new GraphException(err);
		}
		
		parent.delete(subNode);
	}
	
	public void setEdgeHead(Edge edge, Node node) {
		SubgraphEdge subEdge = (SubgraphEdge) edge;
		SubgraphNode subNode = (SubgraphNode) node;
		
		if (subEdge.getOriginalEdge().getHead() == subNode.getOriginalNode()) {
			subEdge.setHead(subNode);
		}
		else {
			System.out.println("originalEdge of SubgraphEdge did not have same head as SubgraphNode's originalNode");
		}
	}
	
	public void setEdgeTail(Edge edge, Node node) {
		SubgraphEdge subEdge = (SubgraphEdge) edge;
		SubgraphNode subNode = (SubgraphNode) node;
		
		if (subEdge.getOriginalEdge().getTail() == subNode.getOriginalNode()) {
			subEdge.setTail(subNode);
		}
		else {
			System.out.println("originalEdge of SubgraphEdge did not have same tail as SubgraphNode's originalNode");
		}
	}
}

/* 
 * $Log: SubgraphImpl.java,v $
 * Revision 1.1  2002/01/15 20:38:35  zcox
 * initial commit of some subgraph classes
 *
 *
 *
 */
