/* $Id: ChildNodeList.java,v 1.1 2002/02/15 20:20:36 zcox Exp $
 *
 *
 */

package fcmodeler.util;

import java.util.List;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/** A node list providing the child nodes of a certain node that have a certain name.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class ChildNodeList extends Object implements NodeList {
	private List _children = null;
	
	/** Creates a new <CODE>ChildNodeList</CODE> using the specified node and node name.
	 *
	 * @param parent the parent node.
	 * @param nodeName the node name that child nodes must have.
	 */
	public ChildNodeList(Node parent, String nodeName) {
		if (parent == null) { throw new NullPointerException("parent was null"); }
		if (nodeName == null) { throw new NullPointerException("nodeName was null"); }
		
		_children = new ArrayList();
		NodeList nodeList = parent.getChildNodes();
		for (int i=0, length=nodeList.getLength(); i<length; i++) {
			Node child = nodeList.item(i);
			if (child.getNodeName().equals(nodeName)) {
				_children.add(child);
			}
		}
	}
	
	/** Returns the number of nodes in this list.
	 *
	 * @return the number of nodes in this list.
	 */	
	public int getLength() {
		return _children.size();
	}
	
	/** Returns the node at the specified index.
	 *
	 * @param index the index of the desired node.
	 * @return the node at the specified index.
	 */	
	public Node item(int index) {
		return (Node) _children.get(index);
	}
	
}

/* 
 * $Log: ChildNodeList.java,v $
 * Revision 1.1  2002/02/15 20:20:36  zcox
 * big time update
 *
 *
 *
 */
