/* $Id: NoTextNodeList.java,v 1.1 2002/02/15 20:20:36 zcox Exp $
 *
 *
 */

package fcmodeler.util;

import java.util.List;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/** A node list containing all nodes of another node list except text nodes.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class NoTextNodeList extends Object implements NodeList {
	private final List _list;
	
	/** Creates a new <CODE>NoTestNodeList</CODE> using the specified node list.
	 *
	 * @param nodeList the node list.
	 */
	public NoTextNodeList(NodeList nodeList) {
		if (nodeList == null) { throw new NullPointerException("nodeList was null"); }
		
		_list = new ArrayList();
		for (int i=0, length=nodeList.getLength(); i<length; i++) {
			Node node = nodeList.item(i);
			if (node.getNodeType() != Node.TEXT_NODE) {
				_list.add(node);
			}
		}
	}
	
	/** Returns the number of nodes in this list.
	 *
	 * @return the number of nodes in this list.
	 */	
	public int getLength() {
		return _list.size();
	}
	
	/** Returns the node at the specified index.
	 *
	 * @param index the index of the desired node.
	 * @return the node at the specified index.
	 */	
	public Node item(int index) {
		return (Node) _list.get(index);
	}
	
}

/* 
 * $Log: NoTextNodeList.java,v $
 * Revision 1.1  2002/02/15 20:20:36  zcox
 * big time update
 *
 *
 *
 */
