/* $Id: SetChangeListener.java,v 1.1 2002/02/15 20:20:36 zcox Exp $
 *
 *
 */

package fcmodeler.util;

import java.util.EventListener;

/** The listener interface for receiving set change events.  The class that is interested in the
 * modifications to a set implements this interface.  The object created with that class is then
 * registered with an observable set using the set's <CODE>addSetChangeListener</CODE> method.
 * When an element is added to or removed from the set, the relevant method in the listener object
 * is called, and the <CODE>SetChangeEvent</CODE> is passed to it.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public interface SetChangeListener extends EventListener {
	/** Invoked when an element is added to the observed set.
	 *
	 * @param event the object representing the element's addition.
	 */	
	public void elementAdded(SetChangeEvent event);
	
	/** Invoked when an element is removed from the observed set.
	 *
	 * @param event the object represent the element's removal.
	 */	
	public void elementRemoved(SetChangeEvent event);
}

/* 
 * $Log: SetChangeListener.java,v $
 * Revision 1.1  2002/02/15 20:20:36  zcox
 * big time update
 *
 *
 *
 */
