/* $Id: AbstractEdgeFigure.java,v 1.4 2002/02/15 20:56:38 zcox Exp $
 *
 *
 */

package fcmodeler.view;

import java.awt.BasicStroke;

import diva.canvas.Site;
import diva.canvas.toolbox.PaintedShape;
import diva.canvas.connector.AbstractConnector;
import diva.canvas.connector.ConnectorEnd;

/** An abstract implementation of <CODE>EdgeFigure</CODE>.  <CODE>AbstractEdgeFigure</CODE> provides
 * support for the <CODE>PermanentFigure</CODE> aspect of
 * <CODE>EdgeFigure</CODE>, as well as skeletal implementations of all of the methods specified
 * in the <CODE>EdgeFigure</CODE> interface.  It does not provide any line geometry; this is left for concrete
 * subclasses to handle.
 *
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public abstract class AbstractEdgeFigure extends AbstractConnector implements EdgeFigure {
	private PaintedShape _permanent = null;
	
	/** Creates a new <CODE>AbstractEdgeFigure</CODE> between the given sites.
	 *
	 * @param tail the site on the tail node to connect to.
	 * @param head the site on the head node to connect to.
	 */
	public AbstractEdgeFigure(Site tail, Site head) {
		super(tail, head);
		
		_permanent = new PaintedShape();
		_permanent.setStroke(new BasicStroke());
		_permanent.setLineWidth(1.0f);
	}
	
	/** Returns the <CODE>PaintedShape</CODE> used for painting this edge figure.
	 *
	 * @return the <CODE>PaintedShape</CODE> used for painting this edge figure.
	 */
	public PaintedShape getFigure() {
		return getPaintedShape();
	}
	
	/** Returns the arrowhead used at the head end of this edge figure.
	 *
	 * @return the arrowhead used at the head end of this edge figure.
	 */	
	public ConnectorEnd getHeadEnd() {
		return super.getHeadEnd();
	}
	
	/** Returns the <CODE>PaintedShape</CODE> used for permanent visual attribute storage in this edge figure.
	 *
	 * @return the <CODE>PaintedShape</CODE> used for permanent visual attribute storage in this edge figure.
	 */
	public PaintedShape getPermanentFigure() {
		return _permanent;
	}
	
	/** Returns the arrowhead used at the tail end of this edge figure.
	 *
	 * @return the arrowhead used at the tail end of this edge figure.
	 */	
	public ConnectorEnd getTailEnd() {
		return super.getTailEnd();
	}
	
	/** Sets the arrowhead at the head end of this edge figure to the specified <CODE>ConnectorEnd</CODE>.
	 *
	 * @param end the new head end arrowhead for this edge figure.
	 */	
	public void setHeadEnd(ConnectorEnd end) {
		super.setHeadEnd(end);
	}
	
	/** Sets the arrowhead at the tail end of this edge figure to the specified <CODE>ConnectorEnd</CODE>.
	 *
	 * @param end the new tail end arrowhead for this edge figure.
	 */	
	public void setTailEnd(ConnectorEnd end) {
		super.setTailEnd(end);
	}
	
	/** Sets the visual attributes for this edge to those stored in the permanent
	 * <CODE>PaintedShape</CODE>.  This can be used to revert back to a permanent visual state
	 * after modifying the appearance of this edge.
	 *
	 */	
	public void usePermanents() {
		setLineWidth(_permanent.getLineWidth());
		setStroke(_permanent.getStroke());
		setStrokePaint(_permanent.outlinePaint);
	}
	
}

/* 
 * $Log: AbstractEdgeFigure.java,v $
 * Revision 1.4  2002/02/15 20:56:38  zcox
 * big time update
 *
 * Revision 1.3  2002/01/23 21:23:20  zcox
 * added javadoc comments
 *
 * Revision 1.2  2002/01/22 22:59:30  zcox
 * added cvs tags
 *
 * Revision 1.1  2002/01/22 22:53:38  zcox
 * modifications to EdgeFigure classes
 *
 *
 *
 */
