/* $Id: BezierEdgeFigureRenderer.java,v 1.7 2002/04/21 20:39:43 xein Exp $
 *
 *
 */

package fcmodeler.view;

import diva.graph.EdgeRenderer;
import diva.graph.model.Edge;
import diva.canvas.Site;
import diva.canvas.connector.Connector;
import diva.canvas.connector.Arrowhead;

import fcmodeler.view.figures.FilledRectangleHead; //xein

/** A factory used to create a <CODE>BezierEdgeFigure</CODE> using a specified <CODE>Edge</CODE>.
 *
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.7 $
 * @since JDK1.3
 */

public class BezierEdgeFigureRenderer implements EdgeRenderer {

	/** Creates a new <CODE>BezierEdgeFigureRenderer</CODE>.
	 *
	 */
	public BezierEdgeFigureRenderer() {

	}

	/** Creates a <CODE>BezierEdgeFigure</CODE> to be used as a visual representation for the
	 * specified <CODE>Edge</CODE>.
	 *
	 * @param edge the edge to render.
	 * @param tailSite the site on the tail node figure to connect this edge figure to.
	 * @param headSite the site on the head node figure to connect this edge figure to.
	 * @return a visual representation of the specified <CODE>Edge</CODE>, of type <CODE>BezierEdgeFigure</CODE>.
	 */
	public Connector render(Edge edge, Site tailSite, Site headSite) {
		EdgeFigure edgeFigure = null;

		edgeFigure = new BezierEdgeFigure(tailSite, headSite);
		edgeFigure.setUserObject(edge);
		if (edge.isDirected()) {
			edgeFigure.setHeadEnd(new Arrowhead(headSite.getX(), headSite.getY(), headSite.getNormal()));
		}

		return edgeFigure;
	}

}


/*
 * $Log: BezierEdgeFigureRenderer.java,v $
 * Revision 1.7  2002/04/21 20:39:43  xein
 * - revert back to having Arrowhead as the initial ConnectorEnd for the EdgeFigure
 *
 * Revision 1.6  2002/03/23 03:33:12  xein
 * *** empty log message ***
 *
 * Revision 1.5  2002/03/22 16:54:30  xein
 * - Changed NodeShapeEditor and NodeShapeValue to include   DiamondNodeShape
 * - BezierEdgeFigureRenderer was used to test FilledRectangleHead connector   end
 * - No change to RectangleNodeShape
 *
 * Revision 1.4  2002/01/23 21:23:20  zcox
 * added javadoc comments
 *
 * Revision 1.3  2002/01/22 22:59:30  zcox
 * added cvs tags
 *
 * Revision 1.2  2002/01/22 22:57:03  zcox
 * added cvs tags
 *
 *
 *
 *
 */
