/* $Id: DebugConnectorInteractor.java,v 1.1 2002/01/15 20:46:33 zcox Exp $
 *
 *
 */

package fcmodeler.view;

import java.awt.geom.*;
import java.util.*;
import diva.canvas.*;
import diva.canvas.connector.*;
import diva.canvas.manipulator.*;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class DebugConnectorInteractor extends ConnectorInteractor {
	private ConnectorManipulator _cm = null;
	private ArrayList _connectorListeners = new ArrayList();
	
	/** Creates new DebugConnectorInteractor */
	public DebugConnectorInteractor(ConnectorManipulator m) {
		super(m);
		_cm = m;
	}
	
	public void addConnectorListener (ConnectorListener l) {
		_connectorListeners.add(l);
	}
	
	private void attach (Site site) {
		if (getHandle().getSite() == getConnector().getHeadSite()) {
			getConnector().setHeadSite(site);
		} else {
			getConnector().setTailSite(site);
		}
		getHandle().setSite(site);
		getHandle().repaint();
	}
	
	private void detach (double x, double y) {
		Site newSite = new AutonomousSite(getConnector().getTransformContext(), x, y);
		
		if (getHandle().getSite() == getConnector().getHeadSite()) {
			getConnector().setHeadSite(newSite);
		} else {
			getConnector().setTailSite(newSite);
		}
		getHandle().setSite(newSite);
		getHandle().repaint();
	}
	
	private Site findSite (Figure f, double x, double y) {
		GrabHandle handle = getHandle();
		Connector connector = getConnector();
		ConnectorTarget target = _cm.getConnectorTarget();
		
		if (handle.getSite() == connector.getHeadSite()) {
			if (f != connector.getTailSite().getFigure()) {
				return  _cm.getConnectorTarget().getHeadSite(f, x, y);
			}
		} else {
			if (f != connector.getHeadSite().getFigure()) {
				return  _cm.getConnectorTarget().getTailSite(f, x, y);
			}
		}
		return null;
	}
	
	private Site findSite (Site f, double x, double y) {
		GrabHandle handle = getHandle();
		Connector connector = getConnector();
		ConnectorTarget target = _cm.getConnectorTarget();
		
		if (handle.getSite() == connector.getHeadSite()) {
			return target.getHeadSite(f, x, y);
		} else {
			return target.getTailSite(f, x, y);
		}
	}
	
	private void fireConnectorEvent (int id) {
		int end;
		
		if (getHandle().getSite() == getConnector().getHeadSite()) {
			end = ConnectorEvent.HEAD_END;
		} else {
			end = ConnectorEvent.TAIL_END;
		}
		ConnectorEvent event = new ConnectorEvent(
		id,
		getConnector().getLayer(),
		getTarget(),
		getConnector(),
		end);
		
		for (Iterator i = _connectorListeners.iterator(); i.hasNext(); ) {
			ConnectorListener l = (ConnectorListener) i.next();
			switch(id) {
				case ConnectorEvent.CONNECTOR_DRAGGED:
					l.connectorDragged(event);
					break;
				case ConnectorEvent.CONNECTOR_DROPPED:
					l.connectorDropped(event);
					break;
				case ConnectorEvent.CONNECTOR_SNAPPED:
					l.connectorSnapped(event);
					break;
				case ConnectorEvent.CONNECTOR_UNSNAPPED:
					l.connectorUnsnapped(event);
					break;
			}
		}
	}
	
	public void removeConnectorListener (ConnectorListener l) {
		_connectorListeners.remove(l);
	}
	
	public void translate(diva.canvas.event.LayerEvent e, double dx, double dy) {
		double x = e.getLayerX();
		double y = e.getLayerY();
		Figure target = getTarget();
		GrabHandle handle = getHandle();
		Connector connector = getConnector();
		
		double h = _cm.getSnapHalo();
		Rectangle2D mouseRect = new Rectangle2D.Double(x-h, y-h, 2*h, 2*h);
		
		// If we were over a target, see if we still are
		if (target != null) {
			if (target.intersects(mouseRect)) {
				// We're still over, so snap to the nearest site -- FIXME
				Site current = handle.getSite();
				Site snap = findSite(current, x, y);
				if (snap != null && snap != current) {
					attach(snap);
					fireConnectorEvent(ConnectorEvent.CONNECTOR_SNAPPED);
				}
				
			} else {
				// Not over any more, so leave
				detach(x, y);
				fireConnectorEvent(ConnectorEvent.CONNECTOR_UNSNAPPED);
				target = null;
			}
			
		} else {
			// Look for a target. The container we need to pick
			// in is the one that has a transform context
			// FIXME: this doesn't deal properly with hierarchy
			FigureContainer container;
			Object component = connector.getTransformContext().getComponent();
			if (component instanceof CanvasPane) {
				container = (FigureContainer) connector.getLayer();
			} else {
				container = (FigureContainer) component;
			}
			
			// Try getting a site from the figure.
			// Require that it have an interactor
			Iterator i = container.figuresFromFront();
			while (i.hasNext()) {
				Figure t = (Figure) i.next();
				
				// The sequence here is a bit tricky. First we filter
				// out ConnectorManipulators. Then, if the figure is
				// a FigureDecorator, get the decorated figure instead.
				// Then filter out figures that are not hit by the mouse.
				if (t instanceof ConnectorManipulator) {
					continue;
				}
				if (t instanceof FigureDecorator) {
					t = ((FigureDecorator) t).getDecoratedFigure();
				}
				if ( !(t.hit(mouseRect)) ) {
					continue;
				}
				// Now try getting an interactor and a site
				if (t.getInteractor() != null) {
					Site snap = findSite(t, x, y);
					if (snap != null) {
						target = t;
						attach(snap);
						fireConnectorEvent(ConnectorEvent.CONNECTOR_SNAPPED);
						break;
					}
				}
			}
			// If the target if still null, we didn't get one, so
			// move the (autonomous) site, but signal a mouse drag.
			if (target == null) {
				handle.getSite().translate(dx, dy);
				connector.reroute();
				fireConnectorEvent(ConnectorEvent.CONNECTOR_DRAGGED);
			}
		}
	}
}

/* 
 * $Log: DebugConnectorInteractor.java,v $
 * Revision 1.1  2002/01/15 20:46:33  zcox
 * name changes & addition of some view-related classes
 *
 * 
 */
