/* $Id: EdgeFigure.java,v 1.5 2002/02/15 20:56:38 zcox Exp $
 *
 *
 */

package fcmodeler.view;

import diva.canvas.connector.Connector;
import diva.canvas.connector.ConnectorEnd;
import diva.canvas.toolbox.PaintedShape;

/** <p>A <CODE>Figure</CODE> representing an edge.  <CODE>EdgeFigure</CODE> paints some type of line
 * between two node figures.  Different concrete implementation classes can determine exactly
 * what type of line is used (straight, Bezier curve, etc.).</p>
 *
 * <p><CODE>EdgeFigure</CODE> is a <CODE>PermanentFigure</CODE>.
 * Therefore, it must provide accessor methods for both its permanent figure and the one used for
 * painting.  Additionally, <CODE>EdgeFigure</CODE> supports dynamic changing of arrowheads.</p>
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */

public interface EdgeFigure extends Connector, PermanentFigure {
	/** Returns the <CODE>PaintedShape</CODE> used for painting this edge figure.
	 *
	 * @return the <CODE>PaintedShape</CODE> used for painting this edge figure.
	 */	
	public PaintedShape getFigure();
	
	/** Returns the arrowhead used at the head end of this edge figure.
	 *
	 * @return the arrowhead used at the head end of this edge figure.
	 */	
	public ConnectorEnd getHeadEnd();
	
	/** Returns the <CODE>PaintedShape</CODE> used for permanent visual attribute storage in this edge figure.
	 *
	 * @return the <CODE>PaintedShape</CODE> used for permanent visual attribute storage in this edge figure.
	 */	
	public PaintedShape getPermanentFigure();
	
	/** Returns the arrowhead used at the tail end of this edge figure.
	 *
	 * @return the arrowhead used at the tail end of this edge figure.
	 */	
	public ConnectorEnd getTailEnd();
	
	/** Sets the arrowhead at the head end of this edge figure to the specified <CODE>ConnectorEnd</CODE>.
	 *
	 * @param end the new head end arrowhead for this edge figure.
	 */	
	public void setHeadEnd(ConnectorEnd end);
	
	/** Sets the arrowhead at the tail end of this edge figure to the specified <CODE>ConnectorEnd</CODE>.
	 *
	 * @param end the new tail end arrowhead for this edge figure.
	 */	
	public void setTailEnd(ConnectorEnd end);
	
}

/* 
 * $Log: EdgeFigure.java,v $
 * Revision 1.5  2002/02/15 20:56:38  zcox
 * big time update
 *
 * Revision 1.4  2002/01/23 21:23:20  zcox
 * added javadoc comments
 *
 * Revision 1.3  2002/01/22 22:59:30  zcox
 * added cvs tags
 *
 * Revision 1.2  2002/01/22 22:53:38  zcox
 * modifications to EdgeFigure classes
 *
 *
 *
 */
