/* $Id: NodeFigureRenderer.java,v 1.3 2002/02/15 20:21:18 zcox Exp $
 *
 *
 */

package fcmodeler.view;

import java.awt.Font;

import diva.canvas.Figure;
import diva.canvas.toolbox.LabelFigure;
import diva.canvas.toolbox.BasicFigure;
import diva.graph.NodeRenderer;
import diva.graph.model.Node;

import fcmodeler.view.figures.TinyRectangleNodeShape;

/** A factory used to create a <CODE>Figure</CODE> using a specified <CODE>Node</CODE>.
 * This implementation of the <CODE>NodeRenderer</CODE> interface creates instances
 * of <CODE>NodeFigure</CODE>.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public class NodeFigureRenderer implements NodeRenderer {
	
	/** Creates a new NodeFigureRenderer.
	 *
	 */
	public NodeFigureRenderer() {
		
	}
	
	/** Creates a <CODE>NodeFigure</CODE> to be used as a visual representation for the
	 * specified <CODE>Node</CODE>.
	 *
	 * @param node the node to render.
	 * @return a visual representation of the specified <CODE>Node</CODE>.
	 */	
	public Figure render(Node node) {
		NodeFigure nodeFigure = null;
		LabelFigure labelFigure = null;
		String label = (String) node.getProperty("label");
		
		if (label == null) {
			label = new String(" ");
		}
		
		//dummy nodes need to be very small
		if (label.equals("") || label.startsWith("dummy")) {
			labelFigure = new LabelFigure("d");
			labelFigure.setFont(new Font("Serif", Font.ITALIC, 1));
			nodeFigure = new NodeFigure(labelFigure);
			nodeFigure.setNodeShape(new TinyRectangleNodeShape());
			nodeFigure.setPermanentNodeShape(new TinyRectangleNodeShape());
			nodeFigure.setUserObject(node);
		}
		else {
			labelFigure = new LabelFigure(label);
			labelFigure.setFont(new Font("Serif", Font.ITALIC, 16));
			nodeFigure = new NodeFigure(labelFigure);
			nodeFigure.setUserObject(node);
		}
		
		return nodeFigure;
	}
}

/* 
 * $Log: NodeFigureRenderer.java,v $
 * Revision 1.3  2002/02/15 20:21:18  zcox
 * big time update
 *
 * Revision 1.2  2002/01/18 19:52:30  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.1  2002/01/15 20:46:33  zcox
 * name changes & addition of some view-related classes
 *
 * 
 */
