/* $Id: ScrollableJGraph.java,v 1.4 2002/05/21 20:19:49 zcox Exp $
 *
 *
 *
 * Created on September 21, 2000, 11:40 PM
 */

package fcmodeler.view;

import javax.swing.JScrollPane;
import javax.swing.Scrollable;
import javax.swing.JViewport;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.Point;

import diva.graph.JGraph;
import diva.graph.model.GraphModel;

import fcmodeler.FCModeler;
import fcmodeler.FCModelerUtilities;

/** This class extends diva.graph.JGraph, and the only change is that it implements the
 * javax.swing.Scrollable interface, to allow scrolling in a JScrollPane.
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public class ScrollableJGraph extends JGraph implements Scrollable {
	private JScrollPane _scrollPane = null;
	private double _zoomInFactor = 1.25d;
	private double _zoomOutFactor = 0.8d;
	private double _currentZoom = 1.0d;
	private AffineTransform _origTransform = null;
	private FCModeler _fcmodeler = null;

	/** Creates new FCMJGraph */
	public ScrollableJGraph(FCModeler fcmodeler) {
		super();
		_origTransform = getGraphPane().getTransformContext().getTransform();
		_fcmodeler = fcmodeler;
	}
	
	/** Creates new FCMJGraph using the specified GraphModel.
	 * @param m the GraphModel to be used in this JGraph.
	 */
	public ScrollableJGraph(FCModeler fcmodeler, GraphModel m) {
		super(m);
		_origTransform = getGraphPane().getTransformContext().getTransform();
		_fcmodeler = fcmodeler;
	}
	
	public ScrollableJGraph(FCModeler fcmodeler, GraphModel m, JScrollPane scrollPane) {
		this(fcmodeler, m);
		setScrollPane(scrollPane);
	}
	
	public double getCurrentZoom() {
		return _currentZoom;
	}
	
	public FCModeler getFCModeler() {
		return _fcmodeler;
	}

	/** see the javax.swing.Scrollable interface description.
	 * @return
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}
	
	/** see the javax.swing.Scrollable interface description.
	 * @param p1
	 * @param p2
	 * @param p3
	 * @return
	 */
	public int getScrollableBlockIncrement(final Rectangle p1, int p2, int p3) {
		int r = 1;
		return r;
	}
	
	/** see the javax.swing.Scrollable interface description.
	 * @return
	 */
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
	
	/** see the javax.swing.Scrollable interface description.
	 * @return
	 */
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}
	
	/** see the javax.swing.Scrollable interface description.
	 * @param p1
	 * @param p2
	 * @param p3
	 * @return
	 */
	public int getScrollableUnitIncrement(final Rectangle p1, int p2, int p3) {
		int r = 1;
		return r;
	}
	
	public JScrollPane getScrollPane() {
		return _scrollPane;
	}
	
	public void setPreferredSize(Dimension d) {
		double width = d.width * _currentZoom;
		double height = d.height * _currentZoom;
		
		d.setSize(width, height);
		super.setPreferredSize(d);
	}
	
	public void setScrollPane(JScrollPane scrollPane) {
		_scrollPane = scrollPane;
	}
	
	public void setSize(int width, int height) {
		width *= _currentZoom;
		height *= _currentZoom;
		super.setSize(width, height);
	}
	
	public void setSize(Dimension d) {
		double width = d.width * _currentZoom;
		double height = d.height * _currentZoom;
		
		d.setSize(width, height);
		super.setSize(d);
	}
	
	public void zoom(double factor) {
		Dimension d = null, newd = null;
		double temp = 0.0d;
		Point2D.Double center = new Point2D.Double();
		Point intCenter = null;
		JViewport viewport = _scrollPane.getViewport();
		Rectangle viewRect = viewport.getViewRect();
		
		//save center point of view before zooming
		center.x = viewRect.getCenterX();
		center.y = viewRect.getCenterY();
		
		//perform zooming
		_currentZoom *= factor;
		getGraphPane().getTransformContext().concatenate(AffineTransform.getScaleInstance(factor, factor));
		
		//adjust the size of the jgraph component (this)
		temp = _currentZoom;
		_currentZoom = factor;
		setPreferredSize(getPreferredSize());
		setSize(getSize());
		_currentZoom = temp;
		
		//repaint the graph
		getGraphPane().repaint();
		
		//make the scrollpane re-adjust it's scroll bar sizes
		_scrollPane.revalidate();
		
		//transform the previous center & position the viewport's center view position to it
		center.x *= factor;
		center.y *= factor;
		intCenter = new Point((int) Math.round(center.x - 0.5*viewRect.getWidth()), 
			(int) Math.round(center.y - 0.5*viewRect.getHeight()));
		FCModelerUtilities.positionViewport(viewport, intCenter);
	}
	
	public void zoomIn() {
		zoom(_zoomInFactor);
	}
	
	public void zoomOut() {
		zoom(_zoomOutFactor);
	}
}

/* 
 * $Log: ScrollableJGraph.java,v $
 * Revision 1.4  2002/05/21 20:19:49  zcox
 * more som work
 *
 * Revision 1.3  2002/01/15 20:45:19  zcox
 * some changes
 *
 * Revision 1.2  2001/05/22 16:43:43  zcox
 * code clean-up
 *
 * Revision 1.1  2001/04/04 01:07:44  zcox
 * initial commit of ScrollableJGraph (used to be FCMJGraph)
 *
 * Revision 1.1  2001/02/22 00:21:12  zcox
 * initial commit of fcmodeler.view package
 *
 *
 *
 */
