/* $Id: TestNodeShape.java,v 1.5 2002/02/15 20:21:18 zcox Exp $
 *
 *
 * Created on October 24, 2000, 8:40 PM
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;

/** A simple example implementation of <CODE>NodeShape</CODE> that does not use one of the <CODE>RectangularShape</CODE>
 * subclasses in the <CODE>java.awt.geom</CODE> package.  Instead, <CODE>TestNodeShape</CODE>
 * uses a <CODE>GeneralPath</CODE> to define the geometry of a simple custom shape.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */

public class TestNodeShape extends Object implements NodeShape {
	private GeneralPath _shape;
	private float _space = 10.0f;
	private static final String _string = "test";

	/** Creates new TestShape */
	public TestNodeShape() {
		_shape = new GeneralPath();
		_shape.moveTo(0, 0);
		_shape.lineTo(2, 0);
		_shape.lineTo(2, 2);
		_shape.quadTo(1, 3, 0, 2);
		_shape.closePath();
	}
	
	/** Creates and returns a copy of this <CODE>TestNodeShape</CODE>.  The returned
	 * <CODE>Object</CODE> is guaranteed to be of type <CODE>TestNodeShape</CODE>
	 * and is also guaranteed to have copies of any mutable fields of this object.
	 *
	 * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
	 * <CODE>Cloneable</CODE> by implementing <CODE>NodeShape</CODE>.
	 * @return a clone of this <CODE>TestNodeShape</CODE>.
	 */	
	public Object clone() throws CloneNotSupportedException {
		//create the clone
		TestNodeShape clone = (TestNodeShape) super.clone();
		
		//create new copies of any mutable fields
		clone._shape = new GeneralPath(_shape);
		
		return clone;
	}
	
	/** Ensures that this <CODE>TestNodeShape</CODE> completely encloses the specified
	 * rectangle.
	 *
	 * @param rectangle the rectangle to enclose.
	 */
	public void enclose(Rectangle2D rectangle) {
		float x = 0.0f, y = 0.0f, w = 0.0f, h = 0.0f;
		
		x = (float) rectangle.getX();
		y = (float) rectangle.getY(); 
		w = (float) rectangle.getWidth();
		h = (float) rectangle.getHeight();
		
		_shape.reset();
		_shape.moveTo(x-_space, y-_space);
		_shape.lineTo(x+w+_space, y-_space);
		_shape.lineTo(x+w+_space, y+h+_space);
		_shape.quadTo(x+(w+_space)/2, y+3*h, x-_space, y+h+_space);
		_shape.closePath();
	}
	
	/** Returns the <CODE>Shape</CODE> object represented by this <CODE>TestNodeShape</CODE>.
	 *
	 * @return the <CODE>Shape</CODE> object represented by this <CODE>TestNodeShape</CODE>.
	 */
	public Shape getShape() {
		return _shape;
	}
	
	/** Positions this <CODE>TestNodeShape</CODE> at the specified coordinates.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param x the x-coordinate of the new position.
	 * @param y the y-coordinate of the new position.
	 */
	public void position(double x, double y) {
		Rectangle2D bounds = _shape.getBounds2D();
		translate(x - bounds.getX(), y - bounds.getY());
	}
	
	public String toString() {
		return _string;
	}
	
	/** Transforms this <CODE>TestNodeShape</CODE> using the specified <CODE>AffineTransform</CODE>.
	 * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
	 * a new call to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param transform the <CODE>AffineTransform</CODE> used to transform this <CODE>TestNodeShape</CODE> object.
	 */
	public void transform(AffineTransform transform) {
		_shape.transform(transform);
	}
	
	/** Translates this <CODE>TestNodeShape</CODE> by the specified x and y values.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param dx the amount to translate the x direction.
	 * @param dy the amount to translate in the y direction.
	 */
	public void translate(double dx, double dy) {
		_shape.transform(AffineTransform.getTranslateInstance(dx, dy));
	}
	
}

/* 
 * $Log: TestNodeShape.java,v $
 * Revision 1.5  2002/02/15 20:21:18  zcox
 * big time update
 *
 * Revision 1.4  2002/01/19 00:39:25  zcox
 * changed class doc
 *
 * Revision 1.3  2002/01/18 19:52:52  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.2  2002/01/15 20:40:06  zcox
 * some changes
 *
 * Revision 1.1  2001/05/22 16:45:01  zcox
 * code clean-up
 *
 * Revision 1.3  2001/04/05 03:01:29  zcox
 * made FCMNodeShape a real flyweight
 *
 * Revision 1.2  2001/04/04 01:33:56  zcox
 * huge architectural changes
 *
 *
 *
 */
