/* $Id: TinyRectangleNodeShape.java,v 1.4 2002/02/15 20:21:18 zcox Exp $
 *
 *
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

/**
 * ... is ...
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public class TinyRectangleNodeShape extends Object implements NodeShape {
	private Rectangle2D.Double _shape = null;
	private double _space = 1.0d;
	private static final String _string = "tiny rectangle";

	/** Creates a new TinyRectangleNodeShape with coordinates (0,0), zero width, and zero height.
	 *
	 */
	public TinyRectangleNodeShape() {
		_shape = new Rectangle2D.Double();
	}
	
	/** Creates a new TinyRectangleNodeShape with the specified coordinates, width, and height.
	 *
	 * @param x the x-coordinate of this <CODE>TinyRectangleNodeShape</CODE>.
	 * @param y the y-coordinate of this <CODE>TinyRectangleNodeShape</CODE>.
	 * @param w the width of this <CODE>TinyRectangleNodeShape</CODE>.
	 * @param h the height of this <CODE>TinyRectangleNodeShape</CODE>.
	 */	
	public TinyRectangleNodeShape(double x, double y, double w, double h) {
		_shape = new Rectangle2D.Double(x, y, w, h);
	}
	
	/** Creates and returns a copy of this <CODE>TinyRectangleNodeShape</CODE>.  The returned
	 * <CODE>Object</CODE> is guaranteed to be of type <CODE>TinyRectangleNodeShape</CODE>
	 * and is also guaranteed to have copies of any mutable fields of this object.
	 *
	 * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
	 * <CODE>Cloneable</CODE> by implementing <CODE>NodeShape</CODE>.
	 * @return a clone of this <CODE>TinyRectangleNodeShape</CODE>.
	 */	
	public Object clone() throws CloneNotSupportedException {
		//create the clone
		TinyRectangleNodeShape clone = (TinyRectangleNodeShape) super.clone();
		
		//create new copies of any mutable fields
		clone._shape = new Rectangle2D.Double(_shape.getX(), _shape.getY(), _shape.getWidth(), _shape.getHeight());
		
		return clone;
	}
	
	/** Ensures that this <CODE>TinyRectangleNodeShape</CODE> completely encloses the specified
	 * rectangle.
	 *
	 * @param rectangle the rectangle to enclose.
	 */
	public void enclose(Rectangle2D rectangle) {
		_shape.setRect(rectangle.getX()-_space, rectangle.getY()-_space, rectangle.getWidth()+2*_space, rectangle.getHeight()+2*_space);
	}
	
	/** Returns the <CODE>Shape</CODE> object represented by this <CODE>TinyRectangleNodeShape</CODE>.
	 *
	 * @return the <CODE>Shape</CODE> object represented by this <CODE>TinyRectangleNodeShape</CODE>.
	 */	
	public Shape getShape() {
		return _shape;
	}
	
	/** Positions this <CODE>TinyRectangleNodeShape</CODE> at the specified coordinates.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param x the x-coordinate of the new position.
	 * @param y the y-coordinate of the new position.
	 */	
	public void position(double x, double y) {
		_shape.setRect(x, y, _shape.getWidth(), _shape.getHeight());
	}
	
	public String toString() {
		return _string;
	}
	
	/** <p>Transforms this <CODE>TinyRectangleNodeShape</CODE> using the specified <CODE>AffineTransform</CODE>.
	 * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
	 * a new call to <CODE>getShape</CODE> may be necessary after calling this method.</p>
	 *
	 * <p>This method will throw an <CODE>UnsupportedOperationException</CODE>
	 * until a way to modify a Rectangle2D.Double based on the specified
	 * <CODE>AffineTransform</CODE> is found.</p>
	 *
	 * @param transform the <CODE>AffineTransform</CODE> used to transform this <CODE>TinyRectangleNodeShape</CODE> object.
	 */	
	public void transform(AffineTransform transform) {
		//can't do this because the call won't return a Rectangle2D.Double
		//if transforms are going to be used, need to find a way to adjust the
		//Rectangle2D.Double appropriately
		//_shape = transform.createTransformedShape(_shape);
		
		throw new UnsupportedOperationException("transform not supported yet");
	}
	
	/** Translates this <CODE>TinyRectangleNodeShape</CODE> by the specified x and y values.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param dx the amount to translate the x direction.
	 * @param dy the amount to translate in the y direction.
	 */	
	public void translate(double dx, double dy) {
		_shape.setRect(_shape.getX()+dx, _shape.getY()+dy, _shape.getWidth(), _shape.getHeight());
	}
}

/* 
 * $Log: TinyRectangleNodeShape.java,v $
 * Revision 1.4  2002/02/15 20:21:18  zcox
 * big time update
 *
 * Revision 1.3  2002/01/18 19:52:52  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.2  2002/01/15 20:40:06  zcox
 * some changes
 *
 *
 *
 */
